SUBROUTINE allocate_bom
!
! allocation of variables. all arrays that are used throughout the
! model run are allocated, not necessarily here, but certainly from
! here, with exception for concentration, nonhydrostatic and s,t or
! rho, which are all allocated if needed, in order to save memory.
!  
  USE CONTROL_PARAMETERS
  USE MOD_MODESPLIT
  USE MOD_BOUND
  USE STATE
  USE BOMPLOT
  USE MOD_CONCENTRATION
  USE TIMESERIES, ONLY : ALLOCATE_TIMESERIES
  
  IMPLICIT NONE
  real dummy

  if(bom_allocated) then
      call abort("allocate_bom already called. this should never happen!")
  end if
  
  if(debug>0 .and. master)then
      write(*,'(/,2x,a)') 'ALLOCATE_BOM:'
      write(*,'(2x,a,i0,a,i0,a,i0)') 'Mesh dimensions IM=',im,' JM=',jm,' KB=',kb
      write(*,'(2x,a,f9.2,a)') 'a global 2d field =        ',im*jm*kind(dummy)/1000000.,'MB' 
      write(*,'(2x,a,f9.2,a)') 'a global 3d field =        ',im*jm*kb*kind(dummy)/1000000.,'MB'
      write(*,'(2x,a,f9.2,a)') 'BOM in total approximately ',70.*im*jm*kb*1e-6*kind(dummy),'MB'
      write(*,'(2x,a,f9.2,a)') '  per processor            ',70.*im*jm*kb*1e-6*kind(dummy)/(numbli*numblj),'MB'
      write(*,'(2x,a)') "(Note that aggressive optimization can lead to significantly higher memory requirements in practice)"
  end if
  
!from state:
  
! Z,DZ,DZR,DZZ,ZZ

  ALLOCATE( Z(KB) )
  ALLOCATE( DZ(KB) )
  ALLOCATE( DZR(KB) )
  ALLOCATE( DZZ(KB) )
  ALLOCATE( ZZ(KB) )

  z=adiv ; dz=adiv ; dzr=adiv; dzz=adiv ; zz=adiv

  
! H is allocated in setup2.f90  
  ALLOCATE( COR(i1:i2,j1:j2) )
  ALLOCATE( FSM(i1:i2,j1:j2) )
  ALLOCATE( DUM(i1:i2,j1:j2) )
  ALLOCATE( DVM(i1:i2,j1:j2) )
  ALLOCATE( ETA(i1:i2,j1:j2) )
  ALLOCATE( ETAP(i1:i2,j1:j2) )
  ALLOCATE( HU(i1:i2,j1:j2) )
  ALLOCATE( HV(i1:i2,j1:j2) )
  ALLOCATE( D(i1:i2,j1:j2) )
  ALLOCATE( DU(i1:i2,j1:j2) )
  ALLOCATE( DV(i1:i2,j1:j2) )
  ALLOCATE( WUSURF(i1:i2,j1:j2) )
  ALLOCATE( WVSURF(i1:i2,j1:j2) )
  ALLOCATE( WUBOT(i1:i2,j1:j2) )
  ALLOCATE( WVBOT(i1:i2,j1:j2) )
  ALLOCATE( CBC(i1:i2,j1:j2) )
  ALLOCATE( COAST(i1:i2,j1:j2))

  cor=adiv ; fsm=adiv ; dum=adiv ; dvm=adiv ; eta=adiv ; etap=adiv ; hu=adiv ; hv=adiv ; d=adiv ; du=adiv ; dv=adiv ;
  wusurf=adiv ; wvsurf=adiv ; wubot=adiv ; wvbot=adiv ; cbc=-1 ; coast=adiv
    
  ALLOCATE( PATM(i1:i2,j1:j2) )
  ALLOCATE( AM2D(i1:i2,j1:j2) )
  ALLOCATE( UA(i1:i2,j1:j2) )
  ALLOCATE( VA(i1:i2,j1:j2) )
  ALLOCATE( HUSQR(i1:i2,j1:j2) )
  ALLOCATE( HVSQR(i1:i2,j1:j2) )

  patm=adiv ; am2d=adiv ; ua=adiv ; va=adiv ; husqr=adiv ; hvsqr=adiv
  
! 3D
  ALLOCATE( U(I1:I2,J1:J2,KB) )
  ALLOCATE( V(I1:I2,J1:J2,KB) )
  ALLOCATE( W(I1:I2,J1:J2,KB) )
  ALLOCATE( WR(I1:I2,J1:J2,KB) )
  ALLOCATE( RHO(I1:I2,J1:J2,KB) )

  u=adiv ; v=adiv ; w=adiv ; wr=adiv ; rho=adiv


  if(advect_density)then
      call allocate_density()
  else
      call allocate_st()
  end if

  if(nonhyd) call allocate_nonhyd()
  
  ALLOCATE( AM(I1:I2,J1:J2,KB) )
  ALLOCATE( AH(I1:I2,J1:J2,KB) )
  ALLOCATE( KM(I1:I2,J1:J2,KB) )
  ALLOCATE( KH(I1:I2,J1:J2,KB) )
  ALLOCATE( KQ(I1:I2,J1:J2,KB) )
  ALLOCATE( DDZ(I1:I2,J1:J2,KB) )
  ALLOCATE( DUDZ(I1:I2,J1:J2,KB) )
  ALLOCATE( DVDZ(I1:I2,J1:J2,KB) )
  ALLOCATE( UADV(I1:I2,J1:J2,KB) )
  ALLOCATE( VADV(I1:I2,J1:J2,KB) )
  ALLOCATE( WADV(I1:I2,J1:J2,KB) )  
  ALLOCATE( Q2(I1:I2,J1:J2,KB) )
  ALLOCATE( Q2L(I1:I2,J1:J2,KB) )
  ALLOCATE( DELTAU(I1:I2,J1:J2,KB) )
  ALLOCATE( DELTAV(I1:I2,J1:J2,KB) )

  am=adiv ;ah=adiv ; km=adiv ; kh=adiv ; kq=adiv ; ddz=adiv ; dudz=adiv ; dvdz=adiv ; uadv=adiv ; vadv=adiv ;
  q2=adiv ; q2l =adiv ; deltau=adiv ; deltav=adiv
    
  ALLOCATE( UN(I1:I2,J1:J2,KB),UNM1(I1:I2,J1:J2,KB),DELUN(I1:I2,J1:J2,KB),DELUSTAR(I1:I2,J1:J2,KB),DELUNM1(I1:I2,J1:J2,KB) )
  ALLOCATE( VN(I1:I2,J1:J2,KB),VNM1(I1:I2,J1:J2,KB),DELVN(I1:I2,J1:J2,KB),DELVSTAR(I1:I2,J1:J2,KB),DELVNM1(I1:I2,J1:J2,KB) )
  ALLOCATE( WN(I1:I2,J1:J2,KB),WNM1(I1:I2,J1:J2,KB),DELWN(I1:I2,J1:J2,KB),DELWSTAR(I1:I2,J1:J2,KB),DELWNM1(I1:I2,J1:J2,KB) )

  un=adiv ; unm1=adiv ; delun=adiv ; delustar=adiv ; delunm1=adiv ; 
  vn=adiv ; vnm1=adiv ; delvn=adiv ; delvstar=adiv ; delvnm1=adiv ; 
  wn=adiv ; wnm1=adiv ; delwn=adiv ; delwstar=adiv ; delwnm1=adiv ; 

  
!From mode_modesplit

  ALLOCATE(UV(I1:I2,J1:J2,KB),VU(I1:I2,J1:J2,KB),UBCLIN(I1:I2,J1:J2,KB),VBCLIN(I1:I2,J1:J2,KB),&
       C3U(I1:I2,J1:J2,KB),C3V(I1:I2,J1:J2,KB))
  ALLOCATE(UCORR(I1:I2,J1:J2,KB),VCORR(I1:I2,J1:J2,KB),WCORR(I1:I2,J1:J2,KB))
  ALLOCATE(AX(i1:i2,j1:j2),AY(i1:i2,j1:j2),UV2D(i1:i2,j1:j2),VU2D(i1:i2,j1:j2),UASUM(i1:i2,j1:j2),VASUM(i1:i2,j1:j2))  
  ALLOCATE(CORU(i1:i2,j1:j2),CORV(i1:i2,j1:j2),C2U(i1:i2,j1:j2),C2V(i1:i2,j1:j2))
  ALLOCATE(UAD(i1:i2,j1:j2),VAD(i1:i2,j1:j2),UASTAR(i1:i2,j1:j2),VASTAR(i1:i2,j1:j2),ETASTAR(i1:i2,j1:j2))
  ALLOCATE(UAF(i1:i2,j1:j2),UAB(i1:i2,j1:j2),VAF(i1:i2,j1:j2),VAB(i1:i2,j1:j2),ETAB(i1:i2,j1:j2),ETAF(i1:i2,j1:j2))

  uv=adiv ; vu=adiv ; ubclin=adiv ; vbclin=adiv ; c3u=adiv ; c3v=adiv ; 
  ucorr=adiv ; vcorr=adiv ; wcorr=adiv ; 
  ax=adiv ; ay=adiv ; uv2d=adiv ; vu2d=adiv ; uasum=adiv ; vasum=adiv ;   
  coru=adiv ; corv=adiv ; c2u=adiv ; c2v=adiv ; 
  uad=adiv ; vad=adiv ; uastar=adiv ; vastar=adiv ; etastar=adiv ; 
  uaf=adiv ; uab=adiv ; vaf=adiv ; vab=adiv ; etab=adiv ; etaf=adiv ; 

!FROM MOD_BOUND:

  ALLOCATE( ALPHA(MAXLB),ALPHAE(MAXLB) )
  ALLOCATE( ALFA_U(i1:i2,j1:j2), ALFA_V(i1:i2,j1:j2), ALFA_ETA(i1:i2,j1:j2), &
       ALFAE_U(i1:i2,j1:j2), ALFAE_V(i1:i2,j1:j2), ALFAE_ETA(i1:i2,j1:j2))
  
  alpha=adiv ; alphae=adiv
  alfa_u=adiv ;  alfa_v=adiv ;  alfa_eta=adiv ;  alfae_u=adiv ;  alfae_v=adiv ;  alfae_eta=adiv  

!! the arrays containing boundary values ready to be applied.  

  ALLOCATE(U_BOUND(I1:I2,J1:J2,KB), V_BOUND(I1:I2,J1:J2,KB), W_BOUND(I1:I2,J1:J2,KB), &
       Q2_BOUND(I1:I2,J1:J2,KB), Q2L_BOUND(I1:I2,J1:J2,KB))
  ALLOCATE(UA_BOUND(i1:i2,j1:j2), VA_BOUND(i1:i2,j1:j2), ETA_BOUND(i1:i2,j1:j2), ETABFRS(i1:i2,j1:j2), etatfrs(i1:i2,j1:j2))
  ALLOCATE(UA_TIDE(i1:i2,j1:j2), VA_TIDE(i1:i2,j1:j2), ETA_TIDE(i1:i2,j1:j2))

  u_bound=adiv ;  v_bound=adiv ;  w_bound=adiv ;  q2_bound=adiv ;  q2l_bound=adiv ; 
  ua_bound=adiv ;  va_bound=adiv ;  eta_bound=adiv ;  etabfrs=adiv ;  etatfrs=adiv ; 
  ua_tide=adiv ;  va_tide=adiv ;  eta_tide=adiv ; 
  

! rivers
  
  ALLOCATE( IRIVER(MAXNUMC), JRIVER(MAXNUMC), KRIVER1(MAXNUMC), KRIVER2(MAXNUMC))
  ALLOCATE( CODERIVER(MAXNUMC) , RIVERFORMAT(MAXNUMC))
  ALLOCATE( SRIVER(MAXNUMC), TRIVER(MAXNUMC), FRIVER(MAXNUMC), DZRIVER(MAXNUMC),CRIVER(MAXNUMCONC,MAXNUMC))
  ALLOCATE( RIVER_NAME(MAXNUMC), DEPRIVER(MAXNUMC))
   
!
! Variables used in the case of fresh water inputs through the bottom, like
! with a dived fresh water source.
!
  ALLOCATE(IWELL(MAXNUMWELL),JWELL(MAXNUMWELL),WELL_FORMAT(MAXNUMWELL))
  ALLOCATE(SWELL(MAXNUMWELL),TWELL(MAXNUMWELL),FWELL(MAXNUMWELL),CWELL(MAXNUMCONC,MAXNUMWELL))
  ALLOCATE(WELL_NAME(MAXNUMWELL))
  
  
! tidal variables
!  
  ALLOCATE( FREQ(MAXKCON) )
  ALLOCATE( ETATAMP(I1:I2,J1:J2,MAXKCON) )
  ALLOCATE( ETATPHA(I1:I2,J1:J2,MAXKCON) )
  ALLOCATE( UATAMP(I1:I2,J1:J2,MAXKCON) )
  ALLOCATE( UATPHA(I1:I2,J1:J2,MAXKCON) )
  ALLOCATE( VATAMP(I1:I2,J1:J2,MAXKCON) )
  ALLOCATE( VATPHA(I1:I2,J1:J2,MAXKCON) )

  freq=adiv ; etatamp=adiv ; etatpha=adiv ; uatamp=adiv ; uatpha=adiv ; vatamp=adiv ; vatpha=adiv
  
!
!  Atmospheric forcing variables
!
   
!   LOGICAL WIND,ATMPRESS
  ALLOCATE( WSPEEDU(i1:i2,j1:j2) )
  ALLOCATE( WSPEEDV(i1:i2,j1:j2) )

  wspeedu=adiv ; wspeedv=adiv
  
  CALL ALLOCATE_BOMPLOT()
  CALL ALLOCATE_TIMESERIES()
  
  BOM_ALLOCATED = .TRUE.

  CALL BARRIER()
  IF(MASTER) WRITE(*,*) " Subroutine ALLOCATE_BOM done"
END SUBROUTINE allocate_bom


subroutine check_allocation()
  use state, only : bom_allocated, message
  if(.not.bom_allocated) then
      call message('Arrays not yet allocated. Make sure IM,JM,KB and H are defined before this point', severity=2)
  end if
end subroutine check_allocation




SUBROUTINE ALLOCATE_ST
  USE STATE
  USE MOD_BOUND
  USE CONTROL_PARAMETERS
  IMPLICIT NONE

  IF(DEBUG>1 .AND. MASTER) WRITE(*,'(2x,a)')'allocating the S and T array family'
  
  ALLOCATE(SSURF(I1:I2,J1:J2) )
  ALLOCATE(TSURF(I1:I2,J1:J2) )
  ALLOCATE(S(I1:I2,J1:J2,KB) )
  ALLOCATE(T(I1:I2,J1:J2,KB) )
  ALLOCATE(SN(I1:I2,J1:J2,KB),SNM1(I1:I2,J1:J2,KB),DELSN(I1:I2,J1:J2,KB),DELSSTAR(I1:I2,J1:J2,KB),DELSNM1(I1:I2,J1:J2,KB))
  ALLOCATE(TN(I1:I2,J1:J2,KB),TNM1(I1:I2,J1:J2,KB),DELTN(I1:I2,J1:J2,KB),DELTSTAR(I1:I2,J1:J2,KB),DELTNM1(I1:I2,J1:J2,KB))
  ALLOCATE(S_BOUND(I1:I2,J1:J2,KB), T_BOUND(I1:I2,J1:J2,KB))
  
  S=adiv
  T=adiv
  SSURF=adiv
  TSURF=adiv
  S_BOUND =adiv
  T_BOUND =adiv

  ST_ALLOCATED=.TRUE.
END SUBROUTINE ALLOCATE_ST

SUBROUTINE DEALLOCATE_ST
  USE STATE  
  USE MOD_BOUND
  USE CONTROL_PARAMETERS
  IMPLICIT NONE

  DEALLOCATE(SSURF,TSURF)
  DEALLOCATE(S,T)
  DEALLOCATE(SN,SNM1,DELSN,DELSSTAR,DELSNM1 )
  DEALLOCATE(TN,TNM1,DELTN,DELTSTAR,DELTNM1 )
  DEALLOCATE(S_BOUND, T_BOUND)

  IF(BINREG_SET) DEALLOCATE(S_INITIAL, T_INITIAL)
  
  ST_ALLOCATED=.FALSE.
END SUBROUTINE DEALLOCATE_ST

SUBROUTINE ALLOCATE_DENSITY
  USE STATE
  USE MOD_BOUND
  USE CONTROL_PARAMETERS
  IMPLICIT NONE

  ALLOCATE(RHOSURF(i1:i2,j1:j2))
  ALLOCATE(RHON(I1:I2,J1:J2,KB),RHONM1(I1:I2,J1:J2,KB),DELRHON(I1:I2,J1:J2,KB),&
       DELRHOSTAR(I1:I2,J1:J2,KB),DELRHONM1(I1:I2,J1:J2,KB) )
  ALLOCATE(RHO_BOUND(I1:I2,J1:J2,KB))
  DENSITY_ALLOCATED=.TRUE.
END SUBROUTINE ALLOCATE_DENSITY

SUBROUTINE DEALLOCATE_DENSITY
  USE STATE
  USE MOD_BOUND
  USE CONTROL_PARAMETERS
  IMPLICIT NONE
  
  DEALLOCATE(RHOSURF)
  DEALLOCATE(RHON,RHONM1,DELRHON,DELRHOSTAR,DELRHONM1 )
  DEALLOCATE(RHO_BOUND)
  DENSITY_ALLOCATED=.FALSE.
END SUBROUTINE DEALLOCATE_DENSITY

SUBROUTINE ALLOCATE_BALANCED
  USE STATE
  USE CONTROL_PARAMETERS
  IMPLICIT NONE

  IF(.NOT.ASSOCIATED(RHOBACK)) THEN
      ALLOCATE(RHOBACK(I1:I2,J1:J2,KB), SOPTU(I1:I2,J1:J2,KB), SOPTV(I1:I2,J1:J2,KB), &
           MASKSOPTU(I1:I2,J1:J2,KB), MASKSOPTV(I1:I2,J1:J2,KB))
  END IF
END SUBROUTINE ALLOCATE_BALANCED

SUBROUTINE DEALLOCATE_BALANCED
  USE STATE
  USE CONTROL_PARAMETERS
  IMPLICIT NONE

  IF(ASSOCIATED(RHOBACK)) THEN
      DEALLOCATE(RHOBACK, SOPTU, SOPTV, MASKSOPTU, MASKSOPTV)
  END IF
END SUBROUTINE DEALLOCATE_BALANCED


SUBROUTINE ALLOCATE_NONHYD
  USE STATE
  USE CONTROL_PARAMETERS
  IMPLICIT NONE

  IF(.NOT.ASSOCIATED(P)) ALLOCATE(P(I1:I2,J1:J2,KB))
  P = ADIV
  NONHYD_ALLOCATED=.TRUE.
END SUBROUTINE ALLOCATE_NONHYD

SUBROUTINE DEALLOCATE_NONHYD
  USE STATE
  USE CONTROL_PARAMETERS
  IMPLICIT NONE
  
  IF(ASSOCIATED(P))DEALLOCATE(P)
  NONHYD_ALLOCATED=.FALSE.
END SUBROUTINE DEALLOCATE_NONHYD

SUBROUTINE ALLOCATE_BINREG
  USE STATE
  USE MOD_BOUND
  USE CONTROL_PARAMETERS
  IMPLICIT NONE

  IF(.NOT.ADVECT_DENSITY.AND..NOT.ASSOCIATED(S_INITIAL).AND..NOT.ASSOCIATED(T_INITIAL))THEN
      ALLOCATE(S_INITIAL(I1:I2,J1:J2,KB), T_INITIAL(I1:I2,J1:J2,KB))
  ELSE
      IF(.NOT.ASSOCIATED(RHO_INITIAL))ALLOCATE(RHO_INITIAL(I1:I2,J1:J2,KB))
  END IF
  BINREG_ALLOCATED=.TRUE.
END SUBROUTINE ALLOCATE_BINREG


SUBROUTINE ALLOCATE_PGPLOT()
  USE STATE, ONLY : NUMSTEP, DT, IM, JM, KB
  USE MOD_MPI, ONLY : MASTER
  USE PGPLOT_CORE
    
  IMPLICIT NONE
  INTEGER I

  IF(PGPLOT_ALLOCATED) RETURN
    
  IF(PGPLOT_MAXSEC < 0) PGPLOT_MAXSEC = IM+JM

  ALLOCATE(PG_VX(PGPLOT_MAXSEC,KB), PG_VZ(PGPLOT_MAXSEC,KB), &
       PG_USEC(PGPLOT_MAXSEC,KB), PG_UTSEC(PGPLOT_MAXSEC,KB), &
       PG_WSEC(PGPLOT_MAXSEC,KB), PG_SSEC(PGPLOT_MAXSEC,KB), &
       PG_TSEC(PGPLOT_MAXSEC,KB))

  IF(MASTER) THEN

! initialize time in hours for trends    
      ALLOCATE(TIMEX(0:NUMSTEP))        
      DO I=0,NUMSTEP
          TIMEX(I) = DT*I/3600.
      END DO

      ALLOCATE(PG_U(IM,JM), PG_V(IM,JM), PG_F(IM,JM), PGMASK(IM,JM))
  ELSE
      ALLOCATE(PG_U(1,1), PG_V(1,1), PG_F(1,1), PGMASK(1,1), TIMEX(1))
  END IF
  PGPLOT_ALLOCATED = .TRUE.
END SUBROUTINE ALLOCATE_PGPLOT
  
SUBROUTINE DEALLOCATE_PGPLOT()
  USE PGPLOT_CORE
  IMPLICIT NONE
  INTEGER i
  IF(ALLOCATED(PG_VX)) DEALLOCATE(PG_VX)
  IF(ALLOCATED(PG_VZ)) DEALLOCATE(PG_VZ)
  IF(ALLOCATED(PG_USEC)) DEALLOCATE(PG_USEC)
  IF(ALLOCATED(PG_UTSEC)) DEALLOCATE(PG_UTSEC)
  IF(ALLOCATED(PG_WSEC)) DEALLOCATE(PG_WSEC)
  IF(ALLOCATED(PG_SSEC)) DEALLOCATE(PG_SSEC)
  IF(ALLOCATED(PG_TSEC)) DEALLOCATE(PG_TSEC)
  IF(ALLOCATED(PG_U)) DEALLOCATE(PG_U)
  IF(ALLOCATED(PG_V)) DEALLOCATE(PG_V)
  IF(ALLOCATED(PG_F)) DEALLOCATE(PG_F)
  IF(ALLOCATED(PGMASK)) DEALLOCATE(PGMASK)
  IF(ALLOCATED(TIMEX)) DEALLOCATE(TIMEX)
  DO I=1,SIZE(PGPLOTS)
      IF(ALLOCATED(PGPLOTS(I)%DATA1D)) DEALLOCATE(PGPLOTS(I)%DATA1D)
      IF(ALLOCATED(PGPLOTS(I)%CACHE1D)) DEALLOCATE(PGPLOTS(I)%CACHE1D)
  END DO
  PGPLOT_ALLOCATED = .FALSE.
END SUBROUTINE DEALLOCATE_PGPLOT

SUBROUTINE DEALLOCATE_BINREG
  USE MOD_BOUND
  USE CONTROL_PARAMETERS
  IMPLICIT NONE
  
  IF(ASSOCIATED(S_INITIAL))DEALLOCATE(S_INITIAL)
  IF(ASSOCIATED(T_INITIAL))DEALLOCATE(T_INITIAL)
  IF(ASSOCIATED(RHO_INITIAL))DEALLOCATE(RHO_INITIAL)
  BINREG_ALLOCATED=.FALSE.
END SUBROUTINE DEALLOCATE_BINREG

SUBROUTINE DEALLOCATE_BOM
  USE CONTROL_PARAMETERS
  USE MOD_MODESPLIT
  USE MOD_BOUND
  USE STATE
  USE BOMPLOT
  USE PGPLOT_CORE  
  USE MOD_CONCENTRATION
  USE MOD_RAMPUP
  USE TIMESERIES
  
  IMPLICIT NONE
  
  DEALLOCATE( Z )
  DEALLOCATE( DZ )
  DEALLOCATE( DZR )
  DEALLOCATE( DZZ )
  DEALLOCATE( ZZ )
  
  
  DEALLOCATE( COR )
  DEALLOCATE( FSM )
  DEALLOCATE( DUM )
  DEALLOCATE( DVM )
  DEALLOCATE( ETA )  
  DEALLOCATE( ETAP )
  DEALLOCATE( H )
  DEALLOCATE( HU )
  DEALLOCATE( HV )
  DEALLOCATE( D )
  DEALLOCATE( DU )
  DEALLOCATE( DV )
  DEALLOCATE( WUSURF )
  DEALLOCATE( WVSURF )
  DEALLOCATE( WUBOT )
  DEALLOCATE( WVBOT )
  DEALLOCATE( CBC )
  DEALLOCATE( COAST)
  DEALLOCATE( PATM )
  DEALLOCATE( AM2D )
  DEALLOCATE( UA )
  DEALLOCATE( VA )
  DEALLOCATE( HUSQR )
  DEALLOCATE( HVSQR )
  
! 3D
  DEALLOCATE( U )
  DEALLOCATE( V )
  DEALLOCATE( W )
  DEALLOCATE( WR )
  DEALLOCATE( RHO )
  DEALLOCATE( AM )
  DEALLOCATE( AH )
  DEALLOCATE( KM )
  DEALLOCATE( KH )
  DEALLOCATE( KQ )
  DEALLOCATE( DDZ )
  DEALLOCATE( DUDZ )
  DEALLOCATE( DVDZ )
  DEALLOCATE( UADV )
  DEALLOCATE( VADV )
  DEALLOCATE( WADV )  
  if(associated(q2)) DEALLOCATE( Q2 )
  if(associated(q2l)) DEALLOCATE( Q2L )
  if(associated(deltau)) DEALLOCATE( DELTAU )
  if(associated(deltav)) DEALLOCATE( DELTAV )

  
  DEALLOCATE( UN,UNM1,DELUN,DELUSTAR,DELUNM1 )
  DEALLOCATE( VN,VNM1,DELVN,DELVSTAR,DELVNM1 )
  DEALLOCATE( WN,WNM1,DELWN,DELWSTAR,DELWNM1 )


!From mode_modesplit

  DEALLOCATE(UV,VU,UBCLIN,VBCLIN,C3U,C3V)
  DEALLOCATE(UCORR,VCORR,WCORR)
  DEALLOCATE(AX,AY,UV2D,VU2D,UASUM,VASUM)  
  DEALLOCATE(CORU,CORV,C2U,C2V)
  DEALLOCATE(UAD,VAD,UASTAR,VASTAR,ETASTAR)
  DEALLOCATE(UAF,UAB,VAF,VAB,ETAB,ETAF)


!FROM MOD_BOUND:

  DEALLOCATE(ALPHA, ALPHAE )
  DEALLOCATE(ALFA_U, ALFA_V, ALFA_ETA, ALFAE_U, ALFAE_V, ALFAE_ETA)

!! the arrays containing boundary values ready to be applied.  
!  real, dimension(:,:,:), allocatable :: u_bound, v_bound, w_bound, s_bound, t_bound, q2_bound, q2l_bound
!  real, dimension(:,:), allocatable :: ua_bound, va_bound, eta_bound

  DEALLOCATE(U_BOUND, V_BOUND, W_BOUND)
  IF(ASSOCIATED(Q2_BOUND)) DEALLOCATE(Q2_BOUND)
  IF(ASSOCIATED(Q2L_BOUND)) DEALLOCATE(Q2L_BOUND) 
  DEALLOCATE(UA_BOUND, VA_BOUND, ETA_BOUND, ETABFRS, ETATFRS)

  DEALLOCATE(UA_TIDE, VA_TIDE, ETA_TIDE)

! rivers
  
  DEALLOCATE(IRIVER, JRIVER, KRIVER1, KRIVER2)
  DEALLOCATE(CODERIVER, RIVERFORMAT)
  DEALLOCATE(SRIVER, TRIVER, FRIVER, DZRIVER, DEPRIVER, RIVER_NAME, CRIVER)
  
!
! Variables used in the case of fresh water inputs through the bottom, like
! with a dived fresh water source.
!
  DEALLOCATE(IWELL,JWELL,WELL_FORMAT)
  DEALLOCATE(SWELL,TWELL,FWELL,CWELL)
  DEALLOCATE(WELL_NAME)
  

  
! tidal variables
!  
  DEALLOCATE(FREQ)
  DEALLOCATE(ETATAMP)
  DEALLOCATE(ETATPHA)
  DEALLOCATE(UATAMP)
  DEALLOCATE(UATPHA)
  DEALLOCATE(VATAMP)
  DEALLOCATE(VATPHA)
 
!
!  Atmospheric forcing variables
!
   
!   LOGICAL WIND,ATMPRESS
!   REAL, DIMENSION :: WSPEEDU,WSPEEDV
!   REAL, DIMENSION(:,:), ALLOCATABLE :: WSPEEDU,WSPEEDV   
  DEALLOCATE(WSPEEDU)
  DEALLOCATE(WSPEEDV)

  CALL DEALLOCATE_BOMPLOT()
  CALL DEALLOCATE_PGPLOT()
  CALL DEALLOCATE_TIMESERIES()
  
  CALL DEALLOCATE_CONCENTRATION()

  IF(NONHYD_ALLOCATED) CALL DEALLOCATE_NONHYD()
  IF(ST_ALLOCATED) CALL DEALLOCATE_ST()
  IF(DENSITY_ALLOCATED) CALL DEALLOCATE_DENSITY()
  CALL DEALLOCATE_RAMPUPS()
  CALL DEALLOCATE_BINREG()
  CALL DEALLOCATE_MOD_MPI()
END SUBROUTINE DEALLOCATE_BOM



