SUBROUTINE ATMOSP
!
!  ATMOSP propagates the solution of the equations
!
!  d(U)/dt = - (1/RHO0)*d(Patm)/dx
!
!  d(V)/dt = - (1/RHO0)*d(Patm)/dy
!
!  where Patm is the atmospheric pressure
!
USE MOD_BOUND
USE STATE
IMPLICIT NONE
REAL XFACT,YFACT
integer i,j,k
!
XFACT = DT/(DX*RHO0)
YFACT = DT/(DY*RHO0)
DO J = J_EQ_2,J_EQ_JM
    DO I = I_EQ_2,I_EQ_IM
        IF (DUM(I,J).GT.0.) THEN
            DO K = 1,KB-1
                U(I,J,K) = U(I,J,K) - XFACT*(PATM(I,J)-PATM(I-1,J))
            END DO
        ENDIF
    end DO
end DO

if(jm > 1)then
    
DO J = J_EQ_2,J_EQ_JM
    DO I = I_EQ_2,I_EQ_IM
        IF (DVM(I,J).GT.0.) THEN
            DO K = 1,KB-1
                V(I,J,K) = V(I,J,K) - YFACT*(PATM(I,J)-PATM(I,J-1))
            END DO
        ENDIF
    END DO
END DO

END IF

!
!  Estimate a water level that balances the atmospheric pressure.
!
ETABFRS = -(PATM-PMEAN)/(RHO0*GRAV)

CALL DUMP_STATE("atmosp")
END
