! This module contain various run time swithces for BOM, that was stored in mod_main before.
! Core physics and numerics have most of the parameters in state.f90

MODULE CONTROL_PARAMETERS                           ! If .TRUE. then:
  LOGICAL :: ADVTURB=.FALSE.                        ! advect Q2 and Q2L?
  LOGICAL :: SMAG=.TRUE.                            ! compute AM, AM2D using Smagorinsky, constant fields otherwise.
  LOGICAL :: CM2DSPONGE=.FALSE.                     ! pick CM2D from the background 2d sponge array.
  LOGICAL :: CMSPONGE=.FALSE.
  LOGICAL :: AMSPONGE=.FALSE.
  LOGICAL :: AM2DSPONGE=.FALSE.
  LOGICAL :: KMSPONGE=.FALSE.
  LOGICAL :: KHSPONGE=.FALSE.
  LOGICAL :: VISCPOM=.FALSE.                        ! use POM style viscosity terms, otherwise simplified BOM terms.
  LOGICAL :: UNESCO=.FALSE.                         ! use accurate unesco eq. of state, otherwise simplified POM routine.
  LOGICAL :: FOURTH=.FALSE.                         ! use fourth order scheme for advection of S,T (RHO), otherwise superbee.
  LOGICAL :: AM2=.FALSE.                            ! use Adams Moulton corrector for momentum and s,t (rho)
  LOGICAL :: INTP4=.FALSE.                          ! use fourth order method(mccalpin) for internal pressure, otherwise std POM
  LOGICAL :: NONHYD=.FALSE.                         ! include effect of nonhydrostatic pressure
  LOGICAL :: NONHYD_ALLOCATED=.FALSE.               ! variables related to nonhydrostatic phys are allocated
  LOGICAL :: BINREG_ALLOCATED=.FALSE.               ! variables related to bin region averaging bc are allocated
  LOGICAL :: ADVECT_DENSITY=.FALSE.                 ! advect density instead of s,t
  LOGICAL :: DENSITY_ALLOCATED=.FALSE.              ! variables related to advection of density are allocated
  LOGICAL :: ST_ALLOCATED=.FALSE.                   ! variables related to advection of s,t are allocated
  LOGICAL :: ADVECT_S=.TRUE.                        ! advect salinity
  LOGICAL :: ADVECT_T=.TRUE.                        ! advect temperature
  LOGICAL :: DODUMPMASK=.true.                      ! dump an ascii chart of the bathymetry to the file grid.data
  LOGICAL :: STORE_SIGMA_LAYERS=.false.             ! dump z-coordinate data to the file sigma-layers.data
  LOGICAL :: SANITY_CHECK_FATAL=.true.              ! if false, ignore errors found in the sanity check
  LOGICAL :: TEST_FORCING_MODE=.false.              ! if true, read complete init and forcing for each step, but skip all numerics
  
  INTEGER :: FIRST_STEP=1                           ! the first timestep for the current session
  INTEGER :: VERTICAL_CLOSURE=1                     ! 0:constant fields for KM etc, 1: KM from Mellor-Yamada 2.5, 2:GOTM(TODO)
  INTEGER :: SOR_CHECK_RESID_INTERVAL=20            ! check for SOR convergence ever SOR_CHECK_RESID_INTERVALth iteration
  INTEGER :: DEFAULT_BOUNDARY_CONDITION=0           ! which OBC is the default, i.e. used if bc is not set, by default FRS.
  REAL    :: ADIV=0.0                               ! array default initial value
  
! data for the main setup file
  INTEGER, PARAMETER :: MAX_ACTIVE_LINES = 1000      ! the maximum number of lines allowed
  CHARACTER(200) :: LINES(MAX_ACTIVE_LINES)=" "     ! valid setup lines
  CHARACTER(200) :: NEWLINES(MAX_ACTIVE_LINES)=" "  ! same, but this is the list to be written
  INTEGER :: KEYWORDS, NEWKEYWORDS                  ! the number of keywords in the above lists.

  CHARACTER(10) :: INTERNAL_PRESSURE="fourth"
  
  CHARACTER(6) :: EOS="wang"                        ! use the wang EOS from POM to compute density by default. 
                                                    ! "unesco" is the other valid option.
  
  INTEGER :: RUNTIME_INTERVAL=-1000000              ! how often to reread and flush runtime.dat
  INTEGER :: WCOURANTFIX=0                          ! if > 0, increase KM by this factor in cells where WADV is too large, see my2halv.f90

! DO_CHECKPOINT is true if we are going to store the state during the
! current timestep. RESTART is true if the current run was started
! from restart files.
  LOGICAL :: RESTART=.FALSE., DO_CHECKPOINT=.FALSE.
  CHARACTER(200) :: CHECKPOINT_DIR                  ! name of directory where restart files are written (must end with /)

! prepend this PATH for input data files
  CHARACTER(LEN=200) :: DATA_PATH = ""

! temporary workaround for mod_particle
  character(80) :: particlets_dir   = "PTRAJ/"
  character(80) :: particledist_dir = "PDIST/"
  character(80) :: particlestat_dir = "PSTAT/"
END MODULE CONTROL_PARAMETERS
