SUBROUTINE DENS
!***BEGIN PROLOGUE DENS
!***DATE WRITTEN   910723 (YYMMDD)
!***PURPOSE DENS computes the density in all sea cells
!           as a function of S and T.
!
!***DESCRIPTION DENS is a computing the density RHO 
!           as a function of S and T by a formula from
!           WANG, JPO '84, 1191-1199.
!
!   ON EXIT
!
!   RHO    contains the densities - RHOAVER.
!           
!***ROUTINES CALLED-NONE
!***END PROLOGUE DENS
!
!   Global variables.
!
USE STATE
IMPLICIT NONE
!   Local variables.
!
REAL R1,R2,RR3,R4,R5,R6
INTEGER I,J,K
!
DATA R1,R2,RR3,R4,R5,R6/28.152,7.35E-2,4.69E-3,8.02E-1,2.0E-3,35.0/
!
!  THIS FUNCTION COMPUTES DENSITY-1; FROM WANG, JPO '84, 1191-1199.
!

DO K = 1,KB-1
  DO J = 1,J_EQ_JM
    DO I = 1,I_EQ_IM
      RHO(I,J,K) = R1-R2*T(I,J,K)-RR3*T(I,J,K)*T(I,J,K)+&
           &             (R4-R5*T(I,J,K))*(S(I,J,K)-R6)
      RHO(I,J,K) = RHO(I,J,K) - RHOAVER
    END DO
  END DO
END DO

CALL DUMP_STATE("dens")
END
