SUBROUTINE DEPTH(KB,Z,DZ,DZR,DZZ,ZZ)
!***BEGIN PROLOGUE DEPTH
!***DATE WRITTEN   910904 (YYMMDD)
!***REVISION DATE  910904 (YYMMDD)
!***AUTHOR
!            Jarle Berntsen, Institute of Marine Research,
!            Postboks 1870,
!            N-5024 Bergen-Nordnes, Norway
!            Email..  jarle.berntsen@mi.uib.no
!
!***PURPOSE DEPTH computes the array Z,DZ,DZR,DZZ and ZZ
!
!   DSBOT is the sigma-thickness of the bottom and surface layers
!   and must be given by the user if the below formulae by Lynch
!   et al. is to be applied.
!
! HA 2009-03-20 moved init of Z into setup2.f90  
!
!***END PROLOGUE DEPTH
!
!   Global variables.
!
  use qmpi, only : master
  use control_parameters, only : store_sigma_layers
  
  IMPLICIT NONE
  INTEGER KB
  REAL Z(KB),DZ(KB),DZR(KB),DZZ(KB),ZZ(KB)
!
!   Local variables.
!
  INTEGER K

  DO K = 1,KB
      DZ(K)=0.0
      DZR(K)=0.0
      DZZ(K)=0.0
  END DO
!
!   Compute ZZ.
!
  DO K = 1,KB-1
      ZZ(K)=(Z(K)+Z(K+1))/2.
  END DO
  ZZ(KB) = 0.
!
!    Compute DZ,DZZ and DZR.
!
  DO K = 1,KB-1
      DZ(K)=Z(K)-Z(K+1)
      DZR(K)=1./DZ(K)
      DZZ(K)=ZZ(K)-ZZ(K+1)
  END DO

!
! Write out the depth parameters.
!
  IF(MASTER)THEN
      WRITE(*,"(/2X,'K',7X,'Z',9X,'ZZ',9X,'DZ',9X,'DZZ',/)")
      DO K = 1,KB
          WRITE(*,"(1X,I5,4F10.3)") K,Z(K),ZZ(K),DZ(K),DZZ(K)
      END DO
      IF(STORE_SIGMA_LAYERS) THEN
          OPEN(2000004,FILE='sigma-layers.data')
          DO K=1,KB
              WRITE(2000004,"(1X,I5,4F10.3)") K,Z(K),ZZ(K),DZ(K),DZZ(K)
          END DO
          CLOSE(2000004)          
      END IF
  END IF
END SUBROUTINE DEPTH
