SUBROUTINE DIAGNOSE(INR)
! 
! Run time diagnostics
!  
  USE STATE
  USE MOD_BOUND
  USE CONTROL_PARAMETERS
  
  IMPLICIT NONE

  INTEGER I,J,K,K2,INR, GI,GJ,ci,cj,ck
  REAL VELMAX,SMAX
  PARAMETER (VELMAX=10.0 ,SMAX = 100.0)
  LOGICAL DO_CONTINUE, WARN
  INTEGER, PARAMETER :: DUNIT=1000005
  CHARACTER(LEN=300) STRING
  
  DO_CONTINUE=.TRUE.

  K=1
  DO J=1,J_EQ_JM
      DO I=1,I_EQ_IM
          IF(DO_CONTINUE)THEN
              IF(ABS(ETA(I,J)) > 20.0) CALL GETIND("Too large ETA",GI,GJ,CI,CJ,CK,DO_CONTINUE)
              IF(ABS(UA(I,J)) > VELMAX) CALL GETIND("Too large UA",GI,GJ,CI,CJ,CK,DO_CONTINUE)
              IF(ABS(VA(I,J)) > VELMAX) CALL GETIND("Too large VA",GI,GJ,CI,CJ,CK,DO_CONTINUE)
          end if
      end do
  end do

  
! Compute maximum vertical courant number
  
  WCOURANT = 0.0
  DO K=1,KB-1
      DO J=1,J_EQ_JM
          DO I=1,I_EQ_IM
              IF(H(I,J) > 0.0) WCOURANT = MAX(WCOURANT, DT*ABS(WADV(I,J,K)/(H(I,J)*DZ(K))))
          END DO
      END DO
  END DO

  CALL REDUCE('max', WCOURANT, root=0)
!  if(wcourant > ... ) do something?

  
  if(do_continue)then
      

  DO K = 1,KB-1
      DO J = 1,J_EQ_JM
          DO I = 1,I_EQ_IM
              IF(DO_CONTINUE)THEN
                  IF (ABS(U(I,J,K)).GT.VELMAX) call getind('TOO LARGE U',GI,GJ,ci,cj,ck,do_continue)
                  IF (ABS(V(I,J,K)).GT.VELMAX) call getind('TOO LARGE V',GI,GJ,ci,cj,ck,do_continue)
                  IF (.not.advect_density)then
                      if(S(I,J,K).GT.SMAX) call getind('TOO LARGE S',GI,GJ,ci,cj,ck,do_continue)
                  end IF
              END IF
          END DO
      END DO
  END DO

  end if

  
  IF(.NOT. DO_CONTINUE) THEN
      write(string,'("diagnose-",i0,"x",i0)') block%ibn, block%jbn
      open(unit=dunit, file=string, recl=200)

      write(*,*) "DIAGNOSE: dumping profiles to file=", trim(string)

      write(dunit,'(a,i0)') "ISTEP = ", istep      
      write(dunit,'(a,i0)') "MYID = ", myid
      write(dunit,'(a,i0,1x,i0,1x,i0,a,i0,a)') "----- LOCATION: i,j,k=", GI,GJ,ck, " INR=",inr,"-----"
      
      write(dunit,*) "U V DU DV ETA ETA_BOUND ETA_TIDE in this point:"
      write(string,*) U(CI,CJ,CK),V(CI,CJ,CK), DU(CI,CJ), DV(CI,CJ), ETA(CI,CJ), ETA_BOUND(CI,CJ), ETA_TIDE(CI,CJ)
      write(dunit,'(a)') trim(string)

      if(advect_density)then
          WRITE(dunit,*)'---- K U V RHO: -----'
      else
          WRITE(dunit,*)'---- K U V RHO S T: -----'
      end if
      
      DO K2 = 1,KB-1
          if(advect_density)then
              WRITE(string,*)K2, U(CI,CJ,K2), V(CI,CJ,K2), RHO(CI,CJ,K2)+RHOAVER+1000
          else
              WRITE(string,*)K2, U(CI,CJ,K2), V(CI,CJ,K2), RHO(CI,CJ,K2)+RHOAVER+1000, S(CI,CJ,K2), T(CI,CJ,K2)
          end if
          WRITE(dunit,'(a)') trim(string)
      END DO
      
      WRITE(dunit,*)'---- Mixing: K AM KH KM: -----'
      DO K2 = 1,KB-1
          write(string,*)K2,AM(CI,CJ,K2),KH(CI,CJ,K2),KM(CI,CJ,K2)
          write(dunit,'(a)') trim(string)
      END DO

      if(advect_density)then
          WRITE(dunit,*)'---- K U_BOUND V_BOUND RHO_BOUND: -----'
      else
          WRITE(dunit,*)'---- K U_BOUND V_BOUND S_BOUND T_BOUND: -----'
      end if
      
      DO K2 = 1,KB-1
          if(advect_density)then
              WRITE(string,*)K2, U_BOUND(CI,CJ,K2), V_BOUND(CI,CJ,K2), RHO_BOUND(CI,CJ,K2)+RHOAVER+1000
          else
              WRITE(string,*)K2, U_BOUND(CI,CJ,K2), V_BOUND(CI,CJ,K2), S_BOUND(CI,CJ,K2), T_BOUND(CI,CJ,K2)
          end if
          WRITE(dunit,'(a)') trim(string)
      END DO

      close(dunit)
  END IF

  CALL DUMP_STATE("diagnose")
  
  IF(.NOT. DO_CONTINUE) CALL ABORT()
  
CONTAINS

  SUBROUTINE GETIND(MESSAGE, GI,GJ,LI,LJ,LK, CONTINUE)
    IMPLICIT NONE
    CHARACTER(*) MESSAGE
    INTEGER GI,GJ,LI,LJ,LK
    LOGICAL CONTINUE

! we want the first error    
    if(.not.continue) return
    
    WRITE(*,'(i0,1x,a)') MYID, MESSAGE
    GI=block%fi+I-1
    GJ=block%fj+J-1
    LI=I
    LJ=J
    LK=K
    CONTINUE=.FALSE.
  END SUBROUTINE GETIND
  
END SUBROUTINE DIAGNOSE
