subroutine dump_state(label)
! 
!  this routine is called at strategic points. the routine prints
!  a label and the norms of a selected list of arrays (chosen below)
!  the routine also has a primitive timer, that measure the time since the last call.  
! 
!  the amount of output is chosen via the variable debugnormlevel (mod_dump_state.f90)
!  
!  debugnormlevel=0  complete silence
!  debugnormlevel=1  just print the label and the wall time since last call to observe progress
!  debugnormlevel=2  print all selected maxmin's
!  debugnormlevel=3  print all selected maxmin's also in 2d loop.
!  debugnormlevel=4  print selected 3d norms and "wet" maxmin values
!  debugnormlevel=5  also print selected norms in 2d loop (see modesplit.f90)
!  
!  the type of norm used is chosen via the variable debugnorm (in mod_dump_state.f90)
!  
!  debugnorm=0 gives the sum norm that can be compared to the mpi model
!  debugnorm=1 gives the "max" of the "abs" of each full array
!  debugnorm=2 is the integer checksum of the array (used to check for bit exactness)  
!  debugnorm=3 the character checksum of the array, same as 2, but
!              avoids the rare cancellation effect, but since it is
!              the sum of positive numbers the headroom is smaller.
! 
!  when comparing model state it is recommended to turn off all optimization, as
!  these rearrange operations, that may lead to differences due to round off.
! 
! Helge Avlesen, para//ab <avle@ii.uib.no>
! 
  use control_parameters
  use mod_bound
  use state
  use mod_modesplit
  use norms
  use time_util
  use maxmin
  use pgplot
  implicit none
  logical dumpmm
  
  character(len=*) label
  integer(8) :: time_since_last=0_8, last_time=0_8, curtime=0_8

  if(dump_state_pgplot == 1 .and. .not. in_2d) call batch_pgplot(label)
  if(dump_state_pgplot == 2) call batch_pgplot(label)
  if(dump_state_pgplot == 3) call batch_pgplot(label)
  
  if(debugnormlevel < 1) return

  if(dump_state_barrier) call barrier()  

  if(istep < first_dump_state) return
  
! let debugnormlevel decide which norms to compute and view:
! level 0; nada, nothing, null
! level 1; print label and timing, nothing else
! level 2; print selected maxmin's
! level 3; print norms in 3d loop
! level 4; print selected maxmin's also in 2d loop
! level 5; also print norms in the 2d loop.  
  
! true if maxmins are to be dumped:  
  dumpmm = (debugnormlevel == 2 .and. .not. in_2d) .or. &
           (debugnormlevel == 3 .and. .not. in_2d) .or. &
           (debugnormlevel == 4 ) .or. &
            debugnormlevel == 5
  
  if(last_time == 0) last_time = rdtsc() 
  curtime = rdtsc()
  time_since_last = curtime - last_time
  last_time = curtime

  if(master)then
      if((.not.in_2d.and.debugnormlevel==1) .or. (in_2d .and. (debugnormlevel==3 .or. debugnormlevel==5)))then
          write(*,'(1x,a,1x,f0.6,a)') trim(label), real(time_since_last,8)/real(clock_frequency,8),"s"
      else
          if(dumpmm) write(*,'(/,a,1x,f0.6,a)') trim(label), real(time_since_last,8)/real(clock_frequency,8),"s"
      end if
  end if
  if(debugnormlevel < 2) return
  
  if(dumpmm) call dump_maxmin()

  if(debugnormlevel < 4) return
  if(debugnormlevel < 5 .and. in_2d) return
  
  if(istep <= 1)then

! these should normally not change after first step

      call find_norm("cor", cor)
      call find_norm("fsm", fsm)
      call find_norm("dum", dum)
      call find_norm("dvm", dvm)
      call find_norm("cbc", cbc)
      call find_norm("h", h)
      call find_norm("hu", hu)
      call find_norm("hv", hv)
      call find_norm("husqr", husqr)
      call find_norm("hvsqr", hvsqr)

      if(tide)then
          call find_norm("etatamp", etatamp)
          call find_norm("etatpha", etatpha)
          call find_norm("uatamp", uatamp)
          call find_norm("uatpha", uatpha)
          call find_norm("vatamp", vatamp)
          call find_norm("vatpha", vatpha)
      end if
  end if

  if(in_modesplit.or.istep < 1)then
      call find_norm("ua", ua)
      call find_norm("va", va)
      call find_norm("eta", eta)
      call find_norm("etap", etap)
      if(in_2d)then
          call find_norm("am2d", am2d) 
          call find_norm("ax",ax)
          call find_norm("ay",ay)
          call find_norm("uv2d",uv2d)
          call find_norm("vu2d",vu2d)
          call find_norm("uasum",uasum)
          call find_norm("vasum",vasum)
          call find_norm("coru",coru)
          call find_norm("corv",corv)
          call find_norm("c2u",c2u)
          call find_norm("uad",uad)
          call find_norm("vad",vad)
          call find_norm("uastar",uastar)
          call find_norm("vastar",vastar)
          call find_norm("etastar",etastar)
          call find_norm("uaf",uaf)
          call find_norm("uab",uab)
          call find_norm("vaf",vaf)
          call find_norm("vab",vab)
          call find_norm("etab",etab)
          call find_norm("etaf",etaf)

          call find_norm("ua_bound", ua_bound)
          call find_norm("va_bound", va_bound)
          call find_norm("eta_bound", eta_bound)
          call find_norm("etabfrs", etabfrs)
          call find_norm("etatfrs", etatfrs)
          call find_norm("ua_tide", ua_tide)
          call find_norm("va_tide", va_tide)
          call find_norm("eta_tide", eta_tide)
          
          call print_norms
          return
      end if
      call find_norm("uv",uv)
      call find_norm("vu",vu)
      call find_norm("ubclin",ubclin)
      call find_norm("c3u",c3u)
      call find_norm("c3v",c3v)
      call find_norm("ucorr",ucorr)
      call find_norm("vcorr",vcorr)
      call find_norm("wcorr",wcorr)
  end if

! velocities

  call find_norm("u", u)
  call find_norm("v", v)
  call find_norm("w", w)
  call find_norm("uadv", uadv)
  call find_norm("vadv", vadv)
  call find_norm("wr", wr)
  call find_norm("deltau", deltau)
  call find_norm("deltav", deltav)

  call find_norm("d", d)
  call find_norm("du", du)
  call find_norm("dv", dv)
  call find_norm("ddz", ddz)
  call find_norm("dudz", dudz)
  call find_norm("dvdz", dvdz)

! top and bottom boundary  
  call find_norm("wspeedu", wspeedu)
  call find_norm("wspeedv", wspeedv)
  call find_norm("wusurf", wusurf)
  call find_norm("wvsurf", wvsurf)
  call find_norm("wubot", wubot)
  call find_norm("wvbot", wvbot)
  if(.not.advect_density)then
      call find_norm("ssurf", ssurf)
      call find_norm("tsurf", tsurf)
  end if
  call find_norm("patm", patm)
  
  if(.not.advect_density)then
      call find_norm("s", s)
      call find_norm("t", t)
  end if
  call find_norm("rho", rho)

  call find_norm("am", am)
  call find_norm("ah", ah)
  call find_norm("km", km)
  call find_norm("kh", kh)
  call find_norm("kq", kq)
  if(vertical_closure == 1)then
      call find_norm("q2", q2)
      call find_norm("q2l", q2l)
  end if

  if(nonhyd)then
      call find_norm("p", p)  
  end if

! boundary arrays

  call find_norm("u_bound", u_bound)
  call find_norm("v_bound", v_bound)
  call find_norm("w_bound", w_bound)
  if(st_allocated)then
      call find_norm("s_bound", s_bound)
      call find_norm("t_bound", t_bound)
  end if
  if(vertical_closure==1) then
      call find_norm("q2_bound", q2_bound)
      call find_norm("q2l_bound", q2l_bound)
  end if
  if(advect_density) call find_norm("rho_bound", rho_bound)
  
  call print_norms

end subroutine dump_state
