module etaimplicit

  private
  public iMain, implicit_eta, eta_numiter

  logical :: implicit_eta=.false.
  integer :: eta_numiter=40

  real, allocatable :: alpha2(:,:), beta2(:,:)
  real, dimension(:,:,:), allocatable :: uv,vu

  real, dimension(:,:,:), allocatable :: usave,vsave
  real, dimension(:,:), allocatable :: w3
  real, pointer :: xflux(:,:,:),yflux(:,:,:)

  
contains

  
  subroutine iMain()
!    
! alternative timestepping which is less accurate but potentially much
! faster than the default modesplitting.  based on the original
! implicit ETA algorithm from seamount/hoved.bak, tries to reuse the
! bom5 routines as much as possible.
!
    use state
    use mod_bound
    use control_parameters

    implicit none
    integer i,j,k

    if(.not.allocated(alpha2)) call allocate_implicit()
    
    etap = eta
    w = 0.
    
! disse to er parallellisert men ikke testet.
    call coriolis
    call crank
    
! crank beregner w paa sm maate som modesplit wadv i bom5. i bom5 settes w lik wadv hydrostatisk. derfor
    wadv = w

! beregn eta_tide, etc. på tiden TIME
    if (tide) call tidefrs(n2d)
    
! oppdater ua,va,eta _bound, antas også ok å gjenbruke direkte fra bom5
    call bound2(1)
    call bound2(3)
    call updatedd
!
! superbeef: ah er med, wadv brukes istedet for w, elver er med, setter
! fieldold=field i begynnelsen og kommenterer ut nytt m modesplt.
!    
    if(advect_density) call superbeef(rhon,rho,delrhostar,rhoaver,1)
    if(advect_s)       call superbeef(sn,s,delsstar,saver,1)
    if(advect_t)       call superbeef(tn,t,deltstar,taver,2)
    do i=1,maxnumconc
        if(co(i)%active) call superbeef(co(i)%concn, co(i)%conc, co(i)%delconcstar, co(i)%aver,i+2)
    end do
!
! include advection and viscosity
!
    call smagor                  ! ok
    
!   call horviscuv(u,v,c3u,c3v)  ! legg bidragene fra diff. i c3u,c3v og bruk de i superbee?
    call superbeeuv              ! mangler AM, wadv brukes for w, forskjell i vertikalfluks (/dz)

! vertviscuvb same as orig implicit by setting ubclin to u, and ua to 0.
    call vertviscuvb(u,v)
!
! include internal pressure.
!
    call internal                ! oppdat deltau obs faktoren du etc.

    do k = 1,kb-1
        do j=j_eq_2,jm_mi_1
            do i=i_eq_2,im_mi_1
                if(du(i,j) > 0.) u(i,j,k) = u(i,j,k) + dt*deltau(i,j,k)/du(i,j)
                if(dv(i,j) > 0.) v(i,j,k) = v(i,j,k) + dt*deltav(i,j,k)/dv(i,j)
            end do
        end do
    end do
  end subroutine iMain


  subroutine allocate_implicit()
    use state
    allocate (uv(i1:i2,j1:j2,kb),vu(i1:i2,j1:j2,kb))
    allocate(alpha2(i1:i2,j1:j2), beta2(i1:i2,j1:j2))

    uv=0
    vu=0
    alpha2(:,:) = cos(dt*cor(:,:))
    beta2(:,:) = sin(dt*cor(:,:))

    allocate (xflux(i1:i2,j1:j2,kb), yflux(i1:i2,j1:j2,kb), w3(i1:i2,j1:j2))
    allocate (usave(i1:i2,j1:j2,kb), vsave(i1:i2,j1:j2,kb))

    xflux=0
    yflux=0
    w3=0
    usave=0
    vsave=0    
  end subroutine allocate_implicit

  
  subroutine deallocate_implicit()
    deallocate(uv,vu,alpha2,beta2)
    deallocate(xflux, yflux, w3, usave, vsave)
  end subroutine deallocate_implicit

  
  SUBROUTINE CORIOLIS
!
!  CORIOLIS propagates the solutions of the
!  equations
!
!  d(UD)/dt = f*VD
!  d(VD)/dt = -f*UD
!
!  with a exact, except for spatial averaging, method.
!
!  U is updated for indices (2:IM,1:JM-1,1:KB-1)
!  V is updated for indices (1:IM-1,2:JM,1:KB-1)
!
    USE STATE
    IMPLICIT NONE
    INTEGER I,J,K
!
!Define V*D in U-points.
!
    DO K = 1,KB-1
        DO J = 1,JM_mi_1
            DO I = i_eq_2,i_eq_IM
                VU(I,J,K) = 0.25*(DV(I,J+1)*V(I,J+1,K)+DV(I,J)*V(I,J,K)+&
                     &  DV(I-1,J+1)*V(I-1,J+1,K)+DV(I-1,J)*V(I-1,J,K))
            END DO
        END DO
    END DO
!
!Define U*D in V-points.
!
    DO K = 1,KB-1
        DO J = j_eq_2,j_eq_JM
            DO I = 1,IM_mi_1
                UV(I,J,K) = 0.25*(DU(I+1,J)*U(I+1,J,K)+DU(I,J)*U(I,J,K)+&
                     & DU(I+1,J-1)*U(I+1,J-1,K)+DU(I,J-1)*U(I,J-1,K))
            END DO
        END DO
    END DO
!
!Propagate the solution taking the Coriolis terms into account.
!
    DO K = 1,KB-1 
        DO J = 1,JM_mi_1
            DO I = i_eq_2,i_eq_IM
                IF (DUM(I,J).NE.0.) THEN
                    U(I,J,K) = ALPHA2(I,J)*U(I,J,K) + BETA2(I,J)*VU(I,J,K)/DU(I,J)
                ENDIF
            END DO
        END DO
    END DO
!
    DO K = 1,KB-1
        DO J = j_eq_2,j_eq_JM
            DO I = 1,IM_mi_1
                IF (DVM(I,J).NE.0.) THEN
                    V(I,J,K) = ALPHA2(I,J)*V(I,J,K) - BETA2(I,J)*UV(I,J,K)/DV(I,J)
                ENDIF
            END DO
        END DO
    END DO

    call dump_state("coriolis")
  END SUBROUTINE CORIOLIS

    
  SUBROUTINE CRANK
!
! CRANK propagates the solution of the equations
!
! d(UD)/dt = -g*D*d(eta)/dx
!
! d(VD)/dt = -g*D*d(eta)/dy
!
! d(eta)/dt = -d(UA)/dx -d(VA)/dy
!
! with the Crank-Nicolson method.
!
    USE STATE
    USE MOD_BOUND
    IMPLICIT NONE
!
! Local variables.
!
    INTEGER I,J,K,L
    REAL RX,GAMMA,RY,DELTA,AREA,bn,frac
    LOGICAL INBLOCK
!
! Save U and V values from timestep n
!
    USAVE = U
    VSAVE = V
!
!Recompute UA.
!
    UA = 0.
    DO K = 1,KB-1
        DO J = 1,j_eq_JM
            DO I = 1,i_eq_IM
                UA(I,J) = UA(I,J) + DUM(I,J)*U(I,J,K)*DZ(K)*DU(I,J)
            END DO
        END DO
    END DO
!
!Recompute VA.
!
    VA = 0.
    DO K = 1,KB-1
        DO J = 1,J_eq_JM
            DO I = 1,i_eq_IM
                VA(I,J) = VA(I,J) + DVM(I,J)*V(I,J,K)*DZ(K)*DV(I,J)
            END DO
        END DO
    END DO

!
!  River Specific
!
    IF (RIVLOG) THEN
        DO K = 1,NUMC
            I=IRIVER(K)
            J=JRIVER(K)
            CALL GLOBALIND2LOCAL(I,J,INBLOCK)

            IF(INBLOCK)THEN            
                IF (CODERIVER(k).EQ.1) THEN
                    UA(I,J) = FRIVER(k)/DY
                ELSEIF (CODERIVER(k).EQ.2) THEN
                    VA(I,J) = FRIVER(k)/DX
                ELSEIF (CODERIVER(k).EQ.3) THEN
                    if(i+1.gt.i2) cycle                      
                    UA(I+1,J) = -FRIVER(k)/DY
                ELSEIF (CODERIVER(k).EQ.4) THEN
                    if(j+1.gt.j2) cycle
                    VA(I,J+1) = -FRIVER(k)/DX
                ELSE
                    WRITE(*,*)'ERROR IN CRANK'
                    WRITE(*,*)I,CODERIVER(k)
                    STOP
                ENDIF
            END IF
        END DO
    END IF
    
!
!Gather contributions from time step n in working arrays.
!
    RX = 0.5*DT/DX
    GAMMA = RX*GRAV
    RY = 0.5*DT/DY
    DELTA = RY*GRAV
!
!Compute contributions from time step n to the U-moment equations.
!
    DO K = 1,KB-1
        DO J = 1,j_eq_JM
            DO I = i_eq_2,i_eq_IM
                XFLUX(I,J,K) = U(I,J,K)*DU(I,J) - GAMMA*DU(I,J)*(ETA(I,J)-ETA(I-1,J))
            END DO
        END DO
    END DO
!
!Compute contributions from time step n to the V-moment equations.
!
    DO K = 1,KB-1
        DO J = j_eq_2,j_eq_JM
            DO I = 1,i_eq_IM
                YFLUX(I,J,K) = V(I,J,K)*DV(I,J) - DELTA*DV(I,J)*(ETA(I,J)-ETA(I,J-1))
            END DO
        END DO
    END DO
!
!Then for the continuity eqn.
!
    DO J = j_eq_2,JM_mi_1  
        DO I = i_eq_2,IM_mi_1
            W3(I,J)=ETA(I,J)+RX*GAMMA*(DU(I+1,J)*(ETA(I+1,J)-ETA(I,J))- &
                 &      DU(I,J)*(ETA(I,J)-ETA(I-1,J)))
            W3(I,J) = W3(I,J) - DT*((UA(I+1,J)-UA(I,J))/DX)
            W3(I,J)=W3(I,J)+RY*DELTA*(DV(I,J+1)*(ETA(I,J+1)-ETA(I,J))- &
                 &      DV(I,J)*(ETA(I,J)-ETA(I,J-1)))
            W3(I,J) = W3(I,J) - DT*((VA(I,J+1)-VA(I,J))/DY)
            W3(I,J) = W3(I,J)*FSM(I,J)
        END DO
    END DO
!
!   Set up the Three-Diagonal system of equations.
!   when solving for the intermediate value of ETA.
!
    bn = 0.
    DO J = j_eq_2,JM_mi_1  
        DO I = i_eq_2,IM_mi_1
            bn = bn + w3(i,j)*w3(i,j)
        end do
    end do

    call gaussit(w3, bn)
!
!Update U.
!
    DO K = 1,KB-1
        DO J = 1,j_eq_JM
            DO I = i_eq_2,i_eq_IM
                IF (DU(I,J).GT.0.0) THEN
                    U(I,J,K) = (XFLUX(I,J,K)-GAMMA*DU(I,J)*(ETA(I,J)-ETA(I-1,J)))/DU(I,J) 
                ENDIF
            END DO
        END DO
    END DO
!
!Update V.
!
    DO K = 1,KB-1
        DO J = j_eq_2,j_eq_JM
            DO I = 1,i_eq_IM
                IF (DV(I,J).GT.0.0) THEN
                    V(I,J,K) = (YFLUX(I,J,K)-DELTA*DV(I,J)*(ETA(I,J)-ETA(I,J-1)))/DV(I,J)
                ENDIF
            END DO
        END DO
    END DO
!
!   compute the sigma-coord. vertical velocity W
!   from the equation
! 
!   d(W)/d(sigma) = -(d(eta)/d(T) + d(U*D)/d(X) + d(V*D)/d(Y)) .
!
!
    AREA = DX*DY
!
!   Compute the horisontal fluxes.
!
!   d(U*D)/d(X) times DX*DY
!
    XFLUX = 0.
    YFLUX = 0.
    UADV = 0.5*(U + USAVE)
    VADV = 0.5*(V + VSAVE)
!
    DO J = 1,j_eq_JM
        DO I = 1,i_eq_IM
            XFLUX(I,J,1) = DZ(1)*UADV(I,J,1)
            DO K = 2,KB-1
                XFLUX(I,J,K) = XFLUX(I,J,K-1) + DZ(K)*UADV(I,J,K)
            END DO
        END DO
    END DO
    XFLUX = XFLUX*DY
!
    DO K = 1,KB-1
        DO J = 1,j_eq_JM
            DO I = 1,i_eq_IM
                XFLUX(I,J,K) = XFLUX(I,J,K)*DU(I,J)
            END DO
        END DO
    END DO
!
!   d(V*D)/d(Y) times DX*DY
!
    DO J = 1,j_eq_JM
        DO I = 1,i_eq_IM
            YFLUX(I,J,1) = DZ(1)*VADV(I,J,1)
            DO K = 2,KB-1
                YFLUX(I,J,K) = YFLUX(I,J,K-1) + DZ(K)*VADV(I,J,K)
            END DO
        END DO
    END DO
    YFLUX = YFLUX*DX
    DO K = 1,KB-1
        DO J = 1,j_eq_JM
            DO I = 1,i_eq_IM
                YFLUX(I,J,K) = YFLUX(I,J,K)*DV(I,J)
            END DO
        END DO
    END DO

    call tag_ghost(xflux, east, 1)
    call tag_ghost(yflux, north, 1)
    call update_ghost()

!
!  River spesific
!
    IF (RIVLOG) THEN
        DO L = 1,NUMC
            I=IRIVER(L)
            J=JRIVER(L)
            CALL GLOBALIND2LOCAL(I,J,INBLOCK)

            IF(INBLOCK)THEN                
                IF (CODERIVER(L).EQ.1) THEN
                    FRAC = DZ(1)/DZRIVER(L)
                    XFLUX(i,j,1) = FRAC*FRIVER(L)
                    IF (KRIVER2(L).GT.1) THEN
                        DO K = 2,KRIVER2(L)
                            FRAC = DZ(K)/DZRIVER(L)
                            XFLUX(i,j,K) = XFLUX(i,j,K-1) + FRAC*FRIVER(L)  
                        END DO
                    ENDIF
                    IF (KRIVER2(L).LT.KB-1) THEN
                        DO K = KRIVER2(L)+1,KB-1
                            XFLUX(i,j,K) = XFLUX(i,j,KRIVER2(L))
                        END DO
                    ENDIF
                ELSEIF (CODERIVER(L).EQ.2) THEN
                    FRAC = DZ(1)/DZRIVER(L)
                    YFLUX(i,j,1) = FRAC*FRIVER(L)
                    IF (KRIVER2(L).GT.1) THEN
                        DO K = 2,KRIVER2(L)
                            FRAC = DZ(K)/DZRIVER(L)
                            YFLUX(i,j,K) = YFLUX(i,j,K-1) + FRAC*FRIVER(L)
                        END DO
                    ENDIF
                    IF (KRIVER2(L).LT.KB-1) THEN
                        DO K = KRIVER2(L)+1,KB-1
                            YFLUX(i,j,K) = YFLUX(i,j,KRIVER2(L))
                        END DO
                    ENDIF
                ELSEIF (CODERIVER(L).EQ.3) THEN
                    if(i+1.gt.i2) cycle                    
                    FRAC = DZ(1)/DZRIVER(L)
                    XFLUX(i+1,j,1) = -FRAC*FRIVER(L)
                    IF (KRIVER2(L).GT.1) THEN
                        DO K = 2,KRIVER2(L)
                            FRAC = DZ(K)/DZRIVER(L)
                            XFLUX(i+1,j,K) = XFLUX(i+1,j,K-1) - FRAC*FRIVER(L)
                        END DO
                    ENDIF
                    IF (KRIVER2(L).LT.KB-1) THEN
                        DO K = KRIVER2(L)+1,KB-1
                            XFLUX(i+1,j,K) = XFLUX(i+1,j,KRIVER2(L))
                        END DO
                    ENDIF
                ELSEIF (CODERIVER(L).EQ.4) THEN
                    if(j+1.gt.j2) cycle
                    FRAC = DZ(1)/DZRIVER(L)
                    YFLUX(i,j+1,1) = -FRAC*FRIVER(L)
                    IF (KRIVER2(L).GT.1) THEN
                        DO K = 2,KRIVER2(L)
                            FRAC = DZ(K)/DZRIVER(L)
                            YFLUX(i,j+1,K) = YFLUX(i,j+1,K-1) - FRAC*FRIVER(L)
                        END DO
                    ENDIF
                    IF (KRIVER2(L).LT.KB-1) THEN
                        DO K = KRIVER2(L)+1,KB-1
                            YFLUX(i,j+1,K) = YFLUX(i,j+1,KRIVER2(L))
                        END DO
                    ENDIF
                ENDIF
            END IF
        END DO
    END IF
!
!   W = 0 for sigma = 0.
!
    W(:,:,1) = 0.
!
!   Compute W from the equation
!   d(w)/d(sigma) = -(d(eta)/d(T) + d(U*D)/d(X) + d(V*D)/d(Y)) .
!
    DO K = 1,KB-1
        DO J = 1,JM_mi_1
            DO I = 1,IM_mi_1
                W(I,J,K+1) = (XFLUX(I+1,J,K) - XFLUX(I,J,K) + &
                     &             YFLUX(I,J+1,K)-YFLUX(I,J,K))/AREA + &
                     &             ((-Z(K+1))*(ETA(I,J)-ETAP(I,J)))/DT
            END DO
        END DO
    END DO

    call dump_state("crank")
  END SUBROUTINE CRANK

  
  subroutine gaussit(w3,bn)
    use state
    implicit none

    integer i,j,iter,numit
    real fact1,fact2,fact3,fact4,bn,ip,norm
    real, dimension(im,jm)  :: c1,c2,c4,r,w3

    numit=30

    fact1 = 0.5*dt/dx
    fact2 = 0.5*dt/dy
    fact3 = grav*fact1**2
    fact4 = grav*fact2**2

    do j = 1,jm
        do i = 1,im
            c1(i,j)=fact3*du(i,j)
            c2(i,j)=fact4*dv(i,j)
        end do
    end do

!   gauss-seidel iterate numit times.
!
    DO ITER = 1,ETA_NUMITER

        do j = 2,jm-1
            do i = 2,im-1           
                c4(i,j) = w3(i,j) + c1(i+1,j)*eta(i+1,j) &
                     +c1(i,j)*eta(i-1,j)  &
                     +c2(i,j+1)*eta(i,j+1)+c2(i,j)*eta(i,j-1)
                eta(i,j) = c4(i,j)/(1.+c1(i+1,j)+c1(i,j) &
                     +c2(i,j+1)+c2(i,j))                      
                eta(i,j) = eta(i,j)*fsm(i,j)           
            end do
        end do

! update dynamical depth also, to make this a true CN approach

        do j = 1,jm
            do i = 1,im
                c1(i,j)=fact3*(hu(i,j)+ 0.5*(eta(i,j)+eta(i-1,j)) ) 
                c2(i,j)=fact4*(hv(i,j)+ 0.5*(eta(i,j)+eta(i,j-1)) )
            end do
        end do


        ip=0.
        do j=2,jm-1
            do i=2,im-1
                r(i,j) =fsm(i,j)*( w3(i,j)  & 
                     +c1(i,j)*eta(i-1,j)    &
                     +c1(i+1,j)*eta(i+1,j)  &
                     +c2(i,j)*eta(i,j-1)    &
                     +c2(i,j+1)*eta(i,j+1)  &
                     -(1+(c1(i+1,j)+c1(i,j))&
                     +(c2(i,j)+c2(i,j+1)))*eta(i,j) )
                ip=ip+r(i,j)*r(i,j)
            end do
        end do

        norm = sqrt(ip/bn)

        if(debug > 2 .and. master) write(*,*) '2-norm |b-Ax|/|b| > ',norm,' ITS=',iter-1

        if(norm < 1e-8) exit
    END DO
  end subroutine gaussit


end module etaimplicit
