SUBROUTINE HORVISCUV(UBCLIN,VBCLIN,C3U,C3V)
!
!           HORVISCUV computes the following terms from the
!           Baroclinic parts of the momentum equations: 
!
!           d(AM*(d(UBCLIN*D)/dy))/dy + d(AM*(d(UBCLIN*D)/dx))/dx
!  and
!           d(AM*(d(VBCLIN*D)/dx))/dx + d(AM*(d(VBCLIN*D)/dy))/dy
!
!           d(*) is the differential operator,
!           D is the dynamic depth,
!           AM is the horizontal eddy viscosity,
!           The net-fluxes for each cell are stored in the 3-D arrays
!           C3U and C3V.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE STATE
IMPLICIT NONE
REAL,dimension(i1:i2,j1:j2,kb):: UBCLIN,VBCLIN,C3U,C3V
!
!   Local variables.
!
!   XFLUX,YFLUX are used to store the diffusive fluxes in
!                     the three space directions
! ..
! .. Local Arrays ..
REAL,DIMENSION(I1:I2,J1:J2,KB) :: FIELDD
REAL, POINTER, DIMENSION(:,:,:) :: XFLUX, YFLUX
!
REAL AREA,NETFLUX
REAL DAM
INTEGER I,J,K,DJ
!
!
ALLOCATE(XFLUX(I1:I2,J1:J2,KB), YFLUX(I1:I2,J1:J2,KB))

C3U = 0.
C3V = 0.
AREA = DX*DY

! the DJ trick
DJ=1
IF(JM == 1) DJ=0

!
!  First the diffusive fluxes for the U equation in X-direction.
!
XFLUX = 0.
FIELDD = 0.
DO K = 1,KB-1
  DO J=1,j2 !J_EQ_JM
    DO I=1,i2  !I_EQ_IM
      FIELDD(I,J,K) = UBCLIN(I,J,K)*DU(I,J)
    END DO
  END DO
END DO

!
!  Compute differences in FIELDD in X-direction
!

DO J=J_EQ_2,JM_MI_1
    DO I=1,IM_MI_1
        IF (FSM(I,J).NE.0.) THEN
            DO K = 1,KB-1
                XFLUX(I,J,K) = DY*AM(I,J,K)*(fieldd(i+1,j,k)-fieldd(i,j,k))/DX
            END DO
        ENDIF
    END DO
END DO
!
!  Compute the diffusive fluxes for the U-equation in Y-direction.
!
YFLUX = 0.

if(jm > 1)then
!
!  Compute differences in FIELDD in Y-direction
!

DO J=1,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
        IF (DVM(I-1,J+1).NE.0. .AND. DVM(I,J+1).NE.0.) THEN
            DO K = 1,KB-1
                DAM = .25*(AM(I,J,K)+AM(I-1,J,K)+AM(I,J+1,K)+AM(I-1,J+1,K))
                YFLUX(I,J,K) = DX*DAM*(fieldd(i,j+1,k)-fieldd(i,j,k))/DY 
            END DO
        ENDIF
    END DO
END DO
end if

call tag_ghost(xflux,west,1)
call tag_ghost(yflux,south,1)
call update_ghost

!
!   Take a step to timestep n+1 for the U-equation
!   by adding up the fluxes in and out of the cell.
!


  DO K = 1,KB-1
      DO J=J_EQ_2,JM_MI_1
         DO I=I_EQ_2,IM_MI_1
            IF (DUM(I,J).GT.0.0) THEN
                NETFLUX = YFLUX(I,J,K) - YFLUX(I,J-DJ,K) 
                NETFLUX = NETFLUX + XFLUX(I,J,K) - XFLUX(I-1,J,K)
                C3U(I,J,K) =   NETFLUX/AREA
             END IF
        END DO
    END DO
END DO
!
!
!   Then the momentum equation in V.
!
!   First the diffusive fluxes in X-direction.
!
XFLUX = 0.
FIELDD = 0.
DO K = 1,KB-1
  DO J=1,j2     !J_EQ_JM
    DO I=1,i2   !I_EQ_IM
      FIELDD(I,J,K) = VBCLIN(I,J,K)*DV(I,J)
    END DO
  END DO
END DO
!
!  Compute differences in FIELDD in X-direction
!

DO K = 1,KB-1
    DO J = J_EQ_2,JM_MI_1
        DO I = 1,IM_MI_1
            IF (DUM(I+1,J-DJ).NE.0. .AND. DUM(I+1,J).NE.0.) THEN
                DAM = .25*(AM(I,J,K)+AM(I,J-DJ,K)+AM(I+1,J,K)+AM(I+1,J-DJ,K))
                XFLUX(I,J,K) = DY*DAM*(fieldd(i+1,j,k)-fieldd(i,j,k))/DX
            END IF
        END DO
    END DO
END DO
!
!  Compute the diffusive fluxes for the V-equation in Y-direction.
!
YFLUX = 0.


if(jm > 1)then
!
!  Compute differences in FIELDD in Y-direction
!

DO J=1,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    IF (FSM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        YFLUX(I,J,K) = DX*AM(I,J,K)*(fieldd(i,j+1,k)-fieldd(i,j,k))/DY
      END DO
    ENDIF
  END DO
END DO

end if

!
!   Take a step to timestep n+1 for the V-equation by adding up the
!   fluxes in and out of the cell.
!
call tag_ghost(xflux,west,1)
call tag_ghost(yflux,south,1)
call update_ghost


DO K = 1,KB-1
    DO J = J_EQ_2,JM_MI_1
        DO I = I_EQ_2,IM_MI_1
            IF (DVM(I,J).GT.0.0) THEN
                NETFLUX = YFLUX(I,J,K) - YFLUX(I,J-DJ,K)
                NETFLUX = NETFLUX + XFLUX(I,J,K) - XFLUX(I-1,J,K)
                C3V(I,J,K) =   NETFLUX/AREA
            END IF
        END DO
    END DO
END DO
CALL DUMP_STATE("horiscuv")
DEALLOCATE(XFLUX,YFLUX)
END
