SUBROUTINE HORVISCUV2D(C2U,C2V)
!
!           HORVISCUV2D computes contributions from the 
!           the horizontal viscosity terms of the depth integrated moment
!           equations:
!
!           d(AM2D*(d(UA*D)/dy))/dy + d(AM2D*(d(UA*D)/dx))/dx
!  and
!           d(AM2D*(d(VA*D)/dx))/dx + d(AM2D*(d(VA*D)/dy))/dy
!
!           The contributions are 
!           stored in the 2-D fields C2U and C2V before
!           exit
!
!           d(*) is the differential operator,
!           D is the dynamic depth,
!           AM2D is the vertically averaged horizontal eddy viscosity.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE STATE
IMPLICIT NONE
REAL C2U(i1:i2,j1:j2),C2V(i1:i2,j1:j2)
!
!   Local variables.
!
!   XFLUX,YFLUX are used to store the viscosity fluxes 
!                     in the two space directions
!   DAM is used to store averages of AM
!
! ..
! .. Local Arrays ..
REAL, POINTER :: XFLUX(:,:), YFLUX(:,:)
REAL :: FIELDD(I1:I2,J1:J2) !,DELTAI(I1:I2,J1:J2)
!
REAL AREA,NETFLUX
REAL DAM
INTEGER I,J, DJ

ALLOCATE(XFLUX(I1:I2,J1:J2),YFLUX(I1:I2,J1:J2))
DJ=1
IF(JM == 1) DJ=0

C2U = 0.
C2V = 0.
AREA = DX*DY
!
! Compute UA * DU 
!
XFLUX = 0.
FIELDD = 0.
DO J=J1,J2
  DO I=I1,I2
    FIELDD(I,J) = UA(I,J)*DU(I,J)
  END DO
END DO
!

DO J=J_EQ_2,JM_MI_1
  DO I=1,IM_MI_1
    IF (FSM(I,J).NE.0.) THEN
      XFLUX(I,J) = DY*AM2D(I,J)*(fieldd(i+1,j)-fieldd(i,j))/DX
    ENDIF
  END DO
END DO


YFLUX = 0.

IF(JM > 1)THEN


DO J=1,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    IF (DVM(I-1,J+1).NE.0. .AND. DVM(I,J+1).NE.0.) THEN
      DAM = .25*(AM2D(I,J)+AM2D(I-1,J)+AM2D(I,J+1)+AM2D(I-1,J+1))
      YFLUX(I,J) = DX*DAM*(fieldd(i,j+1)-fieldd(i,j))/DY  !DELTAI(I,J+1)
    ENDIF
  END DO
END DO

END IF

!
!   Compute the net viscosity fluxes into each ocean cell.
!
call tag_ghost(xflux,west,1)
call tag_ghost(yflux,south,1)
call update_ghost


DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    IF (DUM(I,J).GT.0.0) THEN
        NETFLUX = XFLUX(I,J) - XFLUX(I-1,J) + &
    &             (YFLUX(I,J) - YFLUX(I,J-DJ)) 
        C2U(I,J) =   NETFLUX/AREA
    ENDIF
  END DO
END DO
!
!   Then the momentum equation in V.
!
!   Compute VA * DV
!
XFLUX = 0.
FIELDD = 0.
  DO J=J1,J2
    DO I=I1,I2
      FIELDD(I,J) = VA(I,J)*DV(I,J)
    END DO
  END DO
!
!  Compute differences in FIELDD in X-direction
!

DO J=J_EQ_2,JM_MI_1
  DO I=1,IM_MI_1
    IF (DUM(I+1,J-DJ).NE.0. .AND. DUM(I+1,J).NE.0.) THEN
      DAM = .25*(AM2D(I,J)+AM2D(I,J-DJ)+AM2D(I+1,J)+AM2D(I+1,J-DJ))
      XFLUX(I,J) = DY*DAM*(fieldd(i+1,j)-fieldd(i,j))/DX
    ENDIF
  END DO
END DO
!
!  Compute the viscosity fluxes for the V-equation in Y-direction.
!
YFLUX = 0.

IF(JM > 1) THEN


DO J=1,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    IF (FSM(I,J).NE.0.) THEN
        YFLUX(I,J) = DX*AM2D(I,J)*(fieldd(i,j+1)-fieldd(i,j))/DY  !DELTAI(I,J+1)
    ENDIF
  END DO
END DO

END IF
!
!   Compute the net viscosity fluxes into each ocean cell.
!
call tag_ghost(xflux,west,1)
call tag_ghost(yflux,south,1)
call update_ghost


DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    IF (DVM(I,J).GT.0.0) THEN
        NETFLUX = XFLUX(I,J) - XFLUX(I-1,J) + &
    &             (YFLUX(I,J) - YFLUX(I,J-DJ)) 
        C2V(I,J) =   NETFLUX/AREA
    ENDIF
  END DO
END DO
!
call dump_state("horviscuv2d")
DEALLOCATE(XFLUX,YFLUX)
END
