SUBROUTINE INITIALIZE_FIELDS
!
! Compute all remaining fields that depend on fields initialized via the init system.
!  
  USE STATE
  USE CONTROL_PARAMETERS
  USE MOD_BOUND
  USE MOD_CONCENTRATION
  USE NORMS, ONLY : PARSUM
  USE TIMESERIES, ONLY : INIT_TIMESERIES_DENSITY
  USE SETUP2
  USE MOD_FORCING, ONLY : INITIALIZE_DENSITY_FORCING
  USE UTIL
  
  IMPLICIT NONE

  REAL VOLUME
  INTEGER I,J,K,N
  real, dimension(:,:,:), pointer :: tmp1,tmp2,tmp3

!
!  Define the bottom drag coefficients.
! 
  if(.not. cbc_set) then  
      CBC = 0.0

      DO J = 1,J_EQ_JM
          DO I = 1,I_EQ_IM
              IF (FSM(I,J).GT.0.0) THEN
                  CBC(I,J)=MAX(CD,.16/LOG((ZZ(KB-1)-Z(KB))*H(I,J)/Z0)**2)
              ENDIF
          END DO
      END DO

      call tag_ghost(CBC)
      call update_ghost      
  end if

!
!Compute the Density from Wang
!
  IF(.NOT.ADVECT_DENSITY)THEN
      IF(.NOT.RESTART) RHOAVER = 0.
      SELECT CASE(EOS)
      CASE("unesco")
          CALL DENSUNESCO
      CASE("wang")
          CALL DENS
      END SELECT
  END IF

  
  allocate(tmp1(i1:i2,j1:j2,kb))  ;  tmp1 = 0.0
  if(.not.advect_density) then
      allocate(tmp2(i1:i2,j1:j2,kb), tmp3(i1:i2,j1:j2,kb))
      tmp2 = 0.0 ;      tmp3 = 0.0
  end if
  
!
!  Compute SAVER,TAVER and RHOAVER.
!
  DO I = 1,IM_MI_1
      DO J = 1,JM_MI_1
          IF (FSM(I,J).GT.0.) THEN
              DO K = 1,KB-1
                  tmp1(i,j,k) = RHO(I,J,K)*H(I,J)*DZ(K)
                  if(.not. advect_density)then
                      tmp2(i,j,k) = S(I,J,K)*H(I,J)*DZ(K)
                      tmp3(i,j,k) = T(I,J,K)*H(I,J)*DZ(K)
                  end if
              END DO
          ENDIF
      END DO
  END DO

  if(.not.restart) mass = parsum(tmp1)
  deallocate(tmp1)

  if(.not. advect_density)then
      saver = parsum(tmp2)
      taver = parsum(tmp3)
      deallocate(tmp2,tmp3)      
  end if 

  volume = parsum(h)
  
  if(master)then
     if(volume <= 0.0) then
        write(*,*)'Error, Volume=', volume,' inspect current grid by opening file grid.data in an editor'
        call abort()
     end if
  end if

  if(.not.restart) MASS = MASS/VOLUME
  if(given_mass > 0) mass = given_mass
  if(restart) MASS=RHOAVER

  SAVER = SAVER/VOLUME
  TAVER = TAVER/VOLUME
  RHOAVER = MASS

  if(master)then
     WRITE(*,*)'Initial average RHO - 1000 =',MASS
     if(.not.advect_density) WRITE(*,*)"Average S =",SAVER," Average T =",TAVER
  end if
  
  call tag_ghost(RHO)
  call update_ghost

  
! now for all arrays that have had density read in from file with
! "true" value we need to subtract (1000+rhoaver) to get to the
! "internal" format
  
  if(.not.restart)then
      if(master) write(*,*)'Subtracting RHOAVER from RHO, so that RHO now is RHO=RHO - 1000 - RHOAVER'
      rho(:,:,1:kb-1) = rho(:,:,1:kb-1) - rhoaver

      if(advect_density) rho_bound(:,:,1:kb-1) = rho_bound(:,:,1:kb-1) - rhoaver

! HA sjekk dette
      if(trim(lower(internal_pressure)) == "balanced")then
          if(rhoback_set) rhoback = rhoback - 1000 - rhoaver

          if(rhobackfromrho) rhoback = rho
      end if
  end if

  call initialize_density_forcing(rhoaver)
  
  call init_timeseries_density(rhoaver)

  if(advect_density)then
      do i=1,numc
          sriver(i) = sriver(i) - (1000 + rhoaver)
      end do
  end if

  
!
!Initialize dynamic depths D, DU and DV.
!
  CALL UPDATEDD

!
!   Zero the arrays in land points.
!
  if(.not.advect_density)then
  do k = 1,kb-1
     do j = 1,j_eq_jm
        do i = 1,i_eq_im
           s(i,j,k) = s(i,j,k)*fsm(i,j)
           t(i,j,k) = t(i,j,k)*fsm(i,j)
        end do
     end do
  end do
  end if

  do N=1,maxnumconc
      if(co(n)%allocated)then
          do k = 1,kb-1
              do j = 1,j_eq_jm
                  do i = 1,i_eq_im
                      co(n)%conc(i,j,k) = co(n)%conc(i,j,k)*fsm(i,j)
                  end do
              end do
          end do
      end if
  end do
  do k = 1,kb-1
     do j = 1,j_eq_jm
        do i = 1,i_eq_im
           rho(i,j,k) = rho(i,j,k)*fsm(i,j)
           if(advect_density) rho_bound(i,j,k) = rho_bound(i,j,k)*fsm(i,j) 
           u(i,j,k) = u(i,j,k)*dum(i,j)
           v(i,j,k) = v(i,j,k)*dvm(i,j)
        end do
     end do
  end do
  do j = 1,j_eq_jm
     do i = 1,i_eq_im
        eta(i,j) = eta(i,j)*fsm(i,j)
     end do
  end do
  
  ETAP = ETA

! these are probably also generic initializations
  
  KM(:,:,1) = 0.
  KH(:,:,1) = 0.
  KQ(:,:,1) = 0.
  KM(:,:,KB) = 0.
  KH(:,:,KB) = 0.
  KQ(:,:,KB) = 0.

  
!
!  Initialize ver 4.0 variables. Take care not to overwrite variables
!  that are already initialized from the setup system in order to
!  enable clean restarts.
!
  IF(.NOT. UN_SET) UN = U
  IF(.NOT. UNM1_SET) UNM1 = U
  IF(.NOT. DELUN_SET) DELUN = 0.
  DELUSTAR = 0.
  DELUNM1 = 0.
!
  IF(.NOT. VN_SET) VN = V
  IF(.NOT. VNM1_SET) VNM1 = V
  IF(.NOT. DELVN_SET) DELVN = 0.
  DELVSTAR = 0.
  DELVNM1 = 0.
!
  IF(.NOT. WNM1_SET) WNM1 = 0.
  IF(.NOT. DELWN_SET) DELWN = 0.
  IF(.NOT. WN_SET) WN = 0.
  DELWSTAR = 0.
  DELWNM1 = 0.

  IF(.NOT. W_SET) W = 0.
  IF(.NOT. WADV_SET) WADV = 0.
  
  IF(ADVECT_DENSITY)THEN
      IF(.NOT.RHON_SET)    RHON = RHO
      IF(.NOT.RHONM1_SET)  RHONM1 = RHO
      IF(.NOT.DELRHON_SET) DELRHON = 0.
      DELRHOSTAR = 0.
      DELRHONM1 = 0.
  ELSE
      IF(.NOT.SN_SET)    SN = S
      IF(.NOT.SNM1_SET)  SNM1 = S
      IF(.NOT.DELSN_SET) DELSN = 0.
      DELSSTAR = 0.
      DELSNM1 = 0.
      
      IF(.NOT.TN_SET)    TN = T
      IF(.NOT.TNM1_SET)  TNM1 = T
      IF(.NOT.DELTN_SET) DELTN = 0.
      DELTSTAR = 0.
      DELTNM1 = 0.
  END IF

  IF(BINREG_SET .AND. .NOT. RESTART)THEN
      IF(.NOT.ADVECT_DENSITY)THEN
          IF(.NOT. S_INITIAL_SET) S_INITIAL = S_BOUND
          IF(.NOT. S_INITIAL_SET) T_INITIAL = T_BOUND
      ELSE
          IF(.NOT. RHO_INITIAL_SET) RHO_INITIAL = RHO_BOUND
      END IF
      DO N=1,MAXNUMCONC
          IF(CO(N)%ALLOCATED) CO(N)%CONC_INITIAL = CO(N)%CONC_BOUND
      END DO      
  END IF
  
  DO N=1,MAXNUMCONC
      IF(CO(N)%ALLOCATED)THEN
          CO(N)%CONCN    = CO(N)%CONC
          CO(N)%CONCNM1  = CO(N)%CONC
          CO(N)%DELCONCN = 0.
          CO(N)%DELCONCSTAR = 0.
          CO(N)%DELCONCNM1  = 0.
      END IF
  END DO
  
  CALL DUMP_STATE("initialize_fields")

END SUBROUTINE INITIALIZE_FIELDS

