SUBROUTINE INITIALIZE_GRID()
!
!  The variables FSM, DUM, DVM are initialized.
!  DEPTH is called to set up the vertical distribution of
!  sigma-layers
!  The maximum depth, DEPMAX, in the model region is computed
!  The bottom friction coefficients CBC are computed.
!
  USE STATE
  use norms
  
  IMPLICIT NONE
  REAL AREA
  INTEGER I,J
  
!
!Initialize FSM, DUM and DVM.
!
  FSM = 1.D0
  DUM = 1.D0
  DVM = 1.D0
!
!Define the minimum allowed depth of a sea cell.
!
  DO J = J1,J2
      DO I = I1,I2
          IF(H(I,J) <= 0.0) THEN
              FSM(I,J)=0.
              DUM(I,J)=0.
              DVM(I,J)=0.
          ENDIF
      END DO
  END DO

  CALL TAG_GHOST(H)
  CALL TAG_GHOST(FSM)
  CALL UPDATE_GHOST()
  
  NUMWET = NINT(PARSUM(FSM))
  AREA = NUMWET*DX*DY 
  IF(MASTER) WRITE(*,*) "WET SURFACE AREA =", AREA
  
  IF(JM > 1)THEN
!
!DVM is zero in all V-points with land on one side.
!
      DO J = J_EQ_2,J_eq_JM
          DO I = 1,I_eq_IM
              DVM(I,J)=FSM(I,J)*FSM(I,J-1)
          END DO
      END DO
      IF(BLOCK%JBN==1) DVM(:,1)=FSM(:,1)
  ELSE
      DVM = FSM
  END IF
!
!DUM is zero in all U-points with land on one side.
!
  DO J = 1,J_eq_JM
      DO I = I_EQ_2,I_eq_IM
          DUM(I,J)=FSM(I,J)*FSM(I-1,J)
      END DO
  END DO
  IF(BLOCK%IBN==1 .and. .not. periodic_ew) DUM(1,:)=FSM(1,:)
!
  CALL TAG_GHOST(DVM)
  CALL TAG_GHOST(DUM)
  CALL UPDATE_GHOST
!
!Compute the model static depths H, HU and HV.
!
  H = FSM*H

! dump the bathymetry to a human readable ascii chart with marks for
! land,water,rivers,wells.
  
  CALL DUMPMASK()   

  
  
  DO J = 1,J_EQ_JM
      if(block%ibn==1 .and. .not. periodic_ew)HU(1,J) = H(1,J)*DUM(1,J)
      DO I = I_EQ_2,I_EQ_IM
          HU(I,J) = 0.5*DUM(I,J)*(H(I-1,J)+H(I,J))
      END DO
  END DO
  
  DO I = 1,I_EQ_IM
      if(block%jbn==1)HV(I,1) = H(I,1)*DVM(I,1)
      DO J = J_EQ_2,J_EQ_JM
          if(jm>1)then
              HV(I,J) = 0.5*DVM(I,J)*(H(I,J-1)+H(I,J))
          else
              hv(i,j) = h(i,j)
          end if
      END DO
  END DO
!
  call tag_ghost(HU)
  call tag_ghost(HV)
  call update_ghost

!
!  Define Weights to be used in the Coriolis averaging.
!
  DO I = 1,I_EQ_IM
      DO J = 1,J_EQ_JM
          HVSQR(I,J) = 0.D0
          IF (DVM(I,J) > 0.) THEN
              HVSQR(I,J) = SQRT(HV(I,J))
          ENDIF
          HUSQR(I,J) = 0.D0
          IF (DUM(I,J) > 0.) THEN
              HUSQR(I,J) = SQRT(HU(I,J))
          ENDIF
      END DO
  END DO
!
! COAST is > 0 in all "wet" ETA points that have a "dry"
! neighbour. Coast is the total horisontal length of the cell
! "coastline". if our grid is a lake with one cell, all lateral walls
! should have wall friction.
!
  coast = 0.0
  do j=j_eq_2,jm_mi_1
      do i=i_eq_2,im_mi_1
          if(h(i,j) > 0.) then
              if(h(i-1,j) <= 0.) coast(i,j) = coast(i,j) + dy
              if(h(i+1,j) <= 0.) coast(i,j) = coast(i,j) + dy
              if(jm > 1) then
                  if(h(i,j-1) <= 0.) coast(i,j) = coast(i,j) + dx
                  if(h(i,j+1) <= 0.) coast(i,j) = coast(i,j) + dx
              end if              
          end if
      end do
  end do
  
  call tag_ghost(HUSQR)
  call tag_ghost(HVSQR)
  call tag_ghost(coast)
  call update_ghost
  
  call dump_state("initialize_grid")
  
END SUBROUTINE INITIALIZE_GRID
