SUBROUTINE INTERNAL
!
!  INTERNAL computes the following terms of the
!  moment equations:
!
!   -(g*D**2/RHO0)*Integral from sigma to 0 of
!                  ((dRHO/dx) - (sigma/D)*(dsigma/dx)*(dRHO/dsigma) )
!
!   -(g*D**2/RHO0)*Integral from sigma to 0 of
!                  ((dRHO/dy) - (sigma/D)*(dsigma/dy)*(dRHO/dsigma)
!
!  where g is gravity, D ia dynamic depth
!  and RHO0 reference density,
!  with an explicit technique.
!
!  The results are stored in DELTAU and DELTAV.
!
!  The method is as in Blumberg and Mellor (1987).
!
  USE STATE
  IMPLICIT NONE
  REAL, DIMENSION(I1:I2,J1:J2,KB) :: DRHODX,DRHODY
  REAL, DIMENSION(KB) :: ZZ2(KB)
  REAL REFDENS,XFACTOR,YFACTOR
  INTEGER I,J,K
!
  REFDENS = 1.0 / RHO0
  ZZ2 = 0.
  

  DO K = 2,KB-1
     ZZ2(K) = 0.5*(ZZ(K-1)+ZZ(K))
  END DO
!
!               X COMPONENT OF BAROCLINIC PRESSURE GRADIENT
!
  DRHODX = 0.
  DRHODY = 0.
  
  DO J=J_EQ_2,JM_MI_1
     DO I=I_EQ_2,IM_MI_1
        DRHODX(I,J,1)=GRAV*(-ZZ(1))*DU(I,J)*(RHO(I,J,1)-RHO(I-1,J,1))
     END DO
  END DO
  DO K=2,KB-1
     DO J=J_EQ_2,JM_MI_1
        DO I=I_EQ_2,IM_MI_1
           DRHODX(I,J,K)=DRHODX(I,J,K-1) + GRAV*.5E0*DZZ(K-1)*DU(I,J) &
                &      *(RHO(I,J,K)-RHO(I-1,J,K)+RHO(I,J,K-1)-RHO(I-1,J,K-1)) &
                &      +GRAV*.5E0*ZZ2(K)*(D(I,J)-D(I-1,J)) &
                &      *(RHO(I,J,K)+RHO(I-1,J,K)-RHO(I,J,K-1)-RHO(I-1,J,K-1))
        END DO
     END DO
  END DO
  
  IF(JM > 1) THEN
  
!
!               Y COMPONENT OF BAROCLINIC PRESSURE GRADIENT
!
  DO J=J_EQ_2,JM_MI_1
     DO I=I_EQ_2,IM_MI_1
        DRHODY(I,J,1)=GRAV*(-ZZ(1))*DV(I,J)*(RHO(I,J,1)-RHO(I,J-1,1))
     END DO
  END DO
  DO K=2,KB-1
     DO J=J_EQ_2,JM_MI_1
        DO I=I_EQ_2,IM_MI_1
           DRHODY(I,J,K)=DRHODY(I,J,K-1) +GRAV*.5E0*DZZ(K-1)*DV(I,J) &
                &        *(RHO(I,J,K)-RHO(I,J-1,K)+RHO(I,J,K-1)-RHO(I,J-1,K-1)) &
                &        +GRAV*.5E0*ZZ2(K)*(D(I,J)-D(I,J-1)) &
                &        *(RHO(I,J,K)+RHO(I,J-1,K)-RHO(I,J,K-1)-RHO(I,J-1,K-1)) 
        END DO
     END DO
  END DO
  
  END IF
  
!
!  Include effect of internal pressure.
!
  XFACTOR = REFDENS/DX
  DELTAU = 0.
  DELTAV = 0.
  YFACTOR = REFDENS/DY
  DO K = 1,KB-1
     DO J=J_EQ_2,JM_MI_1
        DO I=I_EQ_2,IM_MI_1
           DELTAU(I,J,K) = - XFACTOR*DRHODX(I,J,K)*DU(I,J)*DUM(I,J)
           DELTAV(I,J,K) = - YFACTOR*DRHODY(I,J,K)*DV(I,J)*DVM(I,J)
        END DO
     END DO
  END DO
!
  CALL DUMP_STATE("internal")
END SUBROUTINE INTERNAL
