SUBROUTINE INTERNAL4
!
!  INTERNAL computes the following terms of the
!  moment equations:
!
!   -(g*D/RHO0)*d(Integral from z to 0 of RHO)/dx
!
!   -(g*D/RHO0)*d(Integral from z to 0 of RHO)/dy
!
!  where g is gravity, D ia dynamic depth
!  and RHO0 reference density,
!  with an explicit technique.
!
!  The method is a 4th order according to McCalpin (1994)
!
USE STATE
IMPLICIT NONE
REAL, DIMENSION(I1:I2,J1:J2,KB) :: DRHODX,DRHODY
REAL, DIMENSION(KB) :: ZZ2(KB)
REAL REFDENS,XFACTOR,YFACTOR
INTEGER I,J,K
REAL DRHO(I1:I2,J1:J2,KB),DDX(I1:I2,J1:J2),RHOU(I1:I2,J1:J2,KB),D4(I1:I2,J1:J2)
!
REFDENS = 1.0 / RHO0
ZZ2 = 0.
DO K = 2,KB-1
  ZZ2(K) = 0.5*(ZZ(K-1)+ZZ(K))
END DO
!
!               X COMPONENT OF BAROCLINIC PRESSURE GRADIENT
!
DRHODX = 0.
DRHODY = 0.
DRHO = 0.
DDX = 0.
RHOU = 0.
D4 = 0.
!
!  Compute DRHO
!
DO K = 1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      DRHO(I,J,K)=(RHO(I,J,K)-RHO(I-1,J,K))
    END DO
  END DO
END DO
DO K = 1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_3,IM_MI_1
      DRHO(I,J,K)=DRHO(I,J,K) - (1./24.)*                   &
      &           (DUM(I+1,J)*(RHO(I+1,J,K)-RHO(I,J,K))-  &
      &            2*(RHO(I,J,K)-RHO(I-1,J,K))+           &
      &            DUM(I-1,J)*(RHO(I-1,J,K)-RHO(I-2,J,K))  )
    END DO
  END DO
END DO
!
!  Compute DDX
!
DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    DDX(I,J)=(D(I,J)-D(I-1,J))
    D4(I,J) = 0.5*(D(I,J)+D(I-1,J))
  END DO
END DO
DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_3,IM_MI_1
    DDX(I,J)=DDX(I,J) - (1./24.)*                   &
      &           (DUM(I+1,J)*(D(I+1,J)-D(I,J))-  &
      &            2*(D(I,J)-D(I-1,J))+           &
      &            DUM(I-1,J)*(D(I-1,J)-D(I-2,J))  )
    D4(I,J) = D4(I,J) + (1./16.)*                   &
      &           (DUM(I+1,J)*(D(I,J)-D(I+1,J))+  &
      &            DUM(I-1,J)*(D(I-1,J)-D(I-2,J))  )
  END DO
END DO
!
!  Compute RHOU
!
DO K = 1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      RHOU(I,J,K)=0.5*(RHO(I,J,K)+RHO(I-1,J,K))
    END DO
  END DO
END DO
DO K = 1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_3,IM_MI_1
      RHOU(I,J,K)=RHOU(I,J,K) + (1./16.)*                   &
      &           (DUM(I+1,J)*(RHO(I,J,K)-RHO(I+1,J,K))+  &
      &            DUM(I-1,J)*(RHO(I-1,J,K)-RHO(I-2,J,K))  )
    END DO
  END DO
END DO
!
!  Integrate vertically contributions to DRHODX
!  Utilizes that int (sigma d(rho)/d(sigma)) dsigma = sigma*rho - int rho dsigma
!  after partial integration
!
DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
   DRHODX(I,J,1)=GRAV*(-ZZ(1))*(D4(I,J)*DRHO(I,J,1)+     &
      &   DDX(I,J)*RHOU(I,J,1) )
  END DO
END DO
DO K=2,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      DRHODX(I,J,K)=DRHODX(I,J,K-1) + GRAV*.5E0*DZZ(K-1)* &
     &      (D4(I,J)*(DRHO(I,J,K-1)+DRHO(I,J,K)) + &
     &       DDX(I,J)*(RHOU(I,J,K-1)+RHOU(I,J,K))  )
    END DO
  END DO
END DO
!
!  Compute direct contribution to DRHODX. from (dD/dx)*sigma*g*RHO
!
DO K=1,KB-1
  DO J=J_eq_2,JM_mi_1
    DO I=I_eq_2,IM_mi_1
      DRHODX(I,J,K)= DRHODX(I,J,K) + DDX(I,J)*ZZ(K)*RHOU(I,J,K)*GRAV
    END DO
  END DO
END DO
!
!               Y COMPONENT OF BAROCLINIC PRESSURE GRADIENT
                                !

IF(JM > 1)THEN

DRHO = 0.
DDX = 0.
RHOU = 0.
D4 = 0.
!
!  Compute DRHO
!
DO K = 1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      DRHO(I,J,K)=(RHO(I,J,K)-RHO(I,J-1,K))
    END DO
  END DO
END DO
DO K = 1,KB-1
  DO J=J_EQ_3,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      DRHO(I,J,K)=DRHO(I,J,K) - (1./24.)*                   &
      &           (DVM(I,J+1)*(RHO(I,J+1,K)-RHO(I,J,K))-  &
      &            2*(RHO(I,J,K)-RHO(I,J-1,K))+           &
      &            DVM(I,J-1)*(RHO(I,J-1,K)-RHO(I,J-2,K))  )
    END DO
  END DO
END DO
!
!  Compute DDX
!
DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
      DDX(I,J)=(D(I,J)-D(I,J-1))
      D4(I,J) = 0.5*(D(I,J)+D(I,J-1))
  END DO
END DO
DO J=J_EQ_3,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    DDX(I,J)=DDX(I,J) - (1./24.)*               &
      &           (DVM(I,J+1)*(D(I,J+1)-D(I,J))-  &
      &            2*(D(I,J)-D(I,J-1))+           &
      &            DVM(I,J-1)*(D(I,J-1)-D(I,J-2)) )
    D4(I,J) = D4(I,J) + (1./16.)*                   &
      &           (DVM(I,J+1)*(D(I,J)-D(I,J+1))+  &
      &            DVM(I,J-1)*(D(I,J-1)-D(I,J-2))  )
  END DO
END DO
!
!  Compute RHOU
!
DO K = 1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      RHOU(I,J,K)=0.5*(RHO(I,J,K)+RHO(I,J-1,K))
    END DO
  END DO
END DO
DO K = 1,KB-1
  DO J=J_EQ_3,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      RHOU(I,J,K)=RHOU(I,J,K) + (1./16.)*                   &
      &           (DVM(I,J+1)*(RHO(I,J,K)-RHO(I,J+1,K))+  &
      &            DVM(I,J-1)*(RHO(I,J-1,K)-RHO(I,J-2,K))  )
    END DO
  END DO
END DO
!
!  Integrate vertically contributions to DRHODY
!
DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
   DRHODY(I,J,1)=GRAV*(-ZZ(1))*(D4(I,J)*DRHO(I,J,1)+     &
      &   DDX(I,J)*RHOU(I,J,1) )
  END DO
END DO
DO K=2,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      DRHODY(I,J,K)=DRHODY(I,J,K-1) + GRAV*.5E0*DZZ(K-1)* &
     &      (D4(I,J)*(DRHO(I,J,K-1)+DRHO(I,J,K)) + &
     &       DDX(I,J)*(RHOU(I,J,K-1)+RHOU(I,J,K))  )
    END DO
  END DO
END DO
!
!  Compute direct contribution to DRHODY. from (dD/dx)*sigma*g*RHO
!
DO K=1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
      DRHODY(I,J,K)= DRHODY(I,J,K) + DDX(I,J)*ZZ(K)*RHOU(I,J,K)*GRAV
    END DO
  END DO
END DO

END IF ! JM > 1

!
!  Include effect of internal pressure.
!
XFACTOR = REFDENS/DX
YFACTOR = REFDENS/DY
  DELTAU = 0.
  DELTAV = 0.
DO K = 1,KB-1
  DO J=J_EQ_2,JM_MI_1
    DO I=I_EQ_2,IM_MI_1
       DELTAU(I,J,K) = - XFACTOR*DRHODX(I,J,K)*DU(I,J)*DUM(I,J)
      DELTAV(I,J,K) = - YFACTOR*DRHODY(I,J,K)*DV(I,J)*DVM(I,J)
    END DO
  END DO
END DO
!
CALL DUMP_STATE("internal4")
END
