MODULE MOD_BOUND
  USE MOD_CONCENTRATION, ONLY : CO, MAXNUMCONC
  IMPLICIT NONE
!
!FRS-zone arrays
!
  INTEGER :: LB=-1
  INTEGER, PARAMETER :: MAXLB=100
  LOGICAL CLOSED
!
  REAL, DIMENSION(:),POINTER :: ALPHA,ALPHAE 
!
! River values.
!
  LOGICAL :: RIVLOG=.false.
  INTEGER :: NUMC=0
  INTEGER, PARAMETER :: MAXNUMC=100
  INTEGER, DIMENSION(:),POINTER :: IRIVER,JRIVER,KRIVER1,KRIVER2,CODERIVER,RIVERFORMAT
  REAL, DIMENSION(:),POINTER :: SRIVER,TRIVER,FRIVER,DZRIVER,DEPRIVER
  REAL, DIMENSION(:,:),POINTER :: CRIVER
  CHARACTER(50), POINTER :: RIVER_NAME(:)

!
! Variables used in the case of fresh water inputs through the bottom, like
! with a dived fresh water source.
!
  LOGICAL :: UPWELL=.false.
  INTEGER :: NUMWELL=0
  INTEGER, PARAMETER :: MAXNUMWELL=50
  INTEGER, DIMENSION(:),POINTER :: IWELL,JWELL,WELL_FORMAT
  REAL, DIMENSION(:),POINTER :: SWELL,TWELL,FWELL
  REAL, DIMENSION(:,:),POINTER :: CWELL
  CHARACTER(50), POINTER :: WELL_NAME(:)

!
! Tidal variables
!
  LOGICAL :: TIDE=.false.
  LOGICAL :: TIDE_FAC=.false.
  INTEGER :: KCON=0
  INTEGER, PARAMETER :: MAXKCON = 10
  REAL,POINTER :: FREQ(:)
  REAL, DIMENSION(:,:,:),POINTER :: ETATAMP,ETATPHA,UATAMP, &
       & UATPHA,VATAMP,VATPHA 
!
!  Atmospheric forcing variables
!
  LOGICAL :: WIND_FORCING=.false., ATMPRESS=.false.
  INTEGER :: WIND_TYPE=0
  REAL, DIMENSION(:,:),POINTER :: WSPEEDU,WSPEEDV


  
!!!! Then, some additional things related to the new setup system.

  
  
!! parameters for the wind1.f90 routine:
  
  REAL :: WIND1_DIRECTION=0.0, WIND1_HOURLY_ROTATION=0.0, WIND1_AMPLITUDE=0.0, WIND1_PERIOD=0.0, WIND1_RAMPUP_TIME=0.0
 
!! parameters for the wind2.f90 routine: real :: wind2_strength=0.0,
  
  REAL :: WIND2_RADIUS=0.0, WIND2_STARTX=0.0, WIND2_STARTY=0.0, WIND2_CENTER_SPEED=0.0, WIND2_CENTER_DIRECTION=0.0, &
       WIND2_RAMPUP_TIME=0.0, wind2_strength
  
!! wind3:
  
  character(200) :: wind3_data_file=" "


!! precomputed relaxation arrays
  real, dimension(:,:), pointer :: alfa_u, alfa_v, alfa_eta, alfae_u, alfae_v, alfae_eta

!! the arrays containing boundary values ready to be applied.  
  real, dimension(:,:,:), pointer :: u_bound, v_bound, w_bound, s_bound, t_bound, q2_bound, q2l_bound, rho_bound
  real, dimension(:,:), pointer :: ua_bound, va_bound, eta_bound, etabfrs,etatfrs

!! the arrays containing tidal values ready to be applied.  
  real, dimension(:,:), pointer :: ua_tide, va_tide, eta_tide

!! the bin region methods requires the initial/external state of the boundary arrays to be stored
  real, dimension(:,:,:), pointer :: s_initial, t_initial, rho_initial
  
  TYPE BOUNDARY_OBJECT
     
!! true if present on the current mpi block
     logical :: inblock=.false.
     
!! the global range is the "good old" one, the local is the equivalent
!! processor local range
     integer :: global_range(4)=-1, local_range(4)=-1

!! edge: (use integer values from mod_mpi)
     integer :: edge=-1
     
!! which zone on the opposite side to "communicate" with for periodic bc     
     integer :: periodic=-1

!! for each variable in this zone, what type of bc to apply     
     integer :: u_type=0, v_type=0, w_type=0, s_type=0, &
          t_type=0, ua_type=0, va_type=0, eta_type=0, q2_type=0, &
          q2l_type=0, rho_type=0, conc_type(maxnumconc)=0
     
!! book keeping too see if a boundary array has been initialized properly
     logical :: u_set=.false., v_set=.false., w_set=.false., &
          s_set=.false., t_set=.false., ua_set=.false., &
          va_set=.false., eta_set=.false., q2_set=.false., &
          q2l_set=.false., rho_set=.false., conc_set(maxnumconc)=.false.
  END TYPE BOUNDARY_OBJECT

  
!! the maximum allowed number of zones along each edge
  integer, parameter :: max_zones=5

!! the present number of open boundaries along each edge
  integer :: num_zones_east=0, num_zones_west=0, num_zones_south=0, num_zones_north=0
  
!! the lists of boundary objects for each edge
  type(boundary_object), save, dimension(max_zones) :: boeast,bowest,bonorth,bosouth
  
!! if boundary zones for two edges are connected in a corner, a flag is set:
  logical :: sw_connected=.false., nw_connected=.false., ne_connected=.false., se_connected=.false.
  
!! a list with names of bc's to be used to prettify output.  
  character(10), dimension(-1:4) :: bcname=(/"user_bound","frs       ","neuman    ","periodic  ","dirichlet ","frs binreg"/)
 


!! true if at least one bin region is active
  logical :: binreg_set=.false.

  
!! max number of bin z levels and bin regions
  integer, parameter :: maxnumbin=150

  TYPE BINREG_OBJECT
     integer :: numwet = 0                                 ! the number of wet cells in a bin region
     integer :: range(4) = -1                              ! global horizontal range for bin
     integer :: local_range(4) = -1                        ! the local piece of the range
     logical :: inblock=.false.                            ! false if no piece is owned by current processor
     logical :: initialized=.false.                        ! true when bin levels are set
     integer :: levels=20                                  ! the number of bin layers
     integer :: edge=-1                                    ! which edge do we belong to (value east,west... from mod_mpi)
     real :: depmax=0.0                                    ! max depth with more than 5 samples from pzsum
     real zlevels(maxnumbin+1), zinn(maxnumbin)            ! the z levels, and "zz" levels
     real, dimension(maxnumbin) :: ubclinz,vbclinz,sz,tz   ! the average profiles for the region
  END TYPE BINREG_OBJECT
  
!! one bin for each boundary zone
  type(binreg_object), dimension(max_zones), save :: bin_north, bin_east, bin_south, bin_west


  
!! Tides
  
!! the number of constituents for each zone
  integer, dimension(max_zones) :: kcon_west=0, kcon_east=0, kcon_south=0, kcon_north=0

!! zone specific type of tidal forcing, 0=constant, 1=from fields.
  integer, dimension(max_zones) :: tidetype_west=0, tidetype_east=0, tidetype_south=0, tidetype_north=0

!! zone specific constituents
  real, dimension(max_zones, maxkcon) :: freq_west, freq_east, freq_south, freq_north

  
contains


  subroutine bound2(idx)
!  
! iterates over all boundary zones and calls appropriate routines to
! transfer boundary values from the xx_bound arrays to the
! corresponding variable to be forced. idx selects which group of
! variables to update, in the same manner as the old bound.f90:
!
!   idx=1  ua,va,eta
!   idx=3  u,v,w
!   idx=4  s,t
!   idx=6  q2, q2l
!   idx=7  uastar, vastar, etcstar   
!   idx=8  uaf, vaf, etaf    
!   idx=9  eta
!
! TODO: we may need to handle u points east, and v points north in a special way.
!
    use state
    use control_parameters
    use mod_modesplit
    
    implicit none
    integer zone, idx, i
    character(2) char_idx
    
    do zone=1, num_zones_west
        select case(idx)
        case(3)
            ! if type is 4, prefetch average of u from bin and place in u_bound
            if(bowest(zone)%u_type == 4) call get_u_bound(bin_west(zone), bowest(zone))
            
            call west_selector(idx, zone, u_point,   u,   u_bound,   bowest(zone)%u_type, kb)

! for now, we only fetch u for east and west, since v_bound is supposed to
! be 0. this is obviously not always true, but the bin region trick
! problably only works for open boundaries where the flow direction is
! normal to the boundary.           
!            if(bowest(zone)%v_type == 4) call get_v_bound(bin_west(zone), bowest(zone))            
            call west_selector(idx, zone, v_point,   v,   v_bound,   bowest(zone)%v_type, kb)            
            call west_selector(idx, zone, eta_point, w,   w_bound,   bowest(zone)%w_type, kb)
        case(4)
            if(any( (/bowest(zone)%s_type,bowest(zone)%t_type,bowest(zone)%rho_type/) == 4)) &
                 call get_st_bound(bin_west(zone), bowest(zone))
            
            if(advect_density)then
                call west_selector(idx, zone, eta_point, rho,   rho_bound,   bowest(zone)%rho_type, kb)
            else
                call west_selector(idx, zone, eta_point, s,   s_bound,   bowest(zone)%s_type, kb)
                call west_selector(idx, zone, eta_point, t,   t_bound,   bowest(zone)%t_type, kb)
            end if
        case(1,9)
            if(bowest(zone)%eta_type == 4) call get_eta_bound(eta, eta_bound, bin_west(zone), bowest(zone)) 
            call west_selector(idx, zone, eta_point, eta, eta_bound, bowest(zone)%eta_type, 1, f_tide=eta_tide)
            if(idx.ne.9)then
                if(bowest(zone)%ua_type == 4) &
                     call get_va_bound(ua, eta, ua_bound, bin_west(zone), bowest(zone))
                call west_selector(idx, zone, u_point,   ua,  ua_bound,  bowest(zone)%ua_type, 1, f_tide=ua_tide)
                call west_selector(idx, zone, v_point,   va,  va_bound,  bowest(zone)%va_type, 1, f_tide=va_tide)
            end if
        case(6)
            call west_selector(idx, zone, eta_point, q2,  q2_bound,  bowest(zone)%q2_type, kb)
            call west_selector(idx, zone, eta_point, q2l, q2l_bound, bowest(zone)%q2l_type, kb)
        case(7)
            if(bowest(zone)%eta_type == 4) call get_eta_bound(etastar, eta_bound, bin_west(zone), bowest(zone)) 
            call west_selector(idx, zone, eta_point, etastar, eta_bound, bowest(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(bowest(zone)%ua_type == 4) call get_va_bound(uastar, eta, ua_bound, bin_west(zone), bowest(zone))
            call west_selector(idx, zone, u_point,   uastar,  ua_bound,  bowest(zone)%ua_type, 1, f_tide=ua_tide)
            call west_selector(idx, zone, v_point,   vastar,  va_bound,  bowest(zone)%va_type, 1, f_tide=va_tide)            
        case(8)
            if(bowest(zone)%eta_type == 4) call get_eta_bound(etaf, eta_bound, bin_west(zone), bowest(zone)) 
            call west_selector(idx, zone, eta_point, etaf, eta_bound, bowest(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(bowest(zone)%ua_type == 4) call get_va_bound(uaf, eta, ua_bound, bin_west(zone), bowest(zone))
            call west_selector(idx, zone, u_point,   uaf,  ua_bound,  bowest(zone)%ua_type, 1, f_tide=ua_tide)
            call west_selector(idx, zone, v_point,   vaf,  va_bound,  bowest(zone)%va_type, 1, f_tide=va_tide)
        case(10)
            do i=1, maxnumconc
                if(co(i)%active) then
                    if(bowest(zone)%conc_type(i) == 4) call get_conc_bound(i, bin_west(zone), bowest(zone))
                    call west_selector(idx,zone,eta_point,co(i)%conc,co(i)%conc_bound,bowest(zone)%conc_type(i),kb)
                end if
            end do
        end select
    end do

    do zone=1, num_zones_east        
        select case(idx)
        case(3)      
            if(boeast(zone)%u_type == 4) call get_u_bound(bin_east(zone), boeast(zone))
            call east_selector(idx, zone, u_point,    u,   u_bound,   boeast(zone)%u_type, kb)
            call east_selector(idx, zone, v_point,    v,   v_bound,   boeast(zone)%v_type, kb)
            call east_selector(idx, zone, eta_point,  w,   w_bound,   boeast(zone)%w_type, kb)
        case(4)                                                        
            if(any((/boeast(zone)%s_type,boeast(zone)%t_type,boeast(zone)%rho_type/) == 4)) &
                 call get_st_bound(bin_east(zone), boeast(zone))

            if(advect_density)then
                call east_selector(idx, zone, eta_point,  rho,   rho_bound,   boeast(zone)%rho_type, kb)
            else
                call east_selector(idx, zone, eta_point,  s,   s_bound,   boeast(zone)%s_type, kb)
                call east_selector(idx, zone, eta_point,  t,   t_bound,   boeast(zone)%t_type, kb)
            end if
        case(1,9)
            if(boeast(zone)%eta_type == 4) call get_eta_bound(eta, eta_bound, bin_east(zone), boeast(zone)) 
            call east_selector(idx, zone, eta_point,  eta, eta_bound, boeast(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(idx.ne.9)then
                if(boeast(zone)%ua_type==4) &
                     call get_va_bound(ua, eta, ua_bound, bin_east(zone), boeast(zone))
                call east_selector(idx, zone, u_point,    ua,  ua_bound,  boeast(zone)%ua_type, 1, f_tide=ua_tide)
                call east_selector(idx, zone, v_point,    va,  va_bound,  boeast(zone)%va_type, 1, f_tide=va_tide)
            end if            
        case(6)
            call east_selector(idx, zone, eta_point,  q2,  q2_bound,  boeast(zone)%q2_type, kb)
            call east_selector(idx, zone, eta_point,  q2l, q2l_bound, boeast(zone)%q2l_type, kb)
        case(7)
            if(boeast(zone)%eta_type == 4) call get_eta_bound(etastar, eta_bound, bin_east(zone), boeast(zone)) 
            call east_selector(idx, zone, eta_point, etastar, eta_bound, boeast(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(boeast(zone)%ua_type==4) call get_va_bound(uastar, eta, ua_bound, bin_east(zone), boeast(zone))
            call east_selector(idx, zone, u_point,   uastar,  ua_bound,  boeast(zone)%ua_type, 1, f_tide=ua_tide)
            
            call east_selector(idx, zone, v_point,   vastar,  va_bound,  boeast(zone)%va_type, 1, f_tide=va_tide)            
        case(8)
            if(boeast(zone)%eta_type == 4) call get_eta_bound(etaf, eta_bound, bin_east(zone), boeast(zone)) 
            call east_selector(idx, zone, eta_point, etaf, eta_bound, boeast(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(boeast(zone)%ua_type == 4) call get_va_bound(uaf, eta, ua_bound, bin_east(zone), boeast(zone))
            call east_selector(idx, zone, u_point,   uaf,  ua_bound,  boeast(zone)%ua_type, 1, f_tide=ua_tide)
            
            call east_selector(idx, zone, v_point,   vaf,  va_bound,  boeast(zone)%va_type, 1, f_tide=va_tide)            
        case(10)
            do i=1, maxnumconc
                if(co(i)%active) then
                    if(boeast(zone)%conc_type(i) == 4) call get_conc_bound(i, bin_east(zone), boeast(zone))
                    call east_selector(idx,zone,eta_point,co(i)%conc,co(i)%conc_bound,boeast(zone)%conc_type(i),kb)
                end if
            end do
        end select
    end do
    
    do zone=1, num_zones_north
        select case(idx)
        case(3)
            if(bonorth(zone)%v_type==4) call get_v_bound(bin_north(zone), bonorth(zone))
            
            call north_selector(idx, zone, u_point,    u,   u_bound,   bonorth(zone)%u_type, kb)            
            call north_selector(idx, zone, v_point,    v,   v_bound,   bonorth(zone)%v_type, kb)            
            call north_selector(idx, zone, eta_point,  w,   w_bound,   bonorth(zone)%w_type, kb)
        case(4)
            if(any((/bonorth(zone)%s_type,bonorth(zone)%t_type,bonorth(zone)%rho_type/) == 4)) &
                 call get_st_bound(bin_north(zone), bonorth(zone))
            if(advect_density)then
                call north_selector(idx, zone, eta_point,  rho,   rho_bound,   bonorth(zone)%rho_type, kb)
            else
                call north_selector(idx, zone, eta_point,  s,   s_bound,   bonorth(zone)%s_type, kb)
                call north_selector(idx, zone, eta_point,  t,   t_bound,   bonorth(zone)%t_type, kb)
            end if
        case(1,9)
            if(bonorth(zone)%eta_type==4) call get_eta_bound(eta, eta_bound, bin_north(zone), bonorth(zone)) 
            call north_selector(idx, zone, eta_point,  eta, eta_bound, bonorth(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(idx.ne.9)then
                if(bonorth(zone)%va_type==4) call get_va_bound(va, eta, va_bound, bin_north(zone), bonorth(zone))
                
                call north_selector(idx, zone, u_point, ua,  ua_bound,  bonorth(zone)%ua_type, 1, f_tide=ua_tide)
                call north_selector(idx, zone, v_point, va,  va_bound,  bonorth(zone)%va_type, 1, f_tide=va_tide)
            end if
        case(6)
            call north_selector(idx, zone, eta_point,  q2,  q2_bound,  bonorth(zone)%q2_type, kb)
            call north_selector(idx, zone, eta_point,  q2l, q2l_bound, bonorth(zone)%q2l_type, kb)
        case(7)
            if(bonorth(zone)%eta_type==4) call get_eta_bound(etastar, eta_bound, bin_north(zone), bonorth(zone)) 
            call north_selector(idx, zone, eta_point, etastar, eta_bound, bonorth(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(bonorth(zone)%va_type==4) call get_va_bound(vastar, etastar, va_bound, bin_north(zone), bonorth(zone))
            
            call north_selector(idx, zone, u_point,   uastar,  ua_bound,  bonorth(zone)%ua_type, 1, f_tide=ua_tide)
            call north_selector(idx, zone, v_point,   vastar,  va_bound,  bonorth(zone)%va_type, 1, f_tide=va_tide)
        case(8)
            if(bonorth(zone)%eta_type==4) call get_eta_bound(etaf, eta_bound, bin_north(zone), bonorth(zone)) 
            call north_selector(idx, zone, eta_point, etaf, eta_bound, bonorth(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(bonorth(zone)%va_type==4) call get_va_bound(vaf, etaf, va_bound, bin_north(zone), bonorth(zone))
            
            call north_selector(idx, zone, u_point,   uaf,  ua_bound,  bonorth(zone)%ua_type, 1, f_tide=ua_tide)
            call north_selector(idx, zone, v_point,   vaf,  va_bound,  bonorth(zone)%va_type, 1, f_tide=va_tide)
        case(10)
            do i=1, maxnumconc
                if(co(i)%active) then
                    if(bonorth(zone)%conc_type(i) == 4) call get_conc_bound(i, bin_north(zone), bonorth(zone))
                    call north_selector(idx,zone,eta_point,co(i)%conc,co(i)%conc_bound,bonorth(zone)%conc_type(i),kb)
                end if
            end do
        end select
    end do
    
    do zone=1, num_zones_south
        select case(idx)
        case(3)
            if(bosouth(zone)%v_type==4) call get_v_bound(bin_south(zone), bosouth(zone))
            
            call south_selector(idx, zone, u_point,    u,   u_bound,   bosouth(zone)%u_type, kb)
            call south_selector(idx, zone, v_point,    v,   v_bound,   bosouth(zone)%v_type, kb)
            call south_selector(idx, zone, eta_point,  w,   w_bound,   bosouth(zone)%w_type, kb)
        case(4)
            if(any((/bosouth(zone)%s_type,bosouth(zone)%t_type,bosouth(zone)%rho_type/) == 4)) &
                 call get_st_bound(bin_south(zone), bosouth(zone))                
            if(advect_density)then
                call south_selector(idx, zone, eta_point,  rho,   rho_bound,   bosouth(zone)%rho_type, kb)
            else
                call south_selector(idx, zone, eta_point,  s,   s_bound,   bosouth(zone)%s_type, kb)
                call south_selector(idx, zone, eta_point,  t,   t_bound,   bosouth(zone)%t_type, kb)
            end if
        case(1,9)
            if(bosouth(zone)%eta_type==4) call get_eta_bound(eta, eta_bound, bin_south(zone), bosouth(zone)) 
            call south_selector(idx, zone, eta_point,  eta, eta_bound, bosouth(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(idx.ne.9)then
                if(bosouth(zone)%va_type==4) call get_va_bound(va, eta, va_bound, bin_south(zone), bosouth(zone))
                
                call south_selector(idx, zone, u_point,    ua,  ua_bound,  bosouth(zone)%ua_type, 1, f_tide=ua_tide)
                call south_selector(idx, zone, v_point,    va,  va_bound,  bosouth(zone)%va_type, 1, f_tide=va_tide)
            end if
        case(6)
            call south_selector(idx, zone, eta_point,  q2,  q2_bound,  bosouth(zone)%q2_type, kb)
            call south_selector(idx, zone, eta_point,  q2l, q2l_bound, bosouth(zone)%q2l_type, kb)
        case(7)
            if(bosouth(zone)%eta_type==4) call get_eta_bound(etastar, eta_bound, bin_south(zone), bosouth(zone)) 
            call south_selector(idx, zone, eta_point, etastar, eta_bound, bosouth(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(bosouth(zone)%va_type==4) call get_va_bound(vastar, etastar, va_bound, bin_south(zone), bosouth(zone))
            
            call south_selector(idx, zone, u_point,   uastar,  ua_bound,  bosouth(zone)%ua_type, 1, f_tide=ua_tide)
            call south_selector(idx, zone, v_point,   vastar,  va_bound,  bosouth(zone)%va_type, 1, f_tide=va_tide)            
        case(8)
            if(bosouth(zone)%eta_type==4) call get_eta_bound(etaf, eta_bound, bin_south(zone), bosouth(zone)) 
            call south_selector(idx, zone, eta_point, etaf, eta_bound, bosouth(zone)%eta_type, 1, f_tide=eta_tide)
            
            if(bosouth(zone)%va_type==4) call get_va_bound(vaf, etaf, va_bound, bin_south(zone), bosouth(zone))
            
            call south_selector(idx, zone, u_point,   uaf,  ua_bound,  bosouth(zone)%ua_type, 1, f_tide=ua_tide)
            call south_selector(idx, zone, v_point,   vaf,  va_bound,  bosouth(zone)%va_type, 1, f_tide=va_tide)            
        case(10)
            do i=1, maxnumconc
                if(co(i)%active) then
                    if(bosouth(zone)%conc_type(i) == 4) call get_conc_bound(i, bin_south(zone), bosouth(zone))
                    call south_selector(idx,zone,eta_point,co(i)%conc,co(i)%conc_bound,bosouth(zone)%conc_type(i),kb)
                end if
            end do
        end select
    end do

    write(char_idx,'(i0)') idx
    call dump_state("bound2_idx"//trim(char_idx))
  end subroutine bound2


  
  
  subroutine west_selector(idx, zone, pointtype, f, f_bound, bctype, kdim, f_tide)
!
!  in ZONE, apply boundary handling method BCTYPE, to F.
!
    use state
    use mod_modesplit, only : etastar, etaf
    implicit none
    integer idx, zone, pointtype, bctype, kdim
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)

    select case(bctype)
    case(-1)
        call user_bound('west', idx)
    case(0,4)
        !! for frs zones, we need to send the correct
        !! alfas. barotropic fields must have alfae's, u,v and eta points
        !! also need separate alpha arrays.
        
        if(kdim > 1) then
            select case(pointtype)
            case(u_point)
                call frs_west(alfa_u, f, f_bound, zone, kdim, f_tide)
            case(v_point)
                call frs_west(alfa_v, f, f_bound, zone, kdim, f_tide)            
            case(eta_point)
                call frs_west(alfa_eta, f, f_bound, zone, kdim, f_tide)            
            end select
        else
            select case(pointtype)
            case(u_point)
                call frs_west(alfae_u, f, f_bound, zone, kdim, f_tide)
            case(v_point)
                call frs_west(alfae_v, f, f_bound, zone, kdim, f_tide)            
            case(eta_point)
                call frs_west(alfae_eta, f, f_bound, zone, kdim, f_tide)            
            end select
        end if
    case(1)
        call neuman_west(f, zone, kdim)
    case(2)
        call periodic_west(f, zone, kdim)
    case(3)
        call dirichlet_west(f, f_bound, zone, kdim, f_tide)
    case(5)
        select case(idx)
        case(1)
            call flather_west(f, f_bound, eta, zone)
        case(7)
            call flather_west(f, f_bound, etastar, zone)
        case(8)
            call flather_west(f, f_bound, etaf, zone)
        case default
            call abort("BC 5 allowed only for 2d")
        end select
    end select
  end subroutine west_selector

  
  subroutine east_selector(idx, zone, pointtype, f, f_bound, bctype, kdim, f_tide)
!
!  in ZONE, apply boundary handling method BCTYPE, to F.
!
    use state
    use mod_modesplit, only : etastar, etaf
    implicit none
    integer idx, zone, pointtype, bctype, kdim
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)

    select case(bctype)
    case(-1)
        call user_bound('east', idx)
    case(0,4)
        if(kdim > 1) then
            select case(pointtype)
            case(u_point)
                call frs_east(alfa_u, f, f_bound, zone, kdim, f_tide)            
            case(v_point)
                call frs_east(alfa_v, f, f_bound, zone, kdim, f_tide)                        
            case(eta_point)
                call frs_east(alfa_eta, f, f_bound, zone, kdim, f_tide)                        
            end select
        else
            select case(pointtype)
            case(u_point)
                call frs_east(alfae_u, f, f_bound, zone, kdim, f_tide)            
            case(v_point)
                call frs_east(alfae_v, f, f_bound, zone, kdim, f_tide)                        
            case(eta_point)
                call frs_east(alfae_eta, f, f_bound, zone, kdim, f_tide)                        
            end select
        end if
    case(1)
        call neuman_east(f, zone, kdim)
    case(2)
        call periodic_east(f, zone, kdim)
    case(3)
        call dirichlet_east(f, f_bound, zone, kdim, f_tide)
    case(5)
        select case(idx)
        case(1)
            call flather_east(f, f_bound, eta, zone)
        case(7)
            call flather_east(f, f_bound, etastar, zone)
        case(8)
            call flather_east(f, f_bound, etaf, zone)
        case default
            call abort("BC 5 allowed only for 2d")
        end select
    end select
  end subroutine east_selector

  
  subroutine south_selector(idx, zone, pointtype, f, f_bound, bctype, kdim, f_tide)
!
!  in ZONE, apply boundary handling method BCTYPE, to F.
!
    use state
    use mod_modesplit, only : etastar, etaf
    implicit none
    integer idx, zone, pointtype, bctype, kdim
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)

    select case(bctype)
    case(-1)
        call user_bound('south', idx)
    case(0,4)
        if(kdim > 1) then
            select case(pointtype)
            case(u_point)
                call frs_south(alfa_u, f, f_bound, zone, kdim, f_tide)            
            case(v_point)
                call frs_south(alfa_v, f, f_bound, zone, kdim, f_tide)            
            case(eta_point)
                call frs_south(alfa_eta, f, f_bound, zone, kdim, f_tide)            
            end select
        else
            select case(pointtype)
            case(u_point)
                call frs_south(alfae_u, f, f_bound, zone, kdim, f_tide)            
            case(v_point)
                call frs_south(alfae_v, f, f_bound, zone, kdim, f_tide)            
            case(eta_point)
                call frs_south(alfae_eta, f, f_bound, zone, kdim, f_tide)            
            end select
        end if
    case(1)
        call neuman_south(f, zone, kdim)
    case(2)
        call periodic_south(f, zone, kdim)
    case(3)
        call dirichlet_south(f, f_bound, zone, kdim, f_tide)        
    case(5)
        select case(idx)
        case(1)
            call flather_south(f, f_bound, eta, zone)
        case(7)
            call flather_south(f, f_bound, etastar, zone)
        case(8)
            call flather_south(f, f_bound, etaf, zone)
        case default
            call abort("BC 5 allowed only for 2d")
        end select
    end select
  end subroutine south_selector

  
  subroutine north_selector(idx, zone, pointtype, f, f_bound, bctype, kdim, f_tide)
!
!  in ZONE, apply boundary handling method BCTYPE, to F.
!
    use state
    use mod_modesplit, only : etastar, etaf
    implicit none
    integer idx, zone, pointtype, bctype, kdim
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)

    select case(bctype)        
    case(-1)
        call user_bound('north', idx)
    case(0,4)
        if(kdim > 1) then
            select case(pointtype)
            case(u_point)
                call frs_north(alfa_u, f, f_bound, zone, kdim, f_tide)
            case(v_point)
                call frs_north(alfa_v, f, f_bound, zone, kdim, f_tide)
            case(eta_point)
                call frs_north(alfa_eta, f, f_bound, zone, kdim, f_tide)
            end select
        else
            select case(pointtype)
            case(u_point)
                call frs_north(alfae_u, f, f_bound, zone, kdim, f_tide)
            case(v_point)
                call frs_north(alfae_v, f, f_bound, zone, kdim, f_tide)
            case(eta_point)
                call frs_north(alfae_eta, f, f_bound, zone, kdim, f_tide)
            end select
        end if
    case(1)
        call neuman_north(f, zone, kdim)
    case(2)
        call periodic_north(f, zone,  kdim)
    case(3)
        call dirichlet_north(f, f_bound, zone, kdim, f_tide)
    case(5)
        select case(idx)
        case(1)
            call flather_north(f, f_bound, eta, zone)
        case(7)
            call flather_north(f, f_bound, etastar, zone)
        case(8)
            call flather_north(f, f_bound, etaf, zone)
        case default
            call abort("BC 5 allowed only for 2d")
        end select
    end select
  end subroutine north_selector




! the routines that apply the bc comes next. as input we have the
! field, the field with the boundary value, the range along the edge
! and the k dimension of the field to make it possible to use routine
! for 2d and 3d fields. the edge to apply the bc to is of course
! implicitly given in the routine name.

  subroutine neuman_west(f, zone, kdim)
    use state, only : i1,i2,j1,j2
    implicit none
    integer zone,r(4),kdim,k,j
    real f(i1:i2,j1:j2,kdim)
    
    if(.not. bowest(zone)%inblock) return
    r = bowest(zone)%local_range
    
    do k=1,kdim
        do j=r(3),r(4)
            f(1,j,k) = f(2,j,k)
        end do
    end do
  end subroutine neuman_west

  subroutine neuman_east(f, zone, kdim)
    use state, only : i1,i2,j1,j2,i_eq_im
    implicit none
    integer zone,r(4),kdim,k,j
    real f(i1:i2,j1:j2,kdim)
    
    if(.not.boeast(zone)%inblock) return
    r = boeast(zone)%local_range
    
    do k=1,kdim
        do j=r(3),r(4)
            f(i_eq_im,j,k) = f(i_eq_im-1,j,k)
        end do
    end do
   end subroutine neuman_east

  subroutine neuman_south(f, zone, kdim)
    use state, only : i1,i2,j1,j2
    implicit none
    integer zone,r(4),kdim,k,i
    real f(i1:i2,j1:j2,kdim)
    
    if(.not.bosouth(zone)%inblock) return
    r = bosouth(zone)%local_range
    
    do k=1,kdim
        do i=r(1),r(2)
            f(i,1,k) = f(i,2,k)
        end do
    end do
  end subroutine neuman_south

  subroutine neuman_north(f, zone, kdim)
    use state, only : i1,i2,j1,j2,j_eq_jm
    implicit none
    integer zone, r(4),kdim,k,i
    real f(i1:i2,j1:j2,kdim)
    
    if(.not.bonorth(zone)%inblock) return
    r = bonorth(zone)%local_range
        
    do k=1,kdim
        do i=r(1),r(2)
            f(i,j_eq_jm,k) = f(i,j_eq_jm-1,k)
        end do
    end do
  end subroutine neuman_north

! -----------------------------------------------------------------

  subroutine dirichlet_west(f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2
    implicit none
    integer zone,r(4),kdim,j,k
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    
    if(.not. bowest(zone)%inblock) return
    r = bowest(zone)%local_range

    if(present(f_tide))then
        do k=1,kdim
            do j=r(3),r(4)
                f(1,j,k) = f_bound(1,j,k) + f_tide(1,j,k)
            end do
        end do
    else
        do k=1,kdim
            do j=r(3),r(4)
                f(1,j,k) = f_bound(1,j,k)
            end do
        end do
    end if
  end subroutine dirichlet_west

  subroutine dirichlet_east(f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2,i_eq_im
    implicit none
    integer zone,r(4),kdim,j,k
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    
    if(.not.boeast(zone)%inblock) return
    r = boeast(zone)%local_range
    
    if(present(f_tide))then
        do k=1,kdim
            do j=r(3),r(4)
                f(i_eq_im,j,k) = f_bound(i_eq_im,j,k) + f_tide(i_eq_im,j,k)
            end do
        end do
    else
        do k=1,kdim
            do j=r(3),r(4)
                f(i_eq_im,j,k) = f_bound(i_eq_im,j,k)
            end do
        end do
    end if
  end subroutine dirichlet_east

  subroutine dirichlet_south(f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2
    implicit none
    integer zone,r(4),kdim,i,k
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    
    if(.not.bosouth(zone)%inblock) return
    r = bosouth(zone)%local_range
    
    if(present(f_tide))then
        do k=1,kdim
            do i=r(1),r(2)
                f(i,1,k) = f_bound(i,1,k) + f_tide(i,1,k)
            end do
        end do
    else
        do k=1,kdim
            do i=r(1),r(2)
                f(i,1,k) = f_bound(i,1,k)
            end do
        end do
    end if
  end subroutine dirichlet_south

  subroutine dirichlet_north(f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2,j_eq_jm
    implicit none
    integer zone,r(4),kdim,i,k
    real f(i1:i2,j1:j2,kdim), f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    
    if(.not.bonorth(zone)%inblock) return
    r = bonorth(zone)%local_range
    
    if(present(f_tide))then
        do k=1,kdim
            do i=r(1),r(2)
                f(i,j_eq_jm,k) = f_bound(i,j_eq_jm,k) + f_tide(i,j_eq_jm,k)
            end do
        end do
    else
        do k=1,kdim
            do i=r(1),r(2)
                f(i,j_eq_jm,k) = f_bound(i,j_eq_jm,k)
            end do
        end do
    end if
  end subroutine dirichlet_north
  
!-------------------------------------------------------------------------

  subroutine flather_west(f, f_bound, elev, zone)
    use state, only : i1,i2,j1,j2
    implicit none
    real f(i1:i2,j1:j2,1), f_bound(i1:i2,j1:j2,1), elev(i1:i2,j1:j2,1)
    integer zone, r(4), j

    if(.not. bowest(zone)%inblock) return
    r = bowest(zone)%local_range

    do j=r(3),r(4)
!   following dinnimann&kinck, something like
!       v internal = v external(=0)  +                              eta internal          eta external
!        f(2,j,1) =                      sqrt(grav/du(2,j))*( 0.5*(elev(1,j)+elev(2,j)) -               )
!        f(1,j,1) = f(2,j,1)        
    end do
    
  end subroutine flather_west

  subroutine flather_east(f, f_bound, elev, zone)
    use state, only : i1,i2,j1,j2
    implicit none
    real f(i1:i2,j1:j2,1), f_bound(i1:i2,j1:j2,1), elev(i1:i2,j1:j2,1)
    integer zone, r(4)

    if(.not. boeast(zone)%inblock) return
    r = boeast(zone)%local_range
!    ...
  end subroutine flather_east
  
  subroutine flather_north(f, f_bound, elev, zone)
    use state, only : i1,i2,j1,j2
    implicit none
    real f(i1:i2,j1:j2,1), f_bound(i1:i2,j1:j2,1), elev(i1:i2,j1:j2,1)
    integer zone, r(4)

    if(.not. bonorth(zone)%inblock) return
    r = bonorth(zone)%local_range
!   ...    
  end subroutine flather_north
  
  subroutine flather_south(f, f_bound, elev, zone)
    use state, only : i1,i2,j1,j2
    implicit none
    real f(i1:i2,j1:j2,1), f_bound(i1:i2,j1:j2,1), elev(i1:i2,j1:j2,1)
    integer zone, r(4)

    if(.not. bosouth(zone)%inblock) return
    r = bosouth(zone)%local_range
!  ...
  end subroutine flather_south

  
!-------------------------------------------------------------------------

  subroutine periodic_west(f, zone, kdim)
!! note that js,je etc here must be given as global coordinates in
!! order for copy_region to work!
    use state, only : i1,i2,j1,j2,im,copy_region
    implicit none
    integer zone,periodic,gjs,gje,kdim,j1east,j2east, from(6),to(6)
    real f(i1:i2,j1:j2,kdim)
    
    gjs = bowest(zone)%global_range(3)
    gje = bowest(zone)%global_range(4)
    periodic = bowest(zone)%periodic
    j1east = boeast(periodic)%global_range(3)
    j2east = boeast(periodic)%global_range(4)
    
    from=(/im-3,im-2, j1east, j2east, 1,kdim/)
    to  =(/1,   2,    gjs,    gje,    1,kdim/)
    call copy_region(from, to, kdim, f)
  end subroutine periodic_west

  subroutine periodic_east(f, zone, kdim)
    use state, only : i1,i2,j1,j2,im,copy_region
    implicit none
    integer gjs, gje,kdim,j1west,j2west, from(6),to(6),zone,periodic
    real f(i1:i2,j1:j2,kdim)

    gjs = boeast(zone)%global_range(3)
    gje = boeast(zone)%global_range(4)
    periodic = boeast(zone)%periodic    
    j1west = bowest(periodic)%global_range(3)
    j2west = bowest(periodic)%global_range(4)
    
    from=(/3,   4,  j1west, j2west, 1,kdim/)
    to  =(/im-1,im, gjs,    gje,    1,kdim/)
    call copy_region(from, to, kdim, f)
  end subroutine periodic_east

  subroutine periodic_south(f, zone, kdim)
    use state, only : i1,i2,j1,j2,jm,copy_region
    implicit none
    integer gis, gie,kdim, i1north, i2north, from(6),to(6),zone,periodic
    real f(i1:i2,j1:j2,kdim)
    
    gis = bosouth(zone)%global_range(1)
    gie = bosouth(zone)%global_range(2)
    periodic = bosouth(zone)%periodic    
    i1north = bonorth(periodic)%global_range(1)
    i2north = bonorth(periodic)%global_range(2)
    
    from=(/i1north,i2north, jm-3,jm-2, 1,kdim/)
    to  =(/gis,    gie,     1,   2,    1,kdim/)
    call copy_region(from, to, kdim, f)
  end subroutine periodic_south

  subroutine periodic_north(f, zone, kdim)
    use state, only : i1,i2,j1,j2,jm,copy_region
    implicit none
    integer gis, gie,kdim,i1south, i2south, from(6),to(6),zone,periodic
    real f(i1:i2,j1:j2,kdim)
    
    gis = bonorth(zone)%global_range(1)
    gie = bonorth(zone)%global_range(2)
    periodic = bonorth(zone)%periodic    
    i1south = bosouth(periodic)%global_range(1)
    i2south = bosouth(periodic)%global_range(2)
    
    from=(/i1south,i2south, 3,   4,  1,kdim/)
    to  =(/gis,    gie,     jm-1,jm, 1,kdim/)
    call copy_region(from, to, kdim, f)
  end subroutine periodic_north

!-------------------------------------------------------------------------  
  
  subroutine frs_west(alfa, f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2
    implicit none
    integer zone,r(4),kdim
    real alfa(i1:i2,j1:j2),f(i1:i2,j1:j2,kdim),f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    integer i,j,k
    
    if(.not. bowest(zone)%inblock) return
    r = bowest(zone)%local_range
    
    if(present(f_tide))then
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*(f_bound(i,j,k)+f_tide(i,j,k)) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    else
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*f_bound(i,j,k) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    end if
  end subroutine frs_west

  subroutine frs_east(alfa, f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2,i_eq_im
    implicit none
    integer zone,r(4),kdim
    real alfa(i1:i2,j1:j2),f(i1:i2,j1:j2,kdim),f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    integer i,j,k
    
    if(.not. boeast(zone)%inblock) return
    r = boeast(zone)%local_range
    
    if(present(f_tide))then
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*(f_bound(i,j,k)+f_tide(i,j,k)) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    else
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*f_bound(i,j,k) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    end if
  end subroutine frs_east

  subroutine frs_south(alfa, f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2
    implicit none
    integer zone,r(4),kdim
    real alfa(i1:i2,j1:j2),f(i1:i2,j1:j2,kdim),f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    integer i,j,k
    
    if(.not. bosouth(zone)%inblock) return
    r = bosouth(zone)%local_range
    
    if(present(f_tide))then
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*(f_bound(i,j,k)+f_tide(i,j,k)) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    else
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*f_bound(i,j,k) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    end if
  end subroutine frs_south

  subroutine frs_north(alfa, f, f_bound, zone, kdim, f_tide)
    use state, only : i1,i2,j1,j2,j_eq_jm
    implicit none
    integer zone,r(4),kdim
    real alfa(i1:i2,j1:j2),f(i1:i2,j1:j2,kdim),f_bound(i1:i2,j1:j2,kdim)
    real, optional :: f_tide(i1:i2,j1:j2,kdim)
    integer i,j,k
    
    if(.not. bonorth(zone)%inblock) return
    r = bonorth(zone)%local_range
    
    if(present(f_tide))then
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*(f_bound(i,j,k)+f_tide(i,j,k)) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    else
        do k=1,kdim
            do j=r(3),r(4)
                do i=r(1),r(2)
                    f(i,j,k) = alfa(i,j)*f_bound(i,j,k) + (1-alfa(i,j))*f(i,j,k)
                end do
            end do
        end do
    end if
  end subroutine frs_north

!------------------------------------------------------------------
  
  subroutine get_v_bound(bin, frs)
    use state
    use mod_modesplit
    implicit none
    real vbclinfrs(i1:i2,j1:j2,kb),vabclinfrs(i1:i2,j1:j2)
    type(binreg_object) bin
    type(boundary_object) frs
    integer i,j,k, r(6), numsamp(maxnumbin), ibin
    real dep

    if(bin%inblock)then
        do k=1,kb-1
            do j=bin%local_range(3),bin%local_range(4)
                do i=bin%local_range(1),bin%local_range(2)
                    vbclin(i,j,k) = v(i,j,k) - va(i,j)
                end do
            end do            
        end do
    end if
    
    r(1:4) = bin%range
    r(5:6)=(/1,kb-1/)
    
    call tag_ghost(vbclin)
    call update_ghost()
    call pzsum(vbclin,im,jm,kb, bin%zlevels(1:bin%levels+1), zz, h, eta, bin%vbclinz(1:bin%levels), numsamp, r)

!! done with collective calls, we may return
    if(.not. frs%inblock) return   
    
    do k=1,bin%levels
        if(numsamp(k) > 0) then
            bin%vbclinz(k) = bin%vbclinz(k)/numsamp(k)
            bin%depmax = bin%zinn(k)
        else
            bin%vbclinz(k) = 0.0
        end if
    end do
    
!  Interpolate from VBCLINZ to VBCLINFRS
    
    r(1:4) = frs%local_range
    vbclinfrs(r(1):r(2),r(3):r(4),:) = 0.0

    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                if(block%fj + j -1 == 1) cycle
                if (dvm(i,j).ne.0.) then                    
                    dep = zz(k)*(hv(i,j)+0.5*(eta(i,j)+eta(i,j-1))) +0.5*(eta(i,j)+eta(i,j-1))
                    if (dep > bin%zinn(1)) then
                        vbclinfrs(i,j,k) = bin%vbclinz(1)
                    elseif (dep <= bin%zinn(bin%levels)) then
                        vbclinfrs(i,j,k) = bin%vbclinz(bin%levels)
                    else
                        do ibin = 1,bin%levels-1
                            if ((dep <= bin%zinn(ibin)) .and. (dep > bin%zinn(ibin+1))) then
                                vbclinfrs(i,j,k) =  bin%vbclinz(ibin+1) + (bin%vbclinz(ibin)-bin%vbclinz(ibin+1))* &
                                     &             (dep-bin%zinn(ibin+1)) / (bin%zinn(ibin)-bin%zinn(ibin+1))
                            endif
                        end do
                    endif
                endif
            end do
        end do
    end do
    
!
!  Check that vertical integrals of VBCLINFRS = 0.
!
    vabclinfrs = 0.
    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                vabclinfrs(i,j) = vabclinfrs(i,j) + dvm(i,j)*vbclinfrs(i,j,k)*dz(k)
            end do
        end do
    end do
    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                vbclinfrs(i,j,k) = dvm(i,j)*(vbclinfrs(i,j,k) - vabclinfrs(i,j))
            end do
        end do
    end do
    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                v_bound(i,j,k) = dvm(i,j)*(va_bound(i,j) + vbclinfrs(i,j,k))
            end do
        end do
    end do
  end subroutine get_v_bound


  subroutine get_u_bound(bin, frs)
! given a bin region with a number of uniformly distributed z layers,
! and a frs region, compute the horisontal average of U in the bins,
! and interpolate this average onto U in the frs region.
    use state
    use mod_modesplit
    implicit none
    real ubclinfrs(i1:i2,j1:j2,kb),uabclinfrs(i1:i2,j1:j2)
    type(binreg_object) bin
    type(boundary_object) frs
    integer i,j,k, r(6), numsamp(maxnumbin), ibin
    real dep

! find ubclin in the bin    
    if(bin%inblock)then
        do k=1,kb-1
            do j=bin%local_range(3),bin%local_range(4)
                do i=bin%local_range(1),bin%local_range(2)
                    ubclin(i,j,k) = u(i,j,k) - ua(i,j)
                end do
            end do
        end do
    end if
    
    r(1:4) = bin%range
    r(5:6)=(/1,kb-1/)

! Find the sum of UBCLIN in the points between the z levels defined in the bin, return in UBCLINZ. the
! number of points found in each layer is in NUMSAMP
    call tag_ghost(ubclin)
    call update_ghost()
    call pzsum(ubclin,im,jm,kb, bin%zlevels(1:bin%levels+1), zz, h, eta, bin%ubclinz(1:bin%levels+1), numsamp, r)

! Done with collective calls, we may return
    if(.not. frs%inblock) return   
    
    do k=1,bin%levels
        if(numsamp(k) > 0) then
            bin%ubclinz(k) = bin%ubclinz(k)/numsamp(k)
            bin%depmax = bin%zinn(k)
        else
            bin%ubclinz(k) = 0.0
        end if
    end do

! the average of U for each z layer is now in BIN%UBCLINZ(:)
! Now, interpolate from UBCLINZ to UBCLINFRS in the frs zone R
    
    r(1:4) = frs%local_range
    ubclinfrs(r(1):r(2),r(3):r(4),:) = 0.0

    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                if(block%fi + i -1 == 1) cycle
                if (dum(i,j).ne.0.) then
                    dep = zz(k)*(hu(i,j)+0.5*(eta(i,j)+eta(i-1,j))) +0.5*(eta(i,j)+eta(i-1,j))
                    if (dep > bin%zinn(1)) then
                        ubclinfrs(i,j,k) = bin%ubclinz(1)
                    elseif (dep <= bin%zinn(bin%levels)) then
                        ubclinfrs(i,j,k) = bin%ubclinz(bin%levels)
                    else
                        do ibin = 1,bin%levels-1
                            if ((dep <= bin%zinn(ibin)) .and. (dep > bin%zinn(ibin+1))) then
                                ubclinfrs(i,j,k) =  bin%ubclinz(ibin+1) + (bin%ubclinz(ibin)-bin%ubclinz(ibin+1))* &
                                     &             (dep-bin%zinn(ibin+1)) / (bin%zinn(ibin)-bin%zinn(ibin+1))
                            endif
                        end do
                    endif
                endif
            end do
        end do
    end do
    
!
!  Check that vertical integrals of VBCLINFRS = 0.
!
    uabclinfrs = 0.
    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                uabclinfrs(i,j) = uabclinfrs(i,j) + ubclinfrs(i,j,k)*dz(k)
            end do
        end do
    end do
    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                ubclinfrs(i,j,k) = ubclinfrs(i,j,k) - uabclinfrs(i,j)
            end do
        end do
    end do
    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                u_bound(i,j,k) = dum(i,j)*(ua_bound(i,j) + ubclinfrs(i,j,k))
            end do
        end do
    end do
  end subroutine get_u_bound

  
  subroutine get_st_bound(bin, frs)
!    
! on outflow, let s and t be relaxed towards a averaged vertical
! profile from the bin region. on inflow let s and t be relaxed
! towards the background values, which is set from get_and_apply
! forcing. if advect_density is true, do it for rho instead.
!    
    use state
    use control_parameters
    implicit none
    type(binreg_object) bin
    type(boundary_object) frs
    integer numsamps(maxnumbin), numsampt(maxnumbin), r(6),k
    
    r(1:4) = bin%range
    r(5:6) = (/1,kb-1/)
    if(.not.advect_density)then
        call tag_ghost(s)
        call tag_ghost(t)
        call update_ghost()
        call pzsum(s,im,jm,kb, bin%zlevels(1:bin%levels+1), zz, h, eta, bin%sz(1:bin%levels+1), numsamps, r)
        call pzsum(t,im,jm,kb, bin%zlevels(1:bin%levels+1), zz, h, eta, bin%tz(1:bin%levels+1), numsampt, r)
    else
        call tag_ghost(rho)
        call update_ghost()
        call pzsum(rho,im,jm,kb, bin%zlevels(1:bin%levels+1), zz, h, eta, bin%sz(1:bin%levels+1), numsamps, r)
    end if
    
    do k=1,bin%levels
        if(numsamps(k) > 0) then
            bin%sz(k) = bin%sz(k)/numsamps(k)
            bin%tz(k) = bin%tz(k)/numsamps(k)
            bin%depmax = bin%zinn(k)
        else
            bin%sz(k) = 0.0
            bin%tz(k) = 0.0
        end if
    end do

! horizontal averages from the bin now stored in bin%sz,tz. now
! interpolate onto s,t_bound in the frs zones.
    if(.not.advect_density)then
        call bin2f(bin%sz, s_bound, bin, frs)
        call bin2f(bin%tz, t_bound, bin, frs)
    else
        call bin2f(bin%sz, rho_bound, bin, frs)
    end if
  end subroutine get_st_bound


  subroutine get_conc_bound(i, bin, frs)
! just like get_st_bound, but for conc arrays
    use state
    use control_parameters
    implicit none
    integer i
    type(binreg_object) bin
    type(boundary_object) frs
    integer numsamps(maxnumbin), numsampt(maxnumbin), r(6),k
    
    r(1:4) = bin%range
    r(5:6) = (/1,kb-1/)

    call tag_ghost(co(i)%conc)
    call update_ghost()

    call pzsum(co(i)%conc,im,jm,kb, bin%zlevels(1:bin%levels+1), zz, h, eta, bin%sz(1:bin%levels+1), numsamps, r)

    do k=1,bin%levels
        if(numsamps(k) > 0) then
            bin%sz(k) = bin%sz(k)/numsamps(k)
            bin%depmax = bin%zinn(k)
        else
            bin%sz(k) = 0.0
        end if
    end do
            
! horizontal averages from the bin now stored in bin%sz. now
! interpolate onto xxx_bound in the frs zones.
    call bin2f(bin%sz, co(i)%conc_bound, bin, frs)
  end subroutine get_conc_bound
  

  subroutine bin2f(binf, f, bin, frs)
!    
! given a vertical profile from a bin called "binf", extrapolate from
! this onto the 3d distributed array f in the specified frs zone, on
! outflow, and only in the vertical region covered by the bin.
!    
    use state
    implicit none
    real binf(:), f(i1:i2,j1:j2,kb)
    type(binreg_object) bin
    type(boundary_object) frs
    integer r(4), numbin, i,j,k, ibin
    real dep
    logical outflow
    
    if(.not. frs%inblock) return

    numbin = bin%levels
    r = frs%local_range

    do k = 1,kb-1
        do j = r(3),r(4)
            do i = r(1),r(2)
                if(fsm(i,j).ne.0.) then

! the way we detect in/outflow should probably be improved. we could
! look at a horisontal average for each layer, the value on the edge,
! an average from the bin region, etc.
                    select case(frs%edge)
                    case(north)
                        outflow = v_bound(i,j,k) > 0.
                    case(south)
                        outflow = v_bound(i,j,k) < 0.
                    case(east)
                        outflow = u_bound(i,j,k) > 0.
                    case(west)
                        outflow = u_bound(i,j,k) < 0.
                    end select
                        
                    dep = zz(k)*(h(i,j)+eta(i,j))+eta(i,j)

                    ! if the depth is below depmax of the bin, do
                    ! nothing, the background (initial) value should
                    ! already be in v_bound, otherwise interpolate
                    ! like this:
                    if(dep > bin%depmax .and. outflow) then
                        if (dep .gt. bin%zinn(1)) then
                            f(i,j,k) = binf(1)
                        elseif (dep .le. bin%zinn(numbin)) then
                            f(i,j,k) = binf(numbin)
                        else
                            do ibin = 1, numbin-1
                                if ((dep .le. bin%zinn(ibin)) .and. (dep .gt. bin%zinn(ibin+1))) then
                                    f(i,j,k) =  binf(ibin+1) + (binf(ibin)-binf(ibin+1))* &
                                         &             (dep-bin%zinn(ibin+1)) / (bin%zinn(ibin)-bin%zinn(ibin+1))
                                endif
                            end do
                        endif
                    end if
                end if
            end do
        end do
    end do
  end subroutine bin2f

  
  subroutine get_eta_bound(elev, elev_bound, bin, frs)
!    
! compute the average value of elev in "bin" and insert it into
! elev_bound in "frs"
!    
    use state
    implicit none
    real, dimension(i1:i2,j1:j2) :: elev, elev_bound
    real, pointer :: tmparr(:,:)
    type(binreg_object) bin
    type(boundary_object) frs
    real average
    integer r(6),i

! provide global range for psum, return average on all processors.    
    r(1:4) = bin%range ; r(5:6) = (/1,1/)

    allocate(tmparr(i1:i2,j1:j2))
    tmparr = fsm*elev

    call tag_ghost(tmparr)
    call update_ghost()
          
    average = psum(tmparr, im,jm,1, range=r)/bin%numwet

    deallocate(tmparr)
    if(debug>4.and.master) write(*,'(a,i0,a,f0.3,a,6(i0,","))') "GET_ETA_BOUND: numwet=",bin%numwet," average=",average," range=",r
    
    if(frs%inblock)then
        r(1:4) = frs%local_range
        elev_bound(r(1):r(2),r(3):r(4)) = average
    end if    
  end subroutine get_eta_bound
  
  
  subroutine get_va_bound(vb, etab, vb_bound, bin, frs)
    use state
    implicit none
    real, dimension(i1:i2,j1:j2) :: vb, etab, vb_bound
    real, pointer :: tmp(:,:)
    type(binreg_object) bin
    type(boundary_object) frs
    integer gj, i,j, r(6), bin_length
    real avtra, areas(max(mblszi*numpbi,mblszj*numpbj))

    allocate(tmp(i1:i2,j1:j2))
    
    select case(frs%edge)
    case(north,south)
! tmp is the volume flux (transport) through the north or south wall of each cell
        tmp = vb*dx*(h + etab)
! the number of boxes across the bin in the north-south direction
        bin_length = bin%range(4)-bin%range(3)+1
    case(east,west)
        tmp = vb*dy*(h + etab)
        bin_length = bin%range(2)-bin%range(1)+1        
    end select
        
    r(1:4) = bin%range ; r(5:6) = (/1,1/)
! the sum of the transports over the bin, divided by the number of
! cross sections gives the average transport normal to the boundary

    call tag_ghost(tmp)
    call update_ghost()
    avtra = psum(tmp, im, jm, 1, range=r)/bin_length
    
    r(1:4) = frs%global_range
    areas = 0

! find the area of each xz layer for the bndry zone frs
    select case(frs%edge)
    case(north,south)
        tmp = dx*(h + etab)
        call tag_ghost(tmp)
        call update_ghost()
        call pvsum(tmp, im, jm, 1, 2, areas, r)
    case(east,west)
        tmp = dy*(h + etab)
        call tag_ghost(tmp)
        call update_ghost()
        call pvsum(tmp, im, jm, 1, 1, areas, r)
    end select    

! set the velocity bound array to the average transport divided by the
! local cross-section area
    if(frs%inblock)then
        do j = frs%local_range(3),frs%local_range(4)
            do i = frs%local_range(1),frs%local_range(2)
                select case(frs%edge)
                case(north,south)
                    gj = block%fj + j - 1
                case(east,west)
                    gj = block%fi + i - 1
                end select
                vb_bound(i,j) = avtra/areas(gj)
            end do
        end do
    end if
    deallocate(tmp)

  end subroutine get_va_bound

  
END MODULE MOD_BOUND 
