module mod_concentration
  implicit none
!
!  Define variables for concentrations
!
  INTEGER, PARAMETER :: MAXNUMCONC=9            ! if you increase this number there is work to be done in setup2 and pgplot.
                                                ! (grep for conc5 for a hint)
  
  INTEGER :: NUMCONC=0                          ! the number of (allocated) concentration fields
                                                ! to advect, also used to turn feature off if 0.
  TYPE CONCENTRATION_TYPE
     LOGICAL :: ALLOCATED=.FALSE.               ! true if all fields of this conc are allocated, indicates initialization.
     LOGICAL :: ACTIVE=.FALSE.                  ! true if the field should be advected during current time step
     REAL :: ACTIVATION_TIME = -1.0             ! Time f(in sec) for activation of tracer field
     REAL :: AVER = 0.0                         ! the average value of field
     CHARACTER(30) :: NAME=" "                  ! optional name, defaults to conc<N>
     REAL, DIMENSION(:,:,:), POINTER :: CONC=>NULL(),CONCN=>NULL(),CONCNM1=>NULL(),&
          DELCONCN=>NULL(),DELCONCSTAR=>NULL(),DELCONCNM1=>NULL(),CONC_BOUND=>NULL(),CONC_INITIAL=>NULL()
     REAL, POINTER :: CONCSURF(:,:) => NULL()
     REAL, POINTER :: AH(:,:,:) => NULL()
     REAL, POINTER :: KH(:,:,:) => NULL()
  END type CONCENTRATION_TYPE

  TYPE(CONCENTRATION_TYPE), SAVE :: CO(MAXNUMCONC)
  
contains
  
  
  subroutine activate_concentration
! this is the routine that activate or deactivate advection of a
! tracer, currently we use only the activation time as a criteria, others
! may be added.
    use state
    implicit none
    integer i

    do i=1,numconc
        if(co(i)%allocated) then        
            if(istep*dt < co(i)%activation_time) then
                co(i)%active = .false.
            end if
        
            if(istep*dt > co(i)%activation_time) then
                if(.not. co(i)%active .and. master) write(*,*) " Activating conc ",i
                co(i)%active = .true.
            end if
        end if
    end do

    if(debug > 1) then
        do i=1,maxnumconc
            if(co(i)%active .and. .not. co(i)%allocated) write(*,*) myid,"ERROR conc,",i,"active but not alloc"
        end do
    end if
    
  end subroutine activate_concentration

  subroutine allocate_concentration(i)
! allocate concentration arrays number i
    use state
    implicit none
    integer i

    if(.not. co(i)%allocated) then
       if(master .and. debug>0) write(*,'(2x,a,i0)') "allocating concentration ",i
       allocate(co(i)%conc(i1:i2,j1:j2,kb),co(i)%concn(i1:i2,j1:j2,kb),co(i)%concnm1(i1:i2,j1:j2,kb),co(i)%delconcn(i1:i2,j1:j2,kb))
       allocate(co(i)%delconcnm1(i1:i2,j1:j2,kb),co(i)%conc_bound(i1:i2,j1:j2,kb),co(i)%delconcstar(i1:i2,j1:j2,kb))
       allocate(co(i)%conc_initial(i1:i2,j1:j2,kb), co(i)%concsurf(i1:i2,j1:j2))
       numconc = numconc + 1        
       co(i)%allocated = .true.

       co(i)%concsurf = 0.0
    end if
  end subroutine allocate_concentration

  subroutine deallocate_concentration
    implicit none
    integer i

    do i=1,numconc
        if(co(i)%allocated) then
            deallocate(co(i)%conc,co(i)%concn,co(i)%concnm1,co(i)%delconcn,co(i)%delconcstar,co(i)%delconcnm1,co(i)%conc_bound)
            deallocate(co(i)%conc_initial, co(i)%concsurf)
            co(i)%allocated = .false.
        end if
    end do
  end subroutine deallocate_concentration

end module mod_concentration
