SUBROUTINE MY2HALV
  USE STATE
  USE CONTROL_PARAMETERS, ONLY : KMSPONGE, KHSPONGE, WCOURANTFIX
  USE MOD_CONCENTRATION
  IMPLICIT NONE
  INTEGER I,J,K
  REAL, DIMENSION(i1:i2,j1:j2,KB) :: PROD,DVELDZ,DRHODZ,L,Q3,PRODIMP
  REAL, DIMENSION(KB-2) :: RHS,A,B,C,ALF,BET,G

  REAL A1,A2,B1,B2,C1,EPS,E1,E2,KAPPA,SQ,B123,DSQ,LDIST,wall,gh,SH,SM,Q,QL
  REAL COEFF1,COEFF2,COEFF3,COEFF4,COEFF5
  REAL KQMIN
  PARAMETER (KQMIN= 1.E-5)
  INTEGER DJ, N
!
  DATA A1/0.92/
  DATA A2/0.74/
  DATA B1/16.6/
  DATA B2/10.1/
  DATA C1/0.08/
  DATA EPS/1.E-11/
  DATA E1/1.8/
  DATA E2/1.33/
  DATA KAPPA/0.40/
  DATA SQ/0.20/
!
  PROD = 0.
  L = 0.
  Q3 = 0.
  DVELDZ = 0.
  DRHODZ = 0.
  WALL = 0.
  GH = 0.
  SM = 0.
  SM = 0.
  Q = 0.
  QL = 0.
  PRODIMP = 0.
  B123 = B1**0.6666667


  DO J=1,J_EQ_JM
      DO K = 1,KB
        DO I=1,I_EQ_IM
              Q2(I,J,K) = MAX(EPS,Q2(I,J,K))
              Q2L(I,J,K) = MAX(EPS,Q2L(I,J,K))
          END DO
      END DO
  END DO

!
!  Define Q2 at the surface from Q2 = B1**(2/3)*|WUSURFACE|**2
!  Define Q2 at the bottom from Q2 = B1**(2/3)*|WUBOT|**2
!
DJ = 1 ; IF(JM==1) DJ=0
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              Q2(I,J,1) = B123*SQRT((.5*(WUSURF(I,J)+WUSURF(I+1,J)))**2 + &
                   &               (.5*(WVSURF(I,J)+WVSURF(I,J+DJ)))**2)
              Q2(I,J,KB) = B123*SQRT((.5*(WUBOT(I,J)+WUBOT(I+1,J)))**2 + &
                   &               (.5*(WVBOT(I,J)+WVBOT(I,J+DJ)))**2)
          ENDIF
      END DO
  END DO
!
!  Define production of Q2 from the term
!  2*KM*((du/dz)**2 + (dv/dz)**2)
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 2,KB-1
                  DVELDZ(I,J,K) = 0.25*KM(I,J,K)* &
                       &       ( (U(I,J,K)-U(I,J,K-1)+U(I+1,J,K)-U(I+1,J,K-1))**2 &
                       &         +(V(I,J,K)-V(I,J,K-1)+V(I,J+DJ,K)-V(I,J+DJ,K-1))**2 ) &
                       &              /(DZZ(K-1)*D(I,J))**2
                  PROD(I,J,K) = 2*DVELDZ(I,J,K)
              END DO
          ENDIF
      END DO
  END DO
!
!   Add production/loss of Q2 from (2*g/RHO0)*KH*(dRHO/dz).


DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 2,KB-1
                  DRHODZ(I,J,K) = (GRAV/RHO0)* &
                       &       (RHO(I,J,K-1)-RHO(I,J,K))/(DZZ(K-1)*D(I,J))
                  IF (DRHODZ(I,J,K).LE.0.) THEN
                      PRODIMP(I,J,K) = 2*KH(I,J,K)*DRHODZ(I,J,K)/(Q2(I,J,K)+EPS)
                  ELSE
                      PROD(I,J,K) = PROD(I,J,K) + 2*KH(I,J,K)*DRHODZ(I,J,K)
                      PRODIMP(I,J,K) = 0.
                  ENDIF
              END DO
          ENDIF
      END DO
  END DO
!
!   Subtract loss of kinetic energy (implicit).
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 2,KB-1
                  L(I,J,K) = Q2L(I,J,K)/(Q2(I,J,K) + EPS)
                  Q3(I,J,K) = Q2(I,J,K)*SQRT(Q2(I,J,K))
                  PRODIMP(I,J,K) = 2*Q3(I,J,K)/(B1*Q2L(I,J,K) + EPS) - PRODIMP(I,J,K)
              END DO
          ENDIF
      END DO
  END DO
!
!  Solve the equation
!  d(Q2)/dt = d(KQ d(Q2)/dz)/dz + PROD
!  with the fully implicit method.
!


DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DSQ = D(I,J)*D(I,J)
!
! Define the right hand side of the solution matrix.
!
              DO K = 2,KB-1
                  RHS(K-1) = Q2(I,J,K) + DT*PROD(I,J,K) 
              END DO
              RHS(1) = RHS(1) + DT*0.5*(KQ(I,J,1)+KQ(I,J,2))* &
                   &      Q2(I,J,1)/(DZ(1)*DZZ(1)*DSQ)
              RHS(KB-2) = RHS(KB-2) + DT*0.5*(KQ(I,J,KB)+KQ(I,J,KB-1))* &
                   &      Q2(I,J,KB)/(DZ(KB-1)*DZZ(KB-2)*DSQ)
!
! Define the upper diagonal elements of the tridiagonal solution matrix.
!
              DO K = 2,KB-2
                  B(K-1) = -DT*0.5*(KQ(I,J,K+1)+KQ(I,J,K))/(DSQ*DZ(K)*DZZ(K-1))
              END DO
!
! Define the lower diagonal elements.
!
              DO K = 3,KB-1
                  C(K-1) = -DT*0.5*(KQ(I,J,K-1)+KQ(I,J,K))/(DSQ*DZ(K-1)*DZZ(K-1))
              END DO
!
! Define the diagonal elements.
!
              A(1) = 1. - B(1)
              DO K = 2,KB-3
                  A(K) = 1. - B(K) - C(K)
              END DO
              A(KB-2) = 1. - C(KB-2)
              DO K = 1,KB-2
                  A(K) = A(K) + DT*PRODIMP(I,J,K+1)
              END DO
!
! Solve the system of equations.
!
              ALF(1) = A(1)
              DO K = 2,KB-2
                  ALF(K) = A(K) - C(K)*B(K-1)/ALF(K-1)
              END DO
              DO K = 2,KB-2
                  BET(K) = C(K)/ALF(K-1)
              END DO
!
!   Forward substitute the right hand side.
!
              G(1) = RHS(1)
              DO K = 2,KB-2
                  G(K) = RHS(K) - BET(K)*G(K-1)
              END DO
!
!   Back substitute to compute the solution Q2.
!
              Q2(I,J,KB-1) = G(KB-2)/ALF(KB-2)
              DO K = KB-3,1,-1
                  Q2(I,J,K+1) = (G(K) - B(K)*Q2(I,J,K+2))/ALF(K)
              END DO

          ENDIF
      END DO
  END DO
!
!  Q2 must be non-negative.
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 1,KB
                  Q2(I,J,K) = MAX(EPS,Q2(I,J,K))
              END DO
          ENDIF
      END DO
  END DO
!
! Then update Q2L at timestep n+1.
!
! First define the boundary values at the surface and the bottom.
!

  DO J=1,J_EQ_JM
     DO I=1,I_EQ_IM
!   Q2L(I,J,1) = 0.0246*Q2(I,J,1)*WSPEED10(I,J)**2
          Q2L(I,J,1) = 0.0
          Q2L(I,J,kb) = 0.0    
      END DO
  END DO
!
!
!  Define production of Q2L from the term
!  L*E1*KM*((du/dz)**2 + (dv/dz)**2)
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 2,KB-1
                  PROD(I,J,K) = L(I,J,K)*E1*DVELDZ(I,J,K)
              END DO
          ENDIF
      END DO
  END DO
!
!   Add production/loss of Q2L from (l*E1*g/RHO0)*KH*(dRHO/dz).
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 2,KB-1
                  IF (DRHODZ(I,J,K).LE.0.) THEN
                      PRODIMP(I,J,K) = L(I,J,K)*E1*KH(I,J,K)*DRHODZ(I,J,K)/(Q2L(I,J,K)+EPS)
                  ELSE
                      PROD(I,J,K) = PROD(I,J,K) + L(I,J,K)*E1*KH(I,J,K)*DRHODZ(I,J,K)
                      PRODIMP(I,J,K) = 0.
                  ENDIF
              END DO
          ENDIF
      END DO
  END DO
!
! Subtract loss of Q2L from -Q3*WALL/B1 (Implicit)
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 2,KB-1
                  LDIST = 1./(Z(1)-Z(K)) + 1./(Z(K)-Z(KB))
                  LDIST = (1./LDIST)*D(I,J)
                  WALL = 1. + E2*(L(I,J,K)/(KAPPA*LDIST))**2
                  PRODIMP(I,J,K) =  Q3(I,J,K)*WALL/(B1*Q2L(I,J,K) + EPS) - PRODIMP(I,J,K)
              END DO
          ENDIF
      END DO
  END DO
!
!
!  Solve the equation
!  d(Q2L)/dt = d(KQ d(Q2L)/dz)/dz + PROD
!  with the fully implicit method.
!


DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DSQ = D(I,J)*D(I,J)
!
! Define the right hand side of the solution matrix.
!
              DO K = 2,KB-1
                  RHS(K-1) = Q2L(I,J,K) + DT*PROD(I,J,K) 
              END DO
              RHS(1) = RHS(1) + DT*0.5*(KQ(I,J,1)+KQ(I,J,2))* &
                   &      Q2L(I,J,1)/(DZ(1)*DZZ(1)*DSQ)
              RHS(KB-2) = RHS(KB-2) + DT*0.5*(KQ(I,J,KB)+KQ(I,J,KB-1))* &
                   &      Q2L(I,J,KB)/(DZ(KB-1)*DZZ(KB-2)*DSQ)
!
! Define the upper diagonal elements of the tridiagonal solution matrix.
!
              DO K = 2,KB-2
                  B(K-1) = -DT*0.5*(KQ(I,J,K+1)+KQ(I,J,K))/(DSQ*DZ(K)*DZZ(K-1))
              END DO
!
! Define the lower diagonal elements.
!
              DO K = 3,KB-1
                  C(K-1) = -DT*0.5*(KQ(I,J,K-1)+KQ(I,J,K))/(DSQ*DZ(K-1)*DZZ(K-1))
              END DO
!
! Define the diagonal elements.
!
              A(1) = 1. - B(1)
              DO K = 2,KB-3
                  A(K) = 1. - B(K) - C(K)
              END DO
              A(KB-2) = 1. - C(KB-2)
              DO K = 1,KB-2
                  A(K) = A(K) + DT*PRODIMP(I,J,K+1)
              END DO
!
! Solve the system of equations.
!
              ALF(1) = A(1)
              DO K = 2,KB-2
                  ALF(K) = A(K) - C(K)*B(K-1)/ALF(K-1)
              END DO
              DO K = 2,KB-2
                  BET(K) = C(K)/ALF(K-1)
              END DO
!
!   Forward substitute the right hand side.
!
              G(1) = RHS(1)
              DO K = 2,KB-2
                  G(K) = RHS(K) - BET(K)*G(K-1)
              END DO
!
!   Back substitute to compute the solution Q2L.
!
              Q2L(I,J,KB-1) = G(KB-2)/ALF(KB-2)
              DO K = KB-3,1,-1
                  Q2L(I,J,K+1) = (G(K) - B(K)*Q2L(I,J,K+2))/ALF(K)
              END DO
!
          ENDIF
      END DO
  END DO
!
! Q2L must be non-negative.
!

DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 2,KB-1
                  Q2L(I,J,K) = MAX(EPS,Q2L(I,J,K))
              END DO
          ENDIF
      END DO
  END DO

!
!  SH*(1-(3*A2*B2+18*A1*A2)*GH) = A2*(1-6*A1/B1)
!
  COEFF1 = A2*(1. - 6.*A1/B1)
  COEFF2 = 3*A2*B2 + 18.*A1*A2
  COEFF3 = A1*(1. - 3*C1 - 6*A1/B1)
  COEFF4 = 18.*A1**2 + 9*A1*A2
  COEFF5 = 9.*A1*A2

! Compute the Richardson number GH.
!


DO J=1,JM_MI_1
  DO I=1,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
              DO K = 1,KB-1
                  L(I,J,K) = Q2L(I,J,K)/(Q2(I,J,K)+EPS)
                  GH = (L(I,J,K)**2/(Q2(I,J,K)+EPS))*DRHODZ(I,J,K)
                  GH = MIN(GH,0.028)
                  SH = COEFF1/(1. - COEFF2*GH)
                  SM = (COEFF3+COEFF4*SH*GH)/(1.-COEFF5*GH)
                  Q = SQRT(Q2(I,J,K))
                  QL = Q*L(I,J,K)
                  KM(I,J,K) = 0.5*(QL*SM+KM(I,J,K))
                  KH(I,J,K) = 0.5*(QL*SH+KH(I,J,K))
                  KQ(I,J,K) = 0.5*(QL*SQ+KQ(I,J,K))        
              END DO
          ENDIF
      END DO
  END DO
!
!   Compute KM,KH,KQ.
!
DO K = 2,KB-1
    DO J=J_EQ_2,JM_MI_1
        DO I=I_EQ_2,IM_MI_1
          IF (FSM(I,J).NE.0.) THEN
                  IF(KMSPONGE) KM(I,J,K) = MAX(SPONGE_KM(I,J), KM(I,J,K))
                  IF(KHSPONGE) KH(I,J,K) = MAX(SPONGE_KH(I,J), KH(I,J,K))
                  KM(I,J,K) = MAX(KMMIN,KM(I,J,K))
                  KH(I,J,K) = MAX(KHMIN,KH(I,J,K))
                  KQ(I,J,K) = MAX(KQMIN,KQ(I,J,K))
          ENDIF
      END DO
  END DO
END DO


! if we have very thin layers, the vertical courant number can give instabilities/overshooting.
! one remedy is to increase the viscosity in all cells where WADV is too large.

IF(WCOURANTFIX > 0) THEN
    DO K=1,KB-1
        DO J=1,J_EQ_JM
            DO I=1,I_EQ_IM
                IF ((H(I,J) > 0.0).AND.(DT*ABS(WADV(I,J,K)/(H(I,J)*DZ(K)))) > 0.5) THEN
                    KM(I,J,K) = KM(I,J,K)*WCOURANTFIX
                    KH(I,J,K) = KH(I,J,K)*WCOURANTFIX
                END IF
            END DO
        END DO
    END DO
end IF

! optionally add vertical diffusion for concentration fields (the CONC_KH keyword)

DO n=1,MAXNUMCONC
    IF(ASSOCIATED(CO(n)%KH)) THEN
        DO K = 2,KB-1
            DO J=J_EQ_2,JM_MI_1
                DO I=I_EQ_2,IM_MI_1
                    IF (FSM(I,J).NE.0.) THEN              
                        CO(n)%KH(I,J,K) = MAX(KH(I,J,K), SPONGE_KH_C1(I,J))
                    END IF
                END DO
            END DO
        END DO
    END IF
END DO

CALL DUMP_STATE("my2halv")
END SUBROUTINE MY2HALV
