module pgplot_core
!
! DUMMY interface for pgplot_core. This file should contain the same
! public variables and a minimum of the subroutine interfaces from
! pgplot_core.f90.
!
  
! data with value pgundef, which should be positive, default real, should be ignored in plots
  real, parameter :: pgundef = real(30000)
  
! plot from step=pgplot_interval(1), until pgplot_interval(2), with
! increment pgplot_interval(3)
  integer :: pgplot_interval(3)=(/0,200000,1/)

  integer :: pgplot_cache_results=0, pgplot_cache_interval=100
  real :: pgplot_cachedt=0.0
  
! if 0, dont call pgplot in dump_state. if 1 dump state in 3d routines, if 2 also in 2d routines.  
  integer :: dump_state_pgplot=0
  
  integer, parameter, private :: sp=4
  
! default rotation angle
  real(sp) :: pgplot_theta = 0.0

  
! transform: f(i,j) are located as   (tr(3) and tr(5) normally 0)
!  x = tr(1) + tr(2)*i + tr(3)*j
!  y = tr(4) + tr(5)*i + tr(6)*j
  
  real(sp) :: pgplot_transform(6)
  logical :: pgplot_core_do_init=.true., pgplot_debug=.false., pgplot_wait_after_plot=.false.
  logical :: pgplot_2d_show_vsects=.false., pgplot_coastline=.false., pgplot_vsect_grid=.false.
  logical :: pgplot_allocated=.false.
  character(30) :: pgplot_device = "?"
  
  character(200) :: pgplot_coastline_file=" "
  integer :: pgplot_coastline_segments=0
  real, allocatable :: pgcl(:,:)
  
  real(sp) :: pgplot_character_height = 1.0, pgplot_aspect = 1.0, pgplot_width = 5.0, &
       pgplot_viewport(4) = (/0.12,0.88,0.12,0.88/), pgplot_range(4) = (/0.0, 1.0, 0.0, 1.0/)

  logical :: pgplot_width_set=.false., pgplot_aspect_set=.false.
  integer :: pgplot_title_time=1
  
  integer :: pgplot_layout(2) = (/1,1/), pgplot_palette = 2, pgplot_contour_levels=10
! plot an arrow avery dens'th cell, default 2:
  integer :: pgplot_quiver_dens=2

! colors
  integer :: pgplot_text_color=1, pgplot_background=0, pgplot_grid_color=2, pgplot_curve_color=36

! line thickness
  integer :: pgplot_numbering_lw=1, pgplot_label_lw=1, pgplot_grid_lw=4, pgplot_curve_lw=5, pgplot_quiver_lw=2

  real(4) :: pgplot_headsize=0.3
  real :: pgplot_quiver_scale=2.0
  integer :: pgplot_quiver_color=1
  
! support for bomplot vsect plots using the normal vsect routine and
! the "pgplot=.true."optional argument.  these arrays should be
! allocated by calling subroutine allocate_pgvsect()  
  integer :: pgplot_maxsec=-1, pgplot_current_totsec  
  real, dimension(:,:), allocatable :: pg_vx, pg_vz, pg_usec, pg_utsec, pg_wsec, pg_ssec, pg_tsec
  
! hsect dumps fields here when called from pgplot  
  real, dimension(:,:), allocatable :: pg_u, pg_v, pg_f

! the integer land mask used by pgplot hsect
  integer, allocatable :: pgmask(:,:)
  
  real, allocatable :: timex(:)
  
  
! count the number of plots, and the number of pgplot timeseries plots
  integer :: pgplot_numplots=0
  integer :: pgplot_numts=0

! timeseries window
  integer :: pgplot_timeseries_window = 0
  
  type pgplot_object   
     integer :: ind=0                                         ! index in the pgplots array for this object     
     character(100) :: title=" ", xlabel=" ", ylabel=" "      ! plot title and labels if applicable     
     character(20) :: what=" "                                ! what variable to plot     
     character(1) :: slice=" "                                ! which slice? "i","j" or "k", only relevant for 3d pgslice.     
     integer :: type=0                                        ! the type of plot, 0 means blank. 1 is 2d, 2 hsect, 3 vsect, 4
                                                              ! timeseries, 5 vprof, 6 3d slice, 7 2d slice.
     integer :: i=0, j=0, k=0                                 ! gridpt. to fetch time series, or slice number for 3d pgslice.
     real :: true_range(4) = 0.0                              ! the horisontal grid range to plot (meters), if applicable
     integer :: range(4)=0                                    ! the horisontal, global array range from which to plot hsect, slice
     integer :: range3d(6)=0                                  ! full 3d range used by maxmin.f90 and maybe others.
     real, pointer:: f2d(:,:) => null(), f3d(:,:,:) => null() ! pointers to arrays to slice
     real :: fmax=-1e30, fmin=1e30                            ! span color map between these values, for trends it is the y range.
     real :: zmax=-1e30, zmin=1e30                            ! vertical range to plot, if applicable
     real :: depth=0.0                                        ! hsect depth     
     logical :: quiv=.true.                                   ! overlay velocity on hsect?     
     integer :: ts_i=0                                        ! index in timeseries for which series to view
     integer :: vsect_np=0                                    ! vsect points
     real :: px(10), py(10)
     real, allocatable :: data1d(:), cache1d(:)               ! storage for e.g. timeseries.
     real, allocatable :: data2d(:,:), data3d(:,:,:)
  end type pgplot_object
    
  integer, parameter :: pgplot_maxplots=30
  type(pgplot_object), save :: pgplots(pgplot_maxplots)
contains
  
  subroutine scatter(x, y, style, color, line_width)
!
! Scatter plot of points located at (x(:),y(:))
!  
    implicit none
    real x(:),y(:)
    integer, optional :: style, color, line_width
  end subroutine scatter
  subroutine pldj(x1, y1, x2, y2, style, color, color_array, line_width)
!
! Plot lines between points
!
    implicit none
    real :: x1(:),y1(:),x2(:),y2(:)
    integer, optional :: color_array(:)
    integer, optional :: style, color, line_width
  end subroutine pldj
end module pgplot_core
