module pgplot
!
! DUMMY interface for pgplot.f90. This file should contain the same
! public variables and a minimum of subroutine interfaces as
! pgplot.f90.
!
  use pgplot_core
  implicit none
  integer, parameter :: sp=4
  logical :: pgplot_do_init=.true.
  character(22) extratitle
  integer :: pgplot_numdumploc=0
  character(50) pgplot_dump_location(20)
  
contains
  
  subroutine batch_pgplot(label)
    use state, only : istep, master
    character(*), optional :: label
    if(master.and. pgplot_numplots > 0 .and. istep < 3) then
        write(*,'(1x,a,i0,a)') "batch_pgplot warning: pgplot_dummy is compiled in, so none of the ",&
             pgplot_numplots," requested plots will be made"
    end if
  end subroutine batch_pgplot
  
  subroutine init_pgplot()
    pgplot_do_init = .false.
  end subroutine init_pgplot

  subroutine pgclos()
  end subroutine pgclos

  subroutine get_pgplot_coastline(splitline)
    character(*) splitline(:)
  end subroutine get_pgplot_coastline
  
  subroutine set_pgplot_colorscale(splitline,f_ind)
    character(*) splitline(:)
    character(4), optional, intent(in) :: f_ind
  end subroutine set_pgplot_colorscale
  
  subroutine pgvsect_zrange(splitline)
    character(*) splitline(:)
  end subroutine pgvsect_zrange
  
  subroutine add_pgplot(type, splitline)
    character(*) type, splitline(:)
  end subroutine add_pgplot

  subroutine assignslice(slice)
! given a pgplot_object SLICE this routine connects the 2d or 3d array
! pointer SLICE%F2D, SLICE%F3D, to the array specified by the string
! SLICE%WHAT.
    use pgplot_core
    use state
    use mod_bound
    use mod_modesplit
    
    implicit none
    type(pgplot_object) slice
    
    select case(slice%what)
    case("u")
        slice%f3d => u
    case("v")
        slice%f3d => v
    case("w")
        slice%f3d => w
    case("wr")
        slice%f3d => wr
    case("s")
        slice%f3d => s
    case("t")
        slice%f3d => t
    case("rho")
        slice%f3d => rho
    case("q2")
        slice%f3d => q2
    case("q2l")
        slice%f3d => q2l
    case("am")
        slice%f3d => am
    case("ah")
        slice%f3d => ah
    case("km")
        slice%f3d => km
    case("kh")
        slice%f3d => kh
    case("p")
        slice%f3d => p
    case("uadv")
        slice%f3d => uadv
    case("vadv")
        slice%f3d => vadv
    case("wadv")
        slice%f3d => wadv
    case("deltau")
        slice%f3d => deltau
    case("deltav")
        slice%f3d => deltav
    case("cor")
        slice%f2d => cor
    case("fsm")
        slice%f2d => fsm
    case("dum")
        slice%f2d => dum
    case("dvm")
        slice%f2d => dvm
    case("eta")
        slice%f2d => eta
    case("etap")
        slice%f2d => etap
    case("h")
        slice%f2d => h
    case("hu")
        slice%f2d => hu
    case("hv")
        slice%f2d => hv
    case("d")
        slice%f2d => d
    case("du")
        slice%f2d => du
    case("dv")
        slice%f2d => dv
    case("wusurf")
        slice%f2d => wusurf
    case("wvsurf")
        slice%f2d => wvsurf
    case("ssurf")
        slice%f2d => ssurf
    case("tsurf")
        slice%f2d => tsurf
    case("rhosurf")
        slice%f2d => rhosurf
    case("wubot")
        slice%f2d => wubot
    case("wvbot")
        slice%f2d => wvbot
    case("cbc")
        slice%f2d => cbc
    case("patm")
        slice%f2d => patm
    case("am2d")
        slice%f2d => am2d
    case("ua")
        slice%f2d => ua
    case("va")
        slice%f2d => va
    case("husqr")
        slice%f2d => husqr
    case("hvsqr")
        slice%f2d => hvsqr
    case("sponge_cm2d")
        slice%f2d => sponge_cm2d
    case("sponge_cm")
        slice%f2d => sponge_cm
    case("sponge_km")
        slice%f2d => sponge_km
    case("sponge_kh")
        slice%f2d => sponge_kh
        
! from mod_bound        
    case("u_bound")
        slice%f3d => u_bound
    case("v_bound")
        slice%f3d => v_bound
    case("w_bound")
        slice%f3d => w_bound
    case("s_bound")
        slice%f3d => s_bound
    case("t_bound")
        slice%f3d => t_bound
    case("q2_bound")
        slice%f3d => q2_bound
    case("q2l_bound")
        slice%f3d => q2l_bound
    case("rho_bound")
        slice%f3d => rho_bound
    case("s_initial")
        slice%f3d => s_initial
    case("t_initial")
        slice%f3d => t_initial
    case("rho_initial")
        slice%f3d => rho_initial

    case("alfa_u")
        slice%f2d => alfa_u
    case("alfa_v")
        slice%f2d => alfa_v
    case("alfa_eta")
        slice%f2d => alfa_eta
    case("alfae_u")
        slice%f2d => alfae_u
    case("alfae_v")
        slice%f2d => alfae_v
    case("alfae_eta")
        slice%f2d => alfae_eta
    case("ua_bound")
        slice%f2d => ua_bound
    case("va_bound")
        slice%f2d => va_bound
    case("eta_bound")
        slice%f2d => eta_bound
    case("etabfrs")
        slice%f2d => etabfrs
    case("etatfrs")
        slice%f2d => etatfrs
    case("ua_tide")
        slice%f2d => ua_tide
    case("va_tide")
        slice%f2d => va_tide
    case("eta_tide")
        slice%f2d => eta_tide
    case("wspeedu")
        slice%f2d => wspeedu
    case("wspeedv")
        slice%f2d => wspeedv
    case("etatamp")
        slice%f3d => etatamp
    case("etatpha")
        slice%f3d => etatpha
    case("uatamp")
        slice%f3d => uatamp
    case("uatpha")
        slice%f3d => uatpha
    case("vatamp")
        slice%f3d => vatamp
    case("vatpha")
        slice%f3d => vatpha

        
! from mod_modesplit
    case("uv")
        slice%f3d => uv
    case("vu")
        slice%f3d => vu
    case("ubclin")
        slice%f3d => ubclin
    case("vbclin")
        slice%f3d => vbclin
    case("c3u")
        slice%f3d => c3u
    case("c3v")
        slice%f3d => c3v
    case("ucorr")
        slice%f3d => ucorr
    case("vcorr")
        slice%f3d => vcorr
    case("wcorr")
        slice%f3d => wcorr
    case("ax")
        slice%f2d => ax
    case("ay")
        slice%f2d => ay
    case("uv2d")
        slice%f2d => uv2d
    case("vu2d")
        slice%f2d => vu2d
    case("uasum")
        slice%f2d => uasum
    case("vasum")
        slice%f2d => vasum
    case("coru")
        slice%f2d => coru
    case("corv")
        slice%f2d => corv
    case("c2u")
        slice%f2d => c2u
    case("c2v")
        slice%f2d => c2v
    case("uad")
        slice%f2d => uad
    case("vad")
        slice%f2d => vad
    case("uastar")
        slice%f2d => uastar
    case("vastar")
        slice%f2d => vastar
    case("etastar")
        slice%f2d => etastar
    case("uaf")
        slice%f2d => uaf
    case("uab")
        slice%f2d => uab
    case("vaf")
        slice%f2d => vaf
    case("vab")
        slice%f2d => vab
    case("etab")
        slice%f2d => etab
    case("etaf")
        slice%f2d => etaf

! from mod_concentration
    case("conc1")
        slice%f3d => co(1)%conc
    case("conc2")
        slice%f3d => co(2)%conc
    case("conc3")
        slice%f3d => co(3)%conc
    case("conc4")
        slice%f3d => co(4)%conc
    case("conc5")
        slice%f3d => co(5)%conc
    case("conc6")
        slice%f3d => co(6)%conc
    case("conc7")
        slice%f3d => co(7)%conc
    case("conc8")
        slice%f3d => co(8)%conc
    case("conc9")
        slice%f3d => co(9)%conc
        
    case default
        call message("not yet taught how to add this 'what'",2)
    end select    
  end subroutine assignslice
  
end module pgplot

