! A dummy (do nothing) interface to qmpi.
! By linking this in, it is not necessary to change the original program in order to run it on a
! system without MPI.

! for timeseries
subroutine mpi_gatherv(sendbuf, n_send, sendtype, recvbuf, n_recv, displs, recvtype, boss, mpi_comm_world, ierr)
  integer sendbuf(*), n_send, sendtype, recvbuf(*), n_recv(*), displs(*), recvtype, boss, mpi_comm_world, ierr
  recvbuf(1:n_send) = sendbuf(1:n_send)
end subroutine mpi_gatherv

module qmpi
  implicit none
  integer, parameter :: MPI_REAL=-1, MPI_2REAL=-1, mpi_status_size=1,mpi_integer=0,mpi_comm_world=0
  integer, parameter :: mpi_proc_null=-1, mpi_comm_null=-1, mpi_group_null=-1
  integer, parameter :: MPI_ADDRESS_KIND=4, mpi_info_null=-1
  character :: mpi_character
  
  integer :: qmpi_proc_num=0, qmpi_num_proc=1, ierr=0, errorcode=0, mpistatus(mpi_status_size)  
  integer :: qmpi_comm_world=0
  integer :: myid=0, numproc=1
  
  logical :: master=.true., slave=.false.

  integer :: qmpi_debug=0       ! if 0 no extra checks or debug output
  
! some kinds. could use selected_real_kind(..) for this instead of hard coding

  integer, parameter :: dp=8, dbl=dp, sp=4, long=8, short=2, dr=kind(1.0)

  integer :: mpi_float_type = MPI_REAL, mpi_2floats = MPI_2REAL
  
  interface send
     module procedure            &
          qmpi_send_real4,       &
          qmpi_send_real4_1d,    &
          qmpi_send_real4_2d,    &
          qmpi_send_real4_3d,    &
          qmpi_send_real4_4d,    &
          qmpi_send_real8,       &
          qmpi_send_real8_1d,    &
          qmpi_send_real8_2d,    &
          qmpi_send_real8_3d,    &
          qmpi_send_real8_4d,    &
          qmpi_send_integer4,    &
          qmpi_send_integer4_1d, &
          qmpi_send_integer4_2d, &
          qmpi_send_integer4_3d, &
          qmpi_send_integer4_4d, &
          qmpi_send_integer8,    &
          qmpi_send_integer8_1d, &
          qmpi_send_integer8_2d, &
          qmpi_send_integer8_3d, &
          qmpi_send_integer8_4d, &
          qmpi_send_string,      &
          qmpi_send_character_1d,&
          qmpi_send_logical
  end interface

  interface receive
     module procedure &
          qmpi_recv_real4,       &
          qmpi_recv_real4_1d,    &
          qmpi_recv_real4_2d,    &
          qmpi_recv_real4_3d,    &
          qmpi_recv_real4_4d,    &
          qmpi_recv_real8,       &
          qmpi_recv_real8_1d,    &
          qmpi_recv_real8_2d,    &
          qmpi_recv_real8_3d,    &
          qmpi_recv_real8_4d,    &
          qmpi_recv_integer4,    & 
          qmpi_recv_integer4_1d, &
          qmpi_recv_integer4_2d, &
          qmpi_recv_integer4_3d, &
          qmpi_recv_integer4_4d, &
          qmpi_recv_integer8,    &
          qmpi_recv_integer8_1d, &
          qmpi_recv_integer8_2d, &
          qmpi_recv_integer8_3d, &
          qmpi_recv_integer8_4d, &
          qmpi_recv_string,      &
          qmpi_recv_character_1d,&
          qmpi_recv_logical
  end interface

  interface reduce
     module procedure &
          qmpi_integer_reduction, &
          qmpi_integer1d_reduction, &          
          qmpi_integer2d_reduction, &
          qmpi_integer4d_reduction, &
          qmpi_integer8_reduction,&
          qmpi_real_reduction,    &
          qmpi_real8_reduction, &
          qmpi_1dreal4_reduction, &
          qmpi_2dreal4_reduction, &
          qmpi_1dreal8_reduction
  end interface

  interface gather
     module procedure &
          qmpi_gather_integer4, &
          qmpi_gather_integer8, &
          qmpi_gather_real4,    &
          qmpi_gather_real8
     ! ...
  end interface
  
  interface broadcast
     module procedure &
          qmpi_broadcast_logical,  &
          qmpi_broadcast_string,   &
          qmpi_broadcast_stringarr,&
          qmpi_broadcast_integer4, &
          qmpi_broadcast_integer4_array1d,  &
          qmpi_broadcast_integer4_array2d,  &
          qmpi_broadcast_integer8, &
          qmpi_broadcast_integer8_array1d, &
          qmpi_broadcast_integer8_array2d, &
          qmpi_broadcast_real4, &
          qmpi_broadcast_real4_array1d, &
          qmpi_broadcast_real4_array2d, &
          qmpi_broadcast_real4_array3d, &
          qmpi_broadcast_real4_array4d, &
          qmpi_broadcast_real8, &
          qmpi_broadcast_real8_array1d, &
          qmpi_broadcast_real8_array2d, &
          qmpi_broadcast_real8_array3d, &
          qmpi_broadcast_real8_array4d
  end interface

  interface mbroadcast
     module procedure &
          qmpi_broadcast_logicals, &
          qmpi_broadcast_real4s, &
          qmpi_broadcast_real8s, &
          qmpi_broadcast_integer4s, &
          qmpi_broadcast_integer8s
  end interface

  interface isend
     module procedure &
          qmpi_isend_integer4, &
          qmpi_isend_integer4_1d, &
          qmpi_isend_integer4_2d, &
          qmpi_isend_character_1d, &
          qmpi_isend_real4, &
          qmpi_isend_real8, &
          qmpi_isend_real4_1d, &
          qmpi_isend_real8_1d, &
          qmpi_isend_real4_2d, &
          qmpi_isend_real8_2d, &
          qmpi_isend_real4_3d, &
          qmpi_isend_real8_3d, &
          qmpi_isend_real4_4d, &
          qmpi_isend_real8_4d
  end interface
  
  interface irecv
     module procedure &
          qmpi_irecv_integer4_1d, &
          qmpi_irecv_real4_1d, &
          qmpi_irecv_real8_1d, &          
          qmpi_irecv_real4_2d, &
          qmpi_irecv_real8_2d, &
          qmpi_irecv_real8_3d     
  end interface

! the maximum number of dangling requests
  integer, parameter :: maxnumrequests=500
  integer:: request_list(maxnumrequests), numrequests=0
  integer:: wait_stat(mpi_status_size,2)

  integer, parameter :: max_members=50
  
  type communicator_object
     integer :: size=0                                    ! the number of processors in the comm
     integer :: comm=mpi_comm_null                        ! the communicator
     integer :: group=mpi_group_null                      ! the group
     integer :: myid=mpi_proc_null                        ! the local rank in the communicator
     integer :: global_rank(max_members)=mpi_proc_null    ! the list of global ranks
  end type communicator_object



! the host processor clock frequency to be used by the high res timer  
  integer(kind=8) :: clock_frequency = 2826206000_8  ! atlasseder
!  integer(kind=8) :: clock_frequency = 2600000000_8  ! hexagon login

! uncomment these 2 lines to link in rdtsc, or use the function below (default)
!  integer(8) rdtsc
!  external rdtsc 


  
! BOM specific, setup2.f90: "current" data while parsing
  logical :: parsing_done=.false.
  integer :: line_number = 0
  character(len=200) :: file_name=" ", keyword=" ", current_setup_line
  public parsing_done, line_number, file_name, keyword, current_setup_line
  logical :: viewblocks=.true.
contains

  
  function rdtsc()
! poor mans (read time stamp counter) rdtsc, if the assembly snippet is not available
    integer(8) rdtsc
    integer time
    
    call system_clock(time)
    rdtsc = time
  end function rdtsc
  
  subroutine start_qmpi()
    implicit none
    real :: default_real=3.14
    
    numproc=1
    myid=0   
    qmpi_num_proc=numproc
    qmpi_proc_num=myid
  
    !! call set_stack_space(hard, soft, ierr)
    master=.false.
    if(myid==0) master=.true.
    if(myid>0) slave=.true.
    
    if(master) then
        write(*,'(a,i0,a)') 'DUMMY MPI:',numproc,' processors found'
    end if
  end subroutine start_qmpi

  subroutine stop_qmpi()
    implicit none
  end subroutine stop_qmpi

  subroutine barrier(label)
    implicit none
    character(*), optional :: label
    if(master.and.present(label)) write(*,*)'---barrier: ',label,'---'
  end subroutine barrier
  
  subroutine abort(message, vals)
    character(len=*), optional :: message
    real, optional :: vals(:)

    if(present(message)) then
        write(*,*) 'BOM abort called:',message,' MYID=',myid
        if(present(vals)) then
            write(*,'(a,10(f0.2,1x))') "Values= ",vals
        end if
    else
        if(master) write(*,*) 'BOM abort called, ending execution'
    end if
    stop
  end subroutine abort  


  subroutine message(text, severity, rval, rval4, rval8, lval, sval, ival, margin, advance, sync)
!
! print a message on screen, severity; 0=info, 1=warning, 2=fatal => call abort()
! if one of the real arrays RVAL, integer array IVAL,
! etc. is present, append values at end of line.  MARGIN is the number
! of spaces at the beginning of the line, by default 0.
! if called from setup2, also print current location in setup file.
! if SYNC, insert barrier before writing text.
! example:
!    call message("Flux F(i,j) out of range. F,I,J,IM,JM,KB = ", 2, rval=(/f(i,j)/), ival=(/i,j,im,jm,kb/))

    implicit none
    character(*), optional :: text
    integer, optional :: severity
    real, optional :: rval(:)
    real(4), optional :: rval4(:)
    real(8), optional :: rval8(:)
    logical, optional :: lval(:)
    character(*), optional :: sval(:)
    integer, optional :: ival(:), margin
    character(3), optional :: advance
    logical, optional :: sync
    character(10) header
    character(3) adv
    character(2) marg
    integer i
    logical val

    if(present(sync)) then
        if(sync) call barrier()
    end if
    
    val = present(rval).or.present(ival).or.present(lval).or.present(sval).or.present(rval4).or.present(rval8)
    
    header=' '
    if(present(severity)) then
        select case(severity)
        case(0)
            header='! INFO:'
        case(1)
            header='! WARNING:'
        case(2)
            header='! ERROR:'
        end select
    end if

    marg=""
    if(present(margin)) write(marg,'(i1,"x,")') margin

    adv="yes"
    if(val) adv="no"
    if(present(advance)) adv=advance
    
    if(master)then
        if(present(text).and. header.ne."") then
            write(*,'('//marg//'a,1x,a)', advance=adv) trim(header),trim(text)
        else if(present(text)) then
            write(*,'('//marg//'a)', advance=adv) trim(text)
        else if(header.ne."") then
            write(*,'(a)', advance=adv) trim(header)
        end if
    end if

    if(val.and.master) then
        ! up to 10 of each val shoould be sufficient
        if(present(rval)) write(*,'(10(1x,f0.7))',advance="no") (rval(i),i=1,size(rval))
        if(present(rval4)) write(*,'(10(1x,f0.7))',advance="no") (rval4(i),i=1,size(rval4))
        if(present(rval8)) write(*,'(10(1x,f0.7))',advance="no") (rval8(i),i=1,size(rval8))
        if(present(ival)) write(*,'(10(1x,i0))',advance="no") (ival(i),i=1,size(ival))
        if(present(lval)) write(*,'(10(1x,l1))',advance="no") (lval(i),i=1,size(lval))
        if(present(sval)) write(*,'(10(1x,a))',advance="no") (sval(i),i=1,size(sval))        
        write(*,*)
    end if
    
! BOM specific:
    if(present(severity).and..not.parsing_done)then
        if(severity==2.and.master) then
            write(*,'(2x,a,a,a,i0,a,a)')'Keyword=',trim(keyword), ', line=', line_number,', in file=', trim(file_name)
            write(*,'(2x,a,//,2x,a,/)') 'Config line in question:',trim(current_setup_line)
        end if
    end if
    
    if(present(severity)) then
        if(severity .eq. 2) call abort()
    end if
  end subroutine message

  subroutine waitall()
  end subroutine waitall
  
  subroutine qmpi_send_logical(data, target, tag)
    implicit none
    logical data
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_logical

  subroutine qmpi_send_string(data, target, tag)
    implicit none
    character(*) data
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_string

  subroutine qmpi_send_character_1d(data, target, tag)
    implicit none
    character data(:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_character_1d
  
  subroutine qmpi_recv_character_1d(data, target, tag)
    implicit none
    character data(:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_character_1d
    
  subroutine qmpi_send_integer4(data, target, tag)
    implicit none
    integer(sp) data
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer4

  subroutine qmpi_isend_integer4(data, target, tag)
    implicit none
    integer(sp) data
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_isend_integer4

  subroutine qmpi_send_integer4_1d(data, target, tag)
    implicit none
    integer(sp) data(:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer4_1d

  subroutine qmpi_isend_integer4_1d(data, target, tag)
    implicit none
    integer(sp) data(:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_isend_integer4_1d
  subroutine qmpi_isend_character_1d(data, target, tag)
    implicit none
    character data(:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_isend_character_1d
  
  subroutine qmpi_send_integer4_2d(data, target, tag)
    implicit none
    integer(sp) data(:,:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer4_2d

  subroutine qmpi_isend_integer4_2d(data, target, tag)
    implicit none
    integer(sp) data(:,:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_isend_integer4_2d

  subroutine qmpi_send_integer4_3d(data, target, tag)
    implicit none
    integer(sp) data(:,:,:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer4_3d

  subroutine qmpi_send_integer4_4d(data, target, tag)
    implicit none
    integer(sp) data(:,:,:,:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer4_4d

  subroutine qmpi_send_integer8(data, target, tag)
    implicit none
    integer(long) data
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer8

  subroutine qmpi_send_integer8_1d(data, target, tag)
    implicit none
    integer(long) data(:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer8_1d

  subroutine qmpi_send_integer8_2d(data, target, tag)
    implicit none
    integer(long) data(:,:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer8_2d

  subroutine qmpi_send_integer8_3d(data, target, tag)
    implicit none
    integer(8) data(:,:,:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer8_3d

  subroutine qmpi_send_integer8_4d(data, target, tag)
    implicit none
    integer(8) data(:,:,:,:)
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_send_integer8_4d

  subroutine qmpi_send_real4(data, target, tag)
    implicit none
    real(sp) data
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real4

  subroutine qmpi_isend_real4(data, target, tag)
    implicit none
    real(sp) data
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real4

  subroutine qmpi_send_real8(data, target, tag)
    implicit none
    real(dp) data
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real8

  subroutine qmpi_isend_real8(data, target, tag)
    implicit none
    real(dp) data
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real8

  subroutine qmpi_send_real4_1d(data, target, tag)
    implicit none
    real(sp) data(:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real4_1d

  subroutine qmpi_isend_real4_1d(data, target, tag)
    implicit none
    real(sp) data(:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real4_1d
 
  subroutine qmpi_send_real8_1d(data, target, tag)
    implicit none
    real(dp) data(:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real8_1d

  subroutine qmpi_isend_real8_1d(data, target, tag)
    implicit none
    real(dp) data(:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real8_1d
  
  subroutine qmpi_send_real4_2d(data, target, tag)
    implicit none
    real(sp) data(:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real4_2d

  subroutine qmpi_isend_real4_2d(data, target, tag)
    implicit none
    real(sp) data(:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real4_2d
 
  subroutine qmpi_send_real8_2d(data, target, tag)
    implicit none
    real(dp) data(:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real8_2d

  subroutine qmpi_isend_real8_2d(data, target, tag)
    implicit none
    real(dp) data(:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real8_2d

  subroutine qmpi_send_real4_3d(data, target, tag)
    implicit none
    real(sp) data(:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real4_3d

  subroutine qmpi_isend_real4_3d(data, target, tag)
    implicit none
    real(sp) data(:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real4_3d

  
  subroutine qmpi_isend_real4_4d(data, target, tag)
    implicit none
    real(sp) data(:,:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real4_4d
  subroutine qmpi_send_real8_3d(data, target, tag)
    implicit none
    real(dp) data(:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real8_3d

  subroutine qmpi_isend_real8_3d(data, target, tag)
    implicit none
    real(dp) data(:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real8_3d

  
  subroutine qmpi_send_real4_4d(data, target, tag)
    implicit none
    real(sp) data(:,:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real4_4d

  subroutine qmpi_isend_real8_4d(data, target, tag)
    implicit none
    real(dp) data(:,:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_isend_real8_4d
  subroutine qmpi_send_real8_4d(data, target, tag)
    implicit none
    real(dp) data(:,:,:,:)
    integer target
    integer, optional :: tag
  end subroutine qmpi_send_real8_4d

  subroutine qmpi_recv_integer4(data, source, tag)
    implicit none
    integer(sp) data
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer4

  subroutine qmpi_recv_integer4_1d(data, source, tag)
    implicit none
    integer(sp) data(:)
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer4_1d
  
  subroutine qmpi_irecv_integer4_1d(data, source, tag)
    implicit none
    integer(sp) data(:)
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_irecv_integer4_1d

  
  subroutine qmpi_recv_integer4_2d(data, source, tag)
    implicit none
    integer(sp) data(:,:)
    integer source, count
    integer, optional :: tag
  end subroutine qmpi_recv_integer4_2d

  subroutine qmpi_recv_integer4_3d(data, source, tag)
    implicit none
    integer(sp) data(:,:,:)
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer4_3d

  subroutine qmpi_recv_integer4_4d(data, source, tag)
    implicit none
    integer(sp) data(:,:,:,:)
    integer source, count
    integer, optional :: tag
  end subroutine qmpi_recv_integer4_4d

  subroutine qmpi_recv_integer8(data, source, tag)
    implicit none
    integer(long) data
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer8

  subroutine qmpi_recv_integer8_1d(data, source, tag)
    implicit none
    integer(long) data(:)
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer8_1d

  subroutine qmpi_recv_integer8_2d(data, source, tag)
    implicit none
    integer(long) data(:,:)
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer8_2d

  subroutine qmpi_recv_integer8_3d(data, source, tag)
    implicit none
    integer(8) data(:,:,:)
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer8_3d

  subroutine qmpi_recv_integer8_4d(data, source, tag)
    implicit none
    integer(8) data(:,:,:,:)
    integer source, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_integer8_4d

  subroutine qmpi_recv_real4(data, source, tag)
    implicit none
    real(sp) data
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real4

  subroutine qmpi_recv_real8(data, source, tag)
    implicit none
    real(dp) data
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real8

  subroutine qmpi_recv_real4_1d(data, source, tag)
    implicit none
    real(sp) data(:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real4_1d

  
  subroutine qmpi_irecv_real4_1d(data, source, tag)
    implicit none
    real(sp) data(:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_irecv_real4_1d
  
  subroutine qmpi_irecv_real4_2d(data, source, tag)
    implicit none
    real(sp) data(:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_irecv_real4_2d

  
  subroutine qmpi_recv_real8_1d(data, source, tag)
    implicit none
    real(dp) data(:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real8_1d

  subroutine qmpi_irecv_real8_1d(data, source, tag)
    implicit none
    real(dp) data(:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_irecv_real8_1d
  
  subroutine qmpi_irecv_real8_2d(data, source, tag)
    implicit none
    real(dp) data(:,:)
    integer source
    integer, optional :: tag
    
  end subroutine qmpi_irecv_real8_2d

  subroutine qmpi_irecv_real8_3d(data, source, tag)
    implicit none
    real(dp) data(:,:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_irecv_real8_3d

  
  subroutine qmpi_recv_real4_2d(data, source, tag)
    implicit none
    real(sp) data(:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real4_2d

  subroutine qmpi_recv_real8_2d(data, source, tag)
    implicit none
    real(dp) data(:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real8_2d

  subroutine qmpi_recv_real4_3d(data, source, tag)
    implicit none
    real(sp) data(:,:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real4_3d

  subroutine qmpi_recv_real8_3d(data, source, tag)
    implicit none
    real(dp) data(:,:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real8_3d

  subroutine qmpi_recv_real4_4d(data, source, tag)
    implicit none
    real(sp) data(:,:,:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real4_4d

  subroutine qmpi_recv_real8_4d(data, source, tag)
    implicit none
    real(dp) data(:,:,:,:)
    integer source
    integer, optional :: tag
  end subroutine qmpi_recv_real8_4d

  subroutine qmpi_recv_logical(data, target, tag)
    implicit none
    logical data
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_logical

  subroutine qmpi_recv_string(data, target, tag)
    implicit none
    character(*) data
    integer target, count, given_tag
    integer, optional :: tag
  end subroutine qmpi_recv_string

  subroutine qmpi_broadcast_string(string,root,comm)
    implicit none
    character(len=*) string
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_string

  subroutine qmpi_broadcast_stringarr(data,root,comm)
    implicit none
    character(len=*) data(:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_stringarr

  subroutine qmpi_broadcast_real4(data,root,comm)
    implicit none
    real(4) data
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real4

  subroutine qmpi_broadcast_real8(data,root,comm)
    implicit none
    real(8) data
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real8

  subroutine qmpi_broadcast_integer4(data,root,comm)
    implicit none
    integer(4) data
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_integer4

  subroutine qmpi_broadcast_integer8(data,root,comm)
    implicit none
    integer(8) data
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_integer8

  subroutine qmpi_broadcast_logical(data, root, comm)
    implicit none
    logical data
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_logical


  subroutine qmpi_broadcast_integer4_array1d(data,root,comm)
    implicit none
    integer(sp) data(:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_integer4_array1d
  
  subroutine qmpi_broadcast_integer8_array1d(data,root,comm)
    implicit none
    integer(long) data(:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_integer8_array1d

  subroutine qmpi_broadcast_integer4_array2d(data,root,comm)
    implicit none
    integer(sp) data(:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_integer4_array2d
    
  subroutine qmpi_broadcast_integer8_array2d(data,root,comm)
    implicit none
    integer(long) data(:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_integer8_array2d

  subroutine qmpi_broadcast_real4_array1d(data,root,comm)
    implicit none
    real(sp) data(:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real4_array1d

  subroutine qmpi_broadcast_real8_array1d(data,root,comm)
    implicit none
    real(dp) data(:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real8_array1d

  subroutine qmpi_broadcast_real4_array2d(data,root,comm)
    implicit none
    real(sp) data(:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real4_array2d

  subroutine qmpi_broadcast_real8_array2d(data,root,comm)
    implicit none
    real(dp) data(:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real8_array2d

  subroutine qmpi_broadcast_real4_array3d(data,root,comm)
    implicit none
    real(sp) data(:,:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real4_array3d

  subroutine qmpi_broadcast_real8_array3d(data,root,comm)
    implicit none
    real(dp) data(:,:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real8_array3d

  subroutine qmpi_broadcast_real4_array4d(data,root,comm)
    implicit none
    real(sp) data(:,:,:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real4_array4d

  subroutine qmpi_broadcast_real8_array4d(data,root,comm)
    implicit none
    real(dp) data(:,:,:,:)
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real8_array4d

  
  function qmpi_gather_integer4(data,root,comm)
    implicit none
    integer(4) data, qmpi_gather_integer4(qmpi_num_proc)
    integer, optional :: root,comm
    qmpi_gather_integer4(1) = data
  end function qmpi_gather_integer4

  function qmpi_gather_integer8(data,root,comm)
    implicit none
    integer(8) data, qmpi_gather_integer8(qmpi_num_proc)
    integer, optional :: root,comm
    qmpi_gather_integer8(1) = data
  end function qmpi_gather_integer8

  function qmpi_gather_real4(data,root,comm)
    implicit none
    real(4) data, qmpi_gather_real4(qmpi_num_proc)
    integer, optional :: root,comm
    qmpi_gather_real4(1) = data    
  end function qmpi_gather_real4

  function qmpi_gather_real8(data,root,comm)
    implicit none
    real(8) data, qmpi_gather_real8(qmpi_num_proc)
    integer, optional :: root,comm
    qmpi_gather_real8(1) = data
  end function qmpi_gather_real8
 
  
  subroutine qmpi_broadcast_real4s(a,b,c,d,e,f,root,comm)
    implicit none
    real(sp) a
    real(sp), optional :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real4s

  subroutine qmpi_broadcast_real8s(a,b,c,d,e,f,root,comm)
    implicit none
    real(dp) a
    real(dp), optional :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_real8s
  
  subroutine qmpi_broadcast_logicals(a,b,c,d,e,f,root,comm)
    implicit none
    logical a
    logical, optional :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_broadcast_logicals

  subroutine qmpi_broadcast_integer4s(a,b,c,d,e,f,root,comm)
    implicit none
    integer(sp) a
    integer(sp), optional :: b,c,d,e,f,root,comm
  end subroutine qmpi_broadcast_integer4s

  subroutine qmpi_broadcast_integer8s(a,b,c,d,e,f,root,comm)
    implicit none
    integer(long) a
    integer(long), optional :: b,c,d,e,f,root,comm
  end subroutine qmpi_broadcast_integer8s

  subroutine qmpi_real_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    real(sp) a
    real(sp), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_real_reduction

  subroutine qmpi_real8_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    integer, parameter :: dp=8
    character(3) type
    real(dp) a
    real(dp), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_real8_reduction

  subroutine qmpi_1dreal4_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    real(sp) a(:)
    real(sp), dimension(:), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_1dreal4_reduction
  
  subroutine qmpi_1dreal8_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    real(dp) a(:)
    real(dp), dimension(:), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_1dreal8_reduction
  
  subroutine qmpi_integer_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    integer(sp) a
    integer(sp), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_integer_reduction

  subroutine qmpi_integer1d_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    integer(sp) a(:)
    integer(sp), dimension(:), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_integer1d_reduction
  subroutine qmpi_integer2d_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    integer(sp) a(:,:)
    integer(sp), dimension(:,:), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_integer2d_reduction
  subroutine qmpi_integer4d_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    integer(sp) a(:,:,:,:)
    integer(sp), dimension(:,:,:,:), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_integer4d_reduction
  
  subroutine qmpi_integer8_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    integer(long) a
    integer(long), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_integer8_reduction
  subroutine qmpi_2dreal4_reduction(type,a,b,c,d,e,f,root,comm)
    implicit none
    character(3) type
    real(sp) a(:,:)
    real(sp), dimension(:,:), optional, intent(inout) :: b,c,d,e,f
    integer, optional :: root,comm
  end subroutine qmpi_2dreal4_reduction
 
  subroutine initialize_communicator(ranks, c)
    implicit none
    integer ranks(:)
    type(communicator_object) c
  end subroutine initialize_communicator
  
  subroutine free_communicator(c)
    implicit none
    type(communicator_object) c
  end subroutine free_communicator
  
end module qmpi
