subroutine runtime_control()
!  
! This routine is run periodically to change control parameters run time.
!  
  use mod_mpi
  use util
  implicit none

  integer, parameter :: maxn=30         ! the maximum number of lines allowed
  character(200) :: rtlines(maxn)=" "   ! valid setup lines
  integer :: rtkeywords, srcln(maxn)    ! the number of keywords in the above lists.
  character(50) splitline(20)
  integer i,j
  logical file_present

  
  rtkeywords = 0
  
  inquire(file="runtime.dat", exist=file_present)

  if(file_present .and. master) then
      call get_lines("runtime.dat", rtkeywords, rtlines, srcln)

      ! then erase the file
      open(1000011, file="runtime.dat", status='replace')
      close(1000011)
  end if  

! On all processors, parse each of the config lines, and act on each valid keyword

  call broadcast(rtkeywords)
  if(rtkeywords == 0) return
  
  call broadcast(rtlines)
  call broadcast(srcln)

  do i=1,rtkeywords
      current_setup_line = rtlines(i)
      line_number = srcln(i)
      
      call split(rtlines(i), splitline)
      call parse_config_line(lower(trim(splitline(1))), splitline, ierr)
  end do
  
contains

  subroutine parse_config_line(keyword, splitline, ierr)
    use state, only : debug, istep
    use checkpointing, only : checkpoint_timestep
    use norms, only : debugnormlevel, debugnorm
    implicit none
    character(*) keyword, splitline(:)
    integer ierr,j
    
    select case(keyword)
    case('debug')
        read(splitline(2),*,err=1,end=1) debug
        if(master) write(*,'(a,i0)') "RUNTIME_CONTROL changing DEBUG level to ",debug
    case('debugnormlevel')
        read(splitline(2),*,err=1,end=1) debugnormlevel
        if(master) write(*,'(a,i0)') "RUNTIME_CONTROL changing DEBUGNORMLEVEL to ",debugnormlevel
    case('debugnorm')
        read(splitline(2),*,err=1,end=1) debugnorm
        if(master) write(*,'(a,i0)') "RUNTIME_CONTROL changing DEBUGNORM to ",debugnorm
    case('checkpoint_timestep')
        read(splitline(2),*,err=1,end=1) checkpoint_timestep
        if(master) write(*,'(a,i0)') "RUNTIME_CONTROL changing CHECKPOINT_TIMESTEP to ",checkpoint_timestep
        if(checkpoint_timestep < istep .and. master) then
            write(*,*) "WARNING, RUNTIME_CONTROL cannot set checkpoints backwards in time"
        end if        
    case default
        if(master) write(*,*) "unknown run time keyword=", keyword
    end select
    return
1   if(master) write(*,*) 'WARNING: RUNTIME_CONTROL failed to read keyword=',keyword," from runtime.dat"
  end subroutine parse_config_line
  
end subroutine runtime_control
