subroutine sanity_check
!
! add all useful tests (and a corresponding useful message :) that can
! identify a problem to this routine
!
! in the future we may want to distill this file into a smallish runtime
! version, and a more comprehensive one to use during setup.

  use state
  use control_parameters  
  use mod_bound
  use norms, only : parsum
  use setup2, only : rhoback_set, rhobackfromrho
  
  implicit none
  
! stop_flag is a list of flags triggered, this integer is prepended to its corresponding message
! smline is the number of errormessages in the buffer stop_message
! ss is a temporary buffer for storing error messagelines to be copied into stop_message.
! just before abort, each processor dumps the buffer to file.
  
  integer :: stop_flag(300)=0, numstopflags=0, smline=0, kverk
  character(300) :: ss(10), stop_message(300)
  character(10) tmps
  real aveta, amlime, amlim, c
  integer i,j,k
  real tmp1, tmp2
  real, pointer :: tmp2d(:,:)
  integer loc(3), maxl(3), minl(3)

  allocate(tmp2d(i1:i2,j1:j2))
  tmp2d = eta*fsm  
  tmp1 = parsum(tmp2d)
  deallocate(tmp2d)
  tmp2 = parsum(fsm)
  aveta = tmp1/tmp2
  
  if( abs(aveta) > 0.1 .and.master) then
      write(*,*)'Warning: it seems like the average ETA is not close to 0. aveta=', aveta
  end if

  if(.not.bom_allocated.and.master) then
      write(*,*)'no arrays allocated, impossible to proceed. make sure im,jm,kb are set'
      call abort()
  end if

  if(lb < 0 .and. num_zones_east+num_zones_west+num_zones_north+num_zones_south > 0 .and.master) then
      write(*,*)'Warning: LB is not set, and you have open boundaries. make sure this is intended'
  end if
  
  call find_max_reals(1, (h+eta), tmp1, loc, root=0)
  if(tmp1 < 0.0 .and.master) then
      write(ss(1),*)'ERROR: surface below bottom; minloc(h+eta) = ', &
           minloc(h+eta), 'minval(h+eta)=',minval(h+eta)
      call appendstopflag(1)
  end if

  call find_max_reals(1, eta, tmp1, loc, root=0)
  if(tmp1 > 0.01*depmax .and.master) then
      write(ss(1),*)'ERROR: max eta > 1% of DEPMAX, max(eta)=', tmp1
      call appendstopflag(2)
  end if
  call find_min_reals(1, eta, tmp2, loc)
  if(abs(tmp2) > 0.01*depmax .and.master) then
      write(ss(1),*)'ERROR: min eta > 1% of DEPMAX, min(eta)=', tmp2
      call appendstopflag(3)
  end if

  call find_max_reals(1, eta_bound, tmp1, loc, root=0)
  if(tmp1 > 0.01*depmax .and.master) then
      write(ss(1),*)'ERROR: max eta_bound > 1% of DEPMAX, max(eta_bound)=', tmp1
      call appendstopflag(4)
  end if
  call find_min_reals(1, eta_bound, tmp2, loc)
  if(abs(tmp2) > 0.01*depmax .and.master) then
      write(ss(1),*)'ERROR: min eta_bound > 1% of DEPMAX, min(eta_bound)=', tmp2
      call appendstopflag(5)
  end if

  
  if( mod(n2d,2).ne.0.and.master) then
      write(ss(1),*)'ERROR: n2d must be an even number'
      call appendstopflag(6)
  end if

  if( n2d <0 .or. n2d > 400 .and. master) then
      write(ss(1),*)'ERROR: n2d looks wrong n2d=',n2d
      call appendstopflag(7)
  end if

  if( ndivis<0 .and. master) then
      write(ss(1),*)'ERROR: ndivis must be positive'
      call appendstopflag(8)
  end if

  if( mod(ndivis,2).ne.0 .and. master) then
      write(ss(1),*)'ERROR: ndivis must be an even number'
      call appendstopflag(9)
  end if

  if(dx<0.0 .or. dy<0.0 .and. master) then
      write(ss(1),*)'ERROR: dx and dy must be positive'
      write(ss(2),*)'dx=',dx,' dy=',dy
      call appendstopflag(10,2)
  end if

! the distance from the bottom to the closest velocity point must be > z0
  do j=1,j_eq_jm
      do i=1,i_eq_im
          if(h(i,j) > 0.0 .and. h(i,j)*(zz(kb-1)-z(kb)) < z0) then
              write(ss(1),*)'zb=', h(i,j)*(zz(kb-1)-z(kb)),' z0=',z0,' i,j=',i,j
              write(ss(2),*)'ERROR: zB < z0, the model for bottom drag will break down. Bottom cell needs to be thicker or z0 smaller.'
              call appendstopflag(11,2)
          end if
      end do
  end do
  
  if(advturb .and. vertical_closure.ne.1) then
      write(ss(1),*)'ERROR: advection of turbulence only makes sense if vertical closure is 1, i.e. mellor yamada'
      call appendstopflag(13)
  end if
  
  if(advect_density .and. (advect_s .or. advect_t)) then
      write(ss(1),*)'ERROR: the selected advection combination is not supported:'
      write(ss(2),*)'advect_density=',advect_density
      write(ss(3),*)'advect_s=', advect_s
      write(ss(4),*)'advect_t=', advect_t
      call appendstopflag(14,4)
  end if

  if(eos .ne. "wang" .and. eos .ne. "unesco" .and. eos.ne."user") then
      write(ss(1),*)'Invalid value for EOS:', trim(eos), 'must be one of "user", "wang" or "unesco"'
      call appendstopflag(15)
  end if
  
  if(internal_pressure .ne. "pom" .and. internal_pressure .ne. "fourth" .and. internal_pressure .ne. "balanced") then
      write(ss(1),*)'Invalid value for INTERNAL_PRESSURE:', internal_pressure, 'must be one of "pom","fourth" or "balanced"'
      call appendstopflag(16)
  end if

  if(internal_pressure == "balanced" .and. .not. (rhoback_set .or. rhobackfromrho)) then
      write(ss(1),*)"internal_pressure set to balanced, but RHOBACK has not been initialized"
      write(ss(2),*)"use keyword RHOBACK or RHOBACK_FROM_INITIALDENSITY for that."
      call appendstopflag(17,2)
  end if

  
  if(master) then
      c = sqrt(2*grav*depmax)
      write(*,*)'The 2-D Courant number is',(dt/n2d)/(dx/c)
  
    if( dt/n2d > 0.9*dx/c ) then
      write(*,*)'rather large...'
      write(*,*)'DX=',dx
      write(*,*)'DT=',dt
      write(*,*)'max(H)=', depmax
      write(*,*)'C=', c,'m/s'
      write(*,*)'N2D=',n2d
      write(*,*)'with this N2D, you could try NDIVIS between ', &
             nint(c*3600./(dx*real(n2d)*0.8))  &
           ,' and ', &
             nint(c*3600./(dx*real(n2d)*0.1))
    end if

  if( 3600./(ndivis*n2d) < 0.1*dx/c  .and. master ) then
      write(*,*)'rather small...'
      write(*,*)'DX=',dx
      write(*,*)'DT=',dt
      write(*,*)'max(H)=', depmax
      write(*,*)'C=', c,'m/s'
      write(*,*)'N2D=',n2d
      write(*,*)'with this N2D, try NDIVIS between ', &
             nint(c*3600./(dx*real(n2d)*0.8))  &
           ,' and ', &
             nint(c*3600./(dx*real(n2d)*0.1))
  end if

  end if


  
  if(.not.advect_density)then

      call find_max_reals(kb, s, tmp1, maxl, wet=.true., root=0)
      call find_min_reals(kb, s, tmp2, minl, wet=.true., root=0)

      if(master)then
          if(tmp2<0. .or. tmp1>50.) then
              write(ss(1),*)'ERROR: S looks wrong: max(S)=',tmp1,'min(S)=',tmp2
              write(ss(2),*)'maxloc=',maxl,' minl=',minl
              call appendstopflag(20,2)
          else if(tmp2<2. .or. tmp1>45.) then
              write(*,*)'WARNING: S looks suspicios: max(S)=',tmp1,'min(S)=',tmp2
              write(*,*)'maxloc=',maxl,' minl=',minl
              write(*,*)'you may want to check your data!'
          end if
      end if
  
      call find_max_reals(kb, t, tmp1, maxl, wet=.true., root=0)
      call find_min_reals(kb, t, tmp2, minl, wet=.true., root=0)

      if(master)then
          if(tmp2 < -2. .or. tmp1 > 40.) then
              write(ss(1),*)'ERROR: T looks wrong: max(T)=',tmp1,'min(t)=',tmp2
              write(ss(2),*)'maxloc=',maxl,' minl=',minl
              call appendstopflag(21,2)
          end if
      end if

      call find_max_reals(kb, s_bound, tmp1, loc, wet=.true., root=0)
      call find_min_reals(kb, s_bound, tmp2, loc, wet=.true., root=0)

      if(master)then
          if(tmp2 < 0. .or. tmp1 > 50.) then
              write(ss(1),*)'ERROR: S_BOUND looks wrong: max(S_BOUND)=',tmp1,'min(S_BOUND)=',tmp2
              call appendstopflag(22)
          end if
      end if

      call find_max_reals(kb, t_bound, tmp1, loc, wet=.true., root=0)
      call find_min_reals(kb, t_bound, tmp2, loc, wet=.true., root=0)

      if(master)then
          if(tmp2 < -2. .or. tmp1 > 40.) then
              write(*,*)'ERROR: T_BOUND looks wrong: max(T_BOUND)=',tmp1,'min(t_bound)=',tmp2
              call appendstopflag(23)
          end if
      end if
      
  end if

  if(theta2d<0.5 .or. theta2d>1.0) then
      if(master)write(ss(1),*)'ERROR: theta2d must be in the interval 0.5 to 1.0, theta2d=',theta2d
      call appendstopflag(24)
  end if

!    if(theta3d<0.5 .or. theta3d>1.0) then
!        write(ss(1),*)'ERROR: theta3d must be in the interval 0.5 to 1.0, theta3d=',theta3d
!        call appendstopflag(25)
!    end if

  call find_min_reals(1, cbc, tmp1, loc, root=0)

  if(tmp1 < 0 .and. master) then
      write(ss(1),*)'ERROR: minval(cbc)=',tmp1,', cbc must not be negative'
      call appendstopflag(26)
  end if
  
  call find_max_reals(kb, rho, tmp1, loc, wet=.true., root=0)
  if(master) tmp1 = tmp1 + 1000.0 + rhoaver  
  call find_min_reals(kb, rho, tmp2, loc, wet=.true., root=0)
  if(master) tmp2 = tmp2 + 1000.0 + rhoaver
  
  if(master) then
      if(tmp2 < 900. .or. tmp1 > 1050.) then
          write(ss(1),*)'ERROR: RHO looks wrong: max(RHO)=',tmp1,'min(rho)=',tmp2
          call appendstopflag(27)
      end if
  end if
  
  if(advect_density)then
      call find_max_reals(kb, rho_bound, tmp1, loc, wet=.true., root=0)
      if(master)tmp1 = tmp1 + 1000.0 + rhoaver  
      call find_min_reals(kb, rho_bound, tmp2, loc, wet=.true., root=0)
      if(master)tmp2 = tmp2 + 1000.0 + rhoaver

      if(master)then
          if(tmp2 < 900. .or. tmp1 > 1050.) then
              write(ss(1),*)'ERROR: RHO_BOUND looks wrong: max(rho_bound)=',tmp1,'min(rho_bound)=',tmp2
              call appendstopflag(28)
          end if
      end if
  end if
  
  call find_max_reals(kb, (sqrt(u**2+v**2)), tmp1, loc, wet=.true., root=0)
  if(tmp1 > 10.0 .and.master) then
      write(ss(1),*)'ERROR: speed looks too high. maxval(sqrt(u**2+v**2))=',tmp1
      call appendstopflag(29)
  end if

  call find_max_reals(kb, (sqrt(u_bound**2+v_bound**2)), tmp1, loc, wet=.true., root=0)
  if(tmp1 > 10.0 .and.master) then
      write(ss(1),*)'ERROR: boundary value for speed looks too high. maxval(sqrt(u_bound**2+v_bound**2))=',tmp1
      call appendstopflag(30)
  end if

  
  amlime = 0.0625*dx**2/(dt/n2d)
  amlim =  0.0625*dx**2/dt

  if(master) then
      write(*,*) "Max limit for AM=", amlim
      write(*,*) "Max limit for AM2D=", amlime
  end if

  if(smag .and. ammin>amlim) then
      if(master)then
          write(ss(1),*) "AMMIN > AMLIM (0.0625*dx**2/dt)  makes no sense!"
          call appendstopflag(31)
      end if          
  end if
  if(smag .and. am2dmin>amlime) then
      if(master)then
          write(ss(1),*) "AM2DMIN > AMLIME (0.0625*dx**2/dt/n2d)  makes no sense!"
          call appendstopflag(32)
      end if          
  end if

  
  if(.not.smag) then
      if(n2d>0 .and. dt>0.0) then
          amlime = 0.0625*dx**2/(dt/n2d) 
          amlim =  0.0625*dx**2/dt

          call find_max_reals(kb, am, tmp1, loc, root=0)
          call find_max_reals(1, am2d, tmp2, loc, root=0)
                    
          if(master)then
              if(tmp1 > amlim .or. tmp2 > amlime) then
                  write(ss(1),*)'AM or AM2D has values that are larger than the recommended maximum:'
                  write(ss(2),*)'max(AM)=',tmp1
                  write(ss(3),*)'max(AM2d)=',tmp2
                  call appendstopflag(33,3)
              end if
          end if
      else
          write(ss(1),*)'could not test AM, AM2D. N2D, DT:',n2d, dt
          call appendstopflag(33)
      end if
  end if
  
  if(maxval(z) > 0.00001 .or. minval(z) < -1.00001) then
      write(ss(1),*)'Z looks wrong, z(1),z(2)...z(kb):'
      write(tmps,'(i0)') kb
      write(ss(2),"("//trim(tmps)//"(f0.2,1x))",err=101) z(1:kb)
101   call appendstopflag(34,2)
  end if

  call check_rivers()

  call check_wells()

  call check_wind()

  call check_forcing()

  call check_boundary_conditions("east", num_zones_east, bin_east, boeast)
  call check_boundary_conditions("west", num_zones_west, bin_west, bowest)
  call check_boundary_conditions("north", num_zones_north, bin_north, bonorth)
  call check_boundary_conditions("south", num_zones_south, bin_south, bosouth)


  kverk=0
  if(any(stop_flag > 0) .and. sanity_check_fatal) kverk=1
  call reduce('sum',kverk)

  if(kverk > 0) then
      if(master) write(*,*) '******** WHOOPS! *********'
      if(master) write(*,*) 'The almighty sanity_check routine found issues!'
      if(master) write(*,*) 'The actual error message(s) are/should be dumped from each processor into files called "stopflag.<processorid>'
      if(master) write(*,*) 'There are two options to proceed;'
      if(master) write(*,*) '1) Fix the reported problem: the input file could be silly, or, theoretically, there could be a bug in the code'
      if(master) write(*,*) '2) Trust yourself and make the model ignore the problem: add "SANITY_CHECK_FATAL F" to the setup file and restart'

      call dumpstopflags()
      call barrier()
      call stop_qmpi()
      call deallocate_bom()
      stop
  end if

contains

  subroutine appendstopflag(i,j)
! add i to list of error codes, append j messages from ss to stop_message    
    implicit none
    integer i,n
    integer, optional :: j
    character(5) marg    
   
    numstopflags = numstopflags+1
    stop_flag(numstopflags) = i
    write(marg,'(i0)') i

    n=1
    if(present(j)) n=j
    do k=1,n
        stop_message(smline+k) = marg // trim(ss(k))
    end do
    smline = smline + n    
  end subroutine appendstopflag

  subroutine dumpstopflags()
    character(4) proc
    if(smline==0) return
    write(proc,'(i4.4)') myid
    open(10000002,file="stopflag."//proc)
    write(10000002,*) 'This is a list of stop flags and messages triggered on processor ',proc
    do i=1,smline
        write(10000002,'(a)') trim(stop_message(i))
    end do
    close(10000002)
  end subroutine dumpstopflags
  
  subroutine binerr(mess)
    implicit none
    character(*) mess
    if(master) write(ss(1),'(2x,a)') mess
    call appendstopflag(34)
  end subroutine binerr
  
  subroutine check_boundary_conditions(name, nz, bin, frs)
! this is the place to add post initialization checks for the boundary objects.    
    implicit none
    character(*) name
    type(binreg_object) bin(:)
    type(boundary_object) frs(:)    
    integer nz,i
    
    do i=1,nz
        if(frs(i)%u_type == 4 .and. .not. bin(i)%initialized)   call binerr("bc_u "//name//" type 4, but no binreg specified")
        if(frs(i)%v_type == 4 .and. .not. bin(i)%initialized)   call binerr("bc_v "//name//" type 4, but no binreg specified")
        if(frs(i)%w_type == 4 .and. .not. bin(i)%initialized)   call binerr("bc_w "//name//" type 4, but no binreg specified")
        if(frs(i)%s_type == 4 .and. .not. bin(i)%initialized)   call binerr("bc_s "//name//" type 4, but no binreg specified")
        if(frs(i)%t_type == 4 .and. .not. bin(i)%initialized)   call binerr("bc_t "//name//" type 4, but no binreg specified")
        if(frs(i)%ua_type == 4 .and. .not. bin(i)%initialized)  call binerr("bc_ua "//name//" type 4, but no binreg specified")
        if(frs(i)%va_type == 4 .and. .not. bin(i)%initialized)  call binerr("bc_va "//name//" type 4, but no binreg specified")
        if(frs(i)%eta_type == 4 .and. .not. bin(i)%initialized) call binerr("bc_eta "//name//" type 4, but no binreg specified")
        if(frs(i)%q2_type == 4 .and. .not. bin(i)%initialized)  call binerr("bc_q2 "//name//" type 4, but no binreg specified")
        if(frs(i)%q2l_type == 4 .and. .not. bin(i)%initialized) call binerr("bc_q2l "//name//" type 4, but no binreg specified")
        if(frs(i)%rho_type == 4 .and. .not. bin(i)%initialized) call binerr("bc_rho "//name//" type 4, but no binreg specified")
    end do
  end subroutine check_boundary_conditions

  
  subroutine check_forcing()
    use mod_forcing

! check that all forcing times are in ascending order.    
    do i=1, numforcings
        do j=2, forcing(i)%numfields
            if(forcing(i)%field(j)%time <= forcing(i)%field(j-1)%time)then
                if(master) then
                    write(ss(1),*)
                    write(ss(2),*) "ERROR: forcing name=", forcing(i)%name," field number=", j,":"
                    write(ss(3),*) "  forcing time=",      forcing(i)%field(j)%time, &
                         ", while previous time was ", forcing(i)%field(j-1)%time
                    write(ss(4),*) "you should check times in the first column in file ", forcing(i)%file_name
                    call appendstopflag(35,4)
                end if
            end if
        end do
    end do
  end subroutine check_forcing
  

  subroutine check_rivers()
    integer river,li,lj,river2,li2,lj2
    logical inblock,inblock2,file_present
    
    do river=1,numc
        i = iriver(river) ; j = jriver(river)
        li=i              ; lj=j
        call globalind2local(li,lj,inblock,ghost=.false.)

        ! check for duplicate rivers.
        do river2=1,numc
            li2 = iriver(river2) ; lj2 = jriver(river2)
            call globalind2local(li2,lj2,inblock2)
            if(river == river2) cycle
            if((inblock .or. inblock2) .and. li==li2 .and. lj==lj2 .and. coderiver(river)==coderiver(river2)) then
                write(ss(1),*)'ERROR: river "', trim(river_name(river)),'" number=',river
                write(ss(2),*)'  same cell and code as river "',trim(river_name(river2)),'" number=',river2
                write(ss(3),*)'  li,lj,li2,lj2',li,lj,li2,lj2,' gi,gj=',block%fi+li-1,block%fj+lj-1
                call appendstopflag(36,3)
            end if            
        end do
        
        if(i>im .or. i<1 .or. j<1 .or. j>jm) then
            if(master)write(ss(1),*)'ERROR: river=', trim(river_name(river))," is outside current grid; i,j=", i,j
            call appendstopflag(37)
        end if

        if(inblock.and.fsm(li,lj) < 0.5) then
            write(ss(1),*)myid,'ERROR: river=', trim(river_name(river))," is located on land, fsm=", fsm(li,lj),i,j
            call appendstopflag(38)
        end if
        
        if(i==1 .or. i==im .or. j==1 .or. j==jm) then
            if(inblock)then
                write(ss(1),*)'ERROR: river=', trim(river_name(river))," is located on the edge (probably not allowed CHECK)"
                call appendstopflag(39)
            end if
        end if

        if(riverformat(river)==0) then
            if(friver(river) < 0.0) then
                if(master)write(*,*)'NOTE, river=', trim(river_name(river))," has negative flux"
            end if
            if(.not.advect_density)then
                if(sriver(river) < 0.0 .or. sriver(river) > 50) then
                    if(master)write(ss(1),*)'ERROR, river=', trim(river_name(river))," salinity looks wrong, S=", sriver(river)
                    call appendstopflag(40)
                end if
            else
                if(sriver(river)+1000+rhoaver < 950.0 .or. sriver(river)+1000+rhoaver > 1050.0) then
                    if(master)write(ss(1),*)'ERROR, river=', trim(river_name(river))," density looks wrong, RHO=", sriver(river)
                    call appendstopflag(41)
                end if
            end if
            
            if(triver(river) < 0.0 .or. triver(river)>40) then
                if(master)write(ss(1),*)'ERROR, river=', trim(river_name(river))," temperature looks wrong, T=", triver(river)
                call appendstopflag(42)
            end if
        end if

        if(inblock)then
            
! check river code, and if neighbour cell is land.
        select case(coderiver(river))
        case(1)
            if(fsm(li-1,lj) > 0.5) then
                write(ss(1),'(a,a,a,i0,a,i0,a,f0.7,a,i0)') 'ERROR: river=',trim(river_name(river)), &
                     ' coderiver is 1, but western grid box is wet, fsm(',i-1,",",j,")=",fsm(li-1,lj),' myid=',myid
                call appendstopflag(45)
            end if
        case(2)
            if(fsm(li,lj-1) > 0.5) then
                write(ss(1),'(a,a,a,i0,a,i0,a,f0.7,a,i0)') 'ERROR: river=',trim(river_name(river)), &
                     ' coderiver is 2, but southern grid box is wet, fsm(',i,",",j-1,")=",fsm(li,lj-1),' myid=',myid
                call appendstopflag(46)
            end if
        case(3)
            if(fsm(li+1,lj) > 0.5) then
                write(ss(1),'(a,a,a,i0,a,i0,a,f0.7,a,i0)') 'ERROR: river=',trim(river_name(river)), &
                     ' coderiver is 3, but eastern grid box is wet, fsm(',i+1,",",j,")=",fsm(li+1,lj),' myid=',myid
                call appendstopflag(47)
            end if
        case(4)
            if(fsm(li,lj+1) > 0.5) then
                write(ss(1),'(a,a,a,i0,a,i0,a,f0.7,a,i0)') 'ERROR: river=',trim(river_name(river)), &
                     ' coderiver is 4, but northern grid box is wet, fsm(',i,",",j+1,")=",fsm(li,lj+1),' myid=',myid
                call appendstopflag(48)
            end if
        case default
            write(ss(1),*)'ERROR: river=', trim(river_name(river))," has code=", coderiver(river)," which is unknown"
            call appendstopflag(49)
        end select

        end if
    end do
  end subroutine check_rivers


  subroutine check_wells()
    implicit none
    integer well,i,j,li,lj
    logical inblock,file_present

    do well=1,numwell          
        i=iwell(well)
        j=jwell(well)

        li=i
        lj=j
        call globalind2local(li,lj,inblock)       
        
        if(i>im .or. i<1 .or. j<1 .or. j>jm) then
            if(master)write(ss(1),*)'ERROR: well=', trim(well_name(well))," is outside current grid; i,j=", i,j
            call appendstopflag(50)
        end if

        if(inblock .and. fsm(li,lj) < 0.5) then
            write(ss(1),*)myid,'ERROR: well=', trim(well_name(well))," is located on land, fsm=", fsm(li,lj)
            call appendstopflag(51)
        end if

        if(well_format(well)==0) then
            if(fwell(well) < 0.0) then
                if(master)write(*,*)'NOTE, well=', trim(well_name(well))," has negative flux"
            end if
            if(swell(well) < 0.0 .or. swell(well)>50) then
                if(master)write(ss(1),*)'ERROR, well=', trim(well_name(well))," salinity looks wrong, S=", swell(well)
                call appendstopflag(52)
            end if
            if(twell(well) < 0.0 .or. twell(well)>40) then
                if(master)write(ss(1),*)'ERROR, well=', trim(well_name(well))," temperature looks wrong, T=", twell(well)
                call appendstopflag(53)
            end if
        end if
    end do
  end subroutine check_wells

  
  subroutine check_wind
    logical file_present

    if(abs(wind1_hourly_rotation) > 360.) then
        if(master)write(ss(1),*)'wind type 1 rotation rate=',wind1_hourly_rotation,'looks too high. fix setupfile or slack this test.'
        call appendstopflag(56)
    end if
    if(wind1_amplitude<0 .or.wind1_amplitude> 60) then
        if(master)write(ss(1),*)'wind type 1 speed=',wind1_amplitude,' are you sure?  fix setupfile or slack this test'
        call appendstopflag(57)
    end if
    if(wind1_period<0) then
        if(master)write(ss(1),*)'wind type 1 period must be > 0, now=',wind1_period
        call appendstopflag(58)
    end if
    if(wind1_rampup_time<0) then
        if(master)write(ss(1),*)'wind type 1 rampup time must be > 0, now=',wind1_rampup_time
        call appendstopflag(59)
    end if
    if(wind2_radius<0 .or. wind2_radius>10000000) then
        if(master)write(ss(1),*)'wind type 2 radius looks wrong:',wind2_radius,' (earth D~13000km)'
        call appendstopflag(60)
    end if
    if(wind2_rampup_time<0) then
        if(master)write(ss(1),*)'wind type 2 rampup time must be > 0, now=',wind2_rampup_time
        call appendstopflag(61)
    end if
    if(wind2_strength>0) then
        if(master)write(ss(1),*)'wind type 2 low pressure strength should be negative, now=', wind2_strength
        call appendstopflag(62)
    end if
    if(wind_type==3) then
        if(master)then
            inquire(file=trim(wind3_data_file), exist=file_present)
            if(.not.file_present) then
                write(ss(1),*)'could not find wind type 3 data file=', trim(wind3_data_file)
                call appendstopflag(63)
            end if
        end if
    end if
  end subroutine check_wind
  
end subroutine sanity_check

