MODULE STATE
  use mod_mpi

! the global grid dimension in the i,j and k (sigma) direction  
  INTEGER :: IM=-1,JM=-1,KB=-1

! Istep is the current time step, numstep the total number of
! timesteps, n2d is the number of 2d time steps per 3d step, ndivis is
! the number of time steps per hour.
  
  INTEGER :: ISTEP=0, NUMSTEP=-1
  INTEGER :: N2D=-1, NDIVIS=-1

! BOM_ALLOCATED is true after allocate_bom has been called, and it is
! safe to assign to arrays.
  LOGICAL :: BOM_ALLOCATED=.FALSE.
  
! How much debug info to view?
! I suggest 0 for nothing, 1 for useful production type info, 2 for debugging, 3 debugging for developers etc.
  INTEGER :: DEBUG=1

! The elliptic solver takes sor_numiter iterations by default, unless
! it reaches the answer before.
  INTEGER :: SOR_NUMITER = -1
! if the relative residual SOR_RESID is reached, stop.  
  REAL :: SOR_RESID = -1
! if > 0, enforce average P to be 0 every AVPEQ0'th step.
  INTEGER :: AVPEQ0 = 0

  integer depmaxloc(3), depminloc(3)
  
!
! Scalars
!
  INTEGER :: NUMWET=0
  REAL EKIN,VOL,MASS,EETA,WCOURANT
  REAL :: KHMIN=1.e-7,KMMIN=1.e-5
  REAL :: DX=-1.,DY=-1.,DT=-1.,CM=1.0,CM2D=5.0,CH=0.0,RHOAVER,SAVER,TAVER
  REAL :: THETA2D=0.5
  REAL :: GRAV = 9.81 , RHO0=1025.0, GIVEN_MASS=-1.0
  REAL :: CD = 0.0025, Z0 = 0.01, PMEAN = 101300., ROAIR = 1.3, CDW=0.0, CDLAND=0.0
  REAL :: DEPMAX=-1.0, DEPMIN=10000.0, ncalfa_dte0=-1.0
  REAL :: LX=-1.0, LY=-1.0
  REAL :: XPOLE=0, YPOLE=0, YLONG=0, MINLON, MAXLON, MINLAT, MAXLAT
  
! minimum values for smagorinsky visc., default 0.
  REAL :: AMMIN=0.0, AM2DMIN=0.0
  
!! constants that denote location of point within box/cell
  integer, parameter :: eta_point=1, u_point=2, v_point=3
  
!
!1-D real arrays.
!
  REAL, DIMENSION(:),POINTER ::  Z,DZ,DZR,DZZ,ZZ
!
!2-D arrays
!
  REAL, DIMENSION(:,:),POINTER :: COR,FSM,DUM,DVM,    &
      & ETA,ETAP,H,HU,HV,D,DU,DV,WUSURF,WVSURF, &
      & SSURF,TSURF,RHOSURF,WUBOT,WVBOT,CBC,PATM,AM2D,  &
      & UA,VA,HUSQR,HVSQR,LONGITUDE,LATITUDE, &
      & SPONGE_CM2D,SPONGE_CM,SPONGE_KM,SPONGE_KH,COAST,&
        SPONGE_KH_C1
!
!3-D arrays
!
  REAL, DIMENSION(:,:,:),POINTER :: U,V,W,WR,S,T,RHO, &
      & AM,AH,KM,KH,KQ,DDZ,DUDZ,DVDZ,    &
      & UADV,VADV,WADV,Q2,Q2L,DELTAU,DELTAV
!
!  Arrays in version 4.0
!
  REAL, DIMENSION(:,:,:),POINTER :: UN,UNM1,DELUN,DELUSTAR,DELUNM1
  REAL, DIMENSION(:,:,:),POINTER :: VN,VNM1,DELVN,DELVSTAR,DELVNM1
  REAL, DIMENSION(:,:,:),POINTER :: SN,SNM1,DELSN,DELSSTAR,DELSNM1
  REAL, DIMENSION(:,:,:),POINTER :: TN,TNM1,DELTN,DELTSTAR,DELTNM1
  REAL, DIMENSION(:,:,:),POINTER :: WN,WNM1,DELWN,DELWSTAR,DELWNM1
  REAL, DIMENSION(:,:,:),POINTER :: RHON,RHONM1,DELRHON,DELRHOSTAR,DELRHONM1

!
!  3-D dynamic pressure for NONHYD
!
   REAL, DIMENSION(:,:,:),POINTER :: P
!
   REAL, DIMENSION(:,:,:),POINTER :: RHOBACK, SOPTU, SOPTV, MASKSOPTU, MASKSOPTV

END MODULE STATE 
