! This file is included in setup2.f90 in order to inherit different
! kind's, interface included for the sake of completeness
!  
! SUBROUTINE TANHPROFILE(F, FMAX, FMIN, DELTAZ, ZSYMM, R)
!
!  Initialize a 3d array using a vertical tanh profile
!  
!    implicit none
!    integer, parameter :: localkind=...
    real(localkind) f(1-ghost_wi:,1-ghost_wj:,:)
    real(localkind), optional :: fmax, fmin, deltaz, zsymm
    integer, optional :: r(:)
    real(localkind) realfmax, realfmin, realdeltaz, realzsymm, dep, deltaf
    integer i,j,k,is,ie,js,je,ks,ke,lis,lie,ljs,lje
    real(localkind) tmpdep
    logical inblock

    if(.not.z_set) then
        call message('sigma layers must be initialized before you do code 6 operations!',2)
    end if

! defaults that are ok for salinity (around Bergen Norway...)
    realfmax = 34._localkind
    realfmin = 30._localkind
    realdeltaz = 3._localkind
    realzsymm = -8._localkind

    if(present(fmax)) realfmax=fmax
    if(present(fmin)) realfmin=fmin
    if(present(deltaz)) realdeltaz=deltaz
    if(present(zsymm)) realzsymm=zsymm

    ks=1  ;  ke=gkb  ;  js=1  ;  je=gjm ;  is=1  ;  ie=gim
    if(present(r)) then
        is=r(1)  ;  ie=r(2)  ;  js=r(3)  ;  je=r(4) ;  ks=r(5) ;  ke=r(6)
    end if

    if(master.and.debug>0) then
        tmpdep = -realzsymm*2.0_localkind
        write(*,*)' Stratification from TANH profile:'
        write(*,*)' K, Depth, value for H=', tmpdep
        do k=ks,ke
            dep = zz(k)*tmpdep
            deltaf = 0.5_localkind*(realfmax-realfmin)
            write(*,'(i3,1x,f7.1,1x,f12.4)') k, dep, 0.5_localkind*(realfmax+realfmin) - deltaf*tanh((dep-realzsymm)/realdeltaz)
        end do
        write(*,*)
    end if

    call  globalrect2local(is,ie,js,je,lis,lie,ljs,lje,inblock)

    if(inblock)then
        do k = ks,ke
            do j = ljs,lje
                do i = lis,lie
                    if(h(i,j) > 0.00001 .and. k<kb) then
                        dep = zz(k)*h(i,j)
                        deltaf = 0.5*(realfmax-realfmin)
                        f(i,j,k) = 0.5*(realfmax+realfmin) - deltaf*tanh((dep-realzsymm)/realdeltaz)
                    end if
                end do
            end do
        end do
    end if
!  end subroutine tanhprofile
