SUBROUTINE TIDEFRS(IS)
  USE STATE
  USE MOD_BOUND
  USE TIME_UTIL  
  IMPLICIT NONE
  INTEGER IS,I,J,K,IC,JC,ZI,R(4)
  REAL TIMESEC
!
!  TIDEFRS updates the fields ETA_TIDE, UA_TIDE, VA_TIDE, UTFRS,VTFRS
!  as a function of time and the amplitudes and phases of the
!  tidal components

  
! the "2d" time in seconds
  TIMESEC = TIME - DT + DT*IS/N2D

  eta_tide = 0.0
  ua_tide = 0.0
  va_tide = 0.0
  
  do zi=1,num_zones_west
      if(.not. bowest(zi)%inblock) cycle
      r = bowest(zi)%local_range
      select case(tidetype_west(zi))
      case(0,1)
          ! pick a reference point IN the given zone, and compute the
          ! constant tidal elevation to be applied to the whole zone
          ! there
          ic=r(1)
          jc=(r(3)+r(4))/2
          do k=1, kcon_west(zi)
              eta_tide(ic,jc) = eta_tide(ic,jc) + etatamp(ic,jc,k)*cos(freq_west(zi,k)*timesec - etatpha(ic,jc,k))
              ua_tide(ic,jc) = ua_tide(ic,jc) + uatamp(ic,jc,k)*cos(freq_west(zi,k)*timesec - uatpha(ic,jc,k))
              va_tide(ic,jc) = va_tide(ic,jc) + vatamp(ic,jc,k)*cos(freq_west(zi,k)*timesec - vatpha(ic,jc,k))
          end do
          do j=r(3),r(4)
              do i=r(1),r(2)
                  eta_tide(i,j) = fsm(i,j)*eta_tide(ic,jc)
                  ua_tide(i,j) = fsm(i,j)*ua_tide(ic,jc)
                  va_tide(i,j) = fsm(i,j)*va_tide(ic,jc)
              end do
          end do
      case(2)
! compute in all points
          do k=1, kcon_west(zi)
              do j=r(3),r(4)
                  do i=r(1),r(2)
                      eta_tide(i,j) = eta_tide(i,j) + etatamp(i,j,k)*cos(freq_west(zi,k)*timesec - etatpha(i,j,k))
                      ua_tide(i,j) = ua_tide(i,j) + uatamp(i,j,k)*cos(freq_west(zi,k)*timesec - uatpha(i,j,k))
                      va_tide(i,j) = va_tide(i,j) + vatamp(i,j,k)*cos(freq_west(zi,k)*timesec - vatpha(i,j,k))
                  end do
              end do
          end do
      case default
          call abort("tidal forcing type not implemented")
      end select      
  end do
  do zi=1,num_zones_east
      if(.not. boeast(zi)%inblock) cycle
      r = boeast(zi)%local_range
      select case(tidetype_east(zi))
      case(0,1)
          ic=r(2)
          jc=(r(3)+r(4))/2
          do k=1, kcon_east(zi)
              eta_tide(ic,jc) = eta_tide(ic,jc) + etatamp(ic,jc,k)*cos(freq_east(zi,k)*timesec - etatpha(ic,jc,k))
              ua_tide(ic,jc) = ua_tide(ic,jc) + uatamp(ic,jc,k)*cos(freq_east(zi,k)*timesec - uatpha(ic,jc,k))
              va_tide(ic,jc) = va_tide(ic,jc) + vatamp(ic,jc,k)*cos(freq_east(zi,k)*timesec - vatpha(ic,jc,k))
          end do
          do j=r(3),r(4)
              do i=r(1),r(2)
                  eta_tide(i,j) = fsm(i,j)*eta_tide(ic,jc)
                  ua_tide(i,j) = fsm(i,j)*ua_tide(ic,jc)
                  va_tide(i,j) = fsm(i,j)*va_tide(ic,jc)
              end do
          end do
      case(2)
! compute in all points
          do k=1, kcon_east(zi)
              do j=r(3),r(4)
                  do i=r(1),r(2)
                      eta_tide(i,j) = eta_tide(i,j) + etatamp(i,j,k)*cos(freq_east(zi,k)*timesec - etatpha(i,j,k))
                      ua_tide(i,j) = ua_tide(i,j) + uatamp(i,j,k)*cos(freq_east(zi,k)*timesec - uatpha(i,j,k))
                      va_tide(i,j) = va_tide(i,j) + vatamp(i,j,k)*cos(freq_east(zi,k)*timesec - vatpha(i,j,k))
                  end do
              end do
          end do
      case default
          call abort("tidal forcing type not implemented")
      end select      
  end do
  do zi=1,num_zones_south
      if(.not. bosouth(zi)%inblock) cycle
      r = bosouth(zi)%local_range
      
      select case(tidetype_south(zi))
      case(0,1)
          ic=(r(1)+r(2))/2
          jc=r(3)
          do k=1, kcon_south(zi)
              eta_tide(ic,jc) = eta_tide(ic,jc) + etatamp(ic,jc,k)*cos(freq_south(zi,k)*timesec - etatpha(ic,jc,k))
              ua_tide(ic,jc) = ua_tide(ic,jc) + uatamp(ic,jc,k)*cos(freq_south(zi,k)*timesec - uatpha(ic,jc,k))
              va_tide(ic,jc) = va_tide(ic,jc) + vatamp(ic,jc,k)*cos(freq_south(zi,k)*timesec - vatpha(ic,jc,k))
          end do
          do j=r(3),r(4)
              do i=r(1),r(2)
                  eta_tide(i,j) = fsm(i,j)*eta_tide(ic,jc)
                  ua_tide(i,j) = fsm(i,j)*ua_tide(ic,jc)
                  va_tide(i,j) = fsm(i,j)*va_tide(ic,jc)
              end do
          end do
      case(2)
! compute in all points
          do k=1, kcon_south(zi)
              do j=r(3),r(4)
                  do i=r(1),r(2)
                      eta_tide(i,j) = eta_tide(i,j) + etatamp(i,j,k)*cos(freq_south(zi,k)*timesec - etatpha(i,j,k))
                      ua_tide(i,j) = ua_tide(i,j) + uatamp(i,j,k)*cos(freq_south(zi,k)*timesec - uatpha(i,j,k))
                      va_tide(i,j) = va_tide(i,j) + vatamp(i,j,k)*cos(freq_south(zi,k)*timesec - vatpha(i,j,k))
                  end do
              end do
          end do
      case default
          call abort("tidal forcing type not implemented")
      end select      
  end do
  do zi=1,num_zones_north
      if(.not. bonorth(zi)%inblock) cycle      
      r = bonorth(zi)%local_range
      
      select case(tidetype_north(zi))
      case(0,1)
          ic=(r(1)+r(2))/2
          jc=r(4)
          do k=1, kcon_north(zi)
              eta_tide(ic,jc) = eta_tide(ic,jc) + etatamp(ic,jc,k)*cos(freq_north(zi,k)*timesec - etatpha(ic,jc,k))
              ua_tide(ic,jc) = ua_tide(ic,jc) + uatamp(ic,jc,k)*cos(freq_north(zi,k)*timesec - uatpha(ic,jc,k))
              va_tide(ic,jc) = va_tide(ic,jc) + vatamp(ic,jc,k)*cos(freq_north(zi,k)*timesec - vatpha(ic,jc,k))
          end do
          do j=r(3),r(4)
              do i=r(1),r(2)
                  eta_tide(i,j) = fsm(i,j)*eta_tide(ic,jc)
                  ua_tide(i,j) = fsm(i,j)*ua_tide(ic,jc)
                  va_tide(i,j) = fsm(i,j)*va_tide(ic,jc)
              end do
          end do
      case(2)
! compute in all points
          do k=1, kcon_north(zi)
              do j=r(3),r(4)
                  do i=r(1),r(2)
                      eta_tide(i,j) = eta_tide(i,j) + etatamp(i,j,k)*cos(freq_north(zi,k)*timesec - etatpha(i,j,k))
                      ua_tide(i,j) = ua_tide(i,j) + uatamp(i,j,k)*cos(freq_north(zi,k)*timesec - uatpha(i,j,k))
                      va_tide(i,j) = va_tide(i,j) + vatamp(i,j,k)*cos(freq_north(zi,k)*timesec - vatpha(i,j,k))
                  end do
              end do
          end do
      case default
          call abort("tidal forcing type not implemented")
      end select      
  end do

  call dump_state("tidefrs")
END SUBROUTINE TIDEFRS
