SUBROUTINE UPSTREAMQ(QFIELD)
!
!           UPSTREAMQ integrates the equation
!
!           d(QFIELD*D)/d(T) + d(QFIELD*U*D)/d(X) + d(QFIELD*V*D)/d(Y) +
!       
!           d(QFIELD*W)/d(SIGMA) = 0
!
!           from timestep n to timestep n+1
!           with the upstream method
!
!           d(*) is the differential operator,
!           QFIELD is Q2 or Q2L,
!           D is the dynamic depth,
!           T is time,
!           X and Y are the horisontal coordinates,
!           U is the horisontal velocity in X-direction,
!           V is the horisontal velocity in Y-direction,
!           SIGMA is the vertical coordinate,
!           W is the vertical velocity. 
!
!   QFIELD is updated for indices (2:IM-1,2:JM-1,2:KB-1)
!
!   Global variables.
!
USE STATE
IMPLICIT NONE
real, pointer :: qfield(:,:,:)
!
!   Local variables.
!
REAL, POINTER :: XFLUX(:,:,:), YFLUX(:,:,:)
REAL :: ZFLUX(I1:I2,J1:J2,KB)
REAL AREAT,VEL,AREATU,NETFLUX,VOLUME
INTEGER I,J,K

allocate(xflux(i1:i2,j1:j2,kb), yflux(i1:i2,j1:j2,kb))

call tag_ghost(qfield, west+south, 1)
call update_ghost

!
!   First advection in X-direction.
!
XFLUX = 0.
DO J=1,J_EQ_JM
  DO I=I_EQ_2,I_EQ_IM
    IF (DUM(I,J).NE.0.) THEN
      DO K = 2,KB-1
        AREAT = DY*DU(I,J)*DZZ(K-1)
        VEL = 0.5*(UADV(I,J,K)+UADV(I,J,K-1))
        AREATU = AREAT*VEL
        IF (VEL.GE.0.) THEN
          XFLUX(I,J,K) = AREATU*QFIELD(I-1,J,K)
        ELSE
          XFLUX(I,J,K) = AREATU*QFIELD(I,J,K)
        ENDIF
      END DO
    ENDIF
  END DO
END DO
!
YFLUX = 0.
DO J=J_EQ_2,J_EQ_JM
  DO I=1,I_EQ_IM
    IF (DVM(I,J).NE.0.) THEN
      DO K = 2,KB-1
        AREAT = DX*DV(I,J)*DZZ(K-1)
        VEL = 0.5*(VADV(I,J,K)+VADV(I,J,K-1))
        AREATU = AREAT*VEL
        IF (VEL.GE.0..AND.JM>1) THEN
          YFLUX(I,J,K) = AREATU*QFIELD(I,J-1,K)
        ELSE
          YFLUX(I,J,K) = AREATU*QFIELD(I,J,K)
        ENDIF
      END DO
    ENDIF
  END DO
END DO
!
ZFLUX = 0.
!
AREAT = DX*DY
DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    IF(FSM(I,J).NE.0.) THEN
      DO K = 2,KB
        VEL = 0.5*(W(I,J,K)+W(I,J,K-1))
        AREATU = AREAT*VEL
        IF (VEL.GE.0.) THEN
          ZFLUX(I,J,K) = AREATU*QFIELD(I,J,K)
        ELSE
          ZFLUX(I,J,K) = AREATU*QFIELD(I,J,K-1)
        ENDIF
      END DO
    ENDIF
  END DO
END DO
call tag_ghost(xflux, east, 1)
call tag_ghost(yflux, north, 1)
call update_ghost

!
!   Take a step to timestep n+1.
!
DO J=J_EQ_2,JM_MI_1
  DO I=I_EQ_2,IM_MI_1
    IF (FSM(I,J).GT.0.0) THEN
      DO K = 2,KB-1
        NETFLUX = (ZFLUX(I,J,K)-ZFLUX(I,J,K+1)) + &
   &            XFLUX(I+1,J,K) - XFLUX(I,J,K)  
        IF(JM>1) NETFLUX = NETFLUX + YFLUX(I,J+1,K) - YFLUX(I,J,K)
        VOLUME = 0.5*DX*DY*(DDZ(I,J,K)+DDZ(I,J,K-1))
        QFIELD(I,J,K) = QFIELD(I,J,K) - DT*NETFLUX/VOLUME
      END DO
    ENDIF
  END DO
END DO

CALL DUMP_STATE("upstreamq")
deallocate(xflux, yflux)

END
