module util
!
! FUNCTION LOWER(STRING)
!   returns a string of the same length as "string" but in lower case  
! FUNCTION UPPER(STRING)
!   returns a string of the same length as "string" but in upper case  
! SUBROUTINE SPLIT(STRING, SPLITSTRING)
!   split "string" into substrings and place these in the array of
!   strings called "splitstring"
! SUBROUTINE JOIN(SPLITSTRING, STRING [, DELIMITER])
!   given an array of strings "splitstring", and optionally a delimiter,
!   join the strings (with delimiter inbetween) and return in "string"
! SUBROUTINE REPLACE(STRING, OLDCHARS, NEWCHAR)
!   in the given STRING, replace all occurences of the characters in OLDCHARS with NEWCHAR.  
! SUBROUTINE GETARGS([ARG, VAL, IVAL, N, MESSAGE])
!   read command line arguments into an array of strings,reals or integers.  
! SUBROUTINE GET_SPLITLINES(FILE, SPLITLINES [, SRCLINE, NUMLINES])
!   reads a file into a matrix of strings, SPLITLINES, where first indice is line, second column.  
! SUBROUTINE GET_LINES(FILE, KEYWORDS, LINES, SRCLINE)  
!   from FILE, read all lines that are not blank or begin with "!"
!   into LINES. the number of lines fetched is KEYWORDS, the actual
!   line number of each fetched line is stored in SRCLINE.
! SUBROUTINE BINDUMP(LABEL,F [,ACCESS])
!   dumps the 2d or 3d array F to a binary file with name constructed from label and array dimensions.
! SUBROUTINE BINLOAD(LABEL,F [,ACCESS])
!   dumps the 2d or 3d array F to a binary file with name constructed from label and array dimensions.
! CHARACTER(100) UNFNAME(F,LABEL)
!   return file name constructed from properties of F and LABEL.
! SUBROUTINE CHECK_FILE(FILE)
!   stop if the file with name FILE is not present and readable
! INTEGER FUNCTION BINSEARCH(ZLEVEL, ZPOS [, FIRST, LAST])
!   return BINSEARCH=lo so that ZPOS(lo-1) < ZLEVEL < ZPOS(lo), optionally search in ZPOS(FIRST:LAST)
! INTEGER FUNCTION BINSEARCH2(ZLEVEL, ZPOS [, FIRST, LAST])
!   return BINSEARCH2=lo so that ZPOS(lo) < ZLEVEL < ZPOS(lo-1), optionally search in ZPOS(FIRST:LAST)
 
  private
  public lower, upper, split, join, bindump, getargs, get_splitlines, replace, unfname, binload, ensureendslash, &
       get_lines, check_file, binsearch, binsearch2
  
  interface bindump
! subroutine bindump(label,f[,access])     
!   dump f to an unformatted("binary") file, which name is constructed
!   from "label" and the array dimensions of f. if access is "stream"     
!   the binary format is compatible with C stream files. if access is
!   not provided a processor dependent fortran unformatted is dumped.     
     module procedure dump1d, dump2d, dump3d, dump1di
  end interface

  interface binload
! subroutine binload(label,f[,access])     
!   load f from an unformatted("binary") file, which name is constructed
!   from "label" and the array dimensions of f. if access is "stream"     
!   the binary format is compatible with C stream files, if not
!   provided a default fortran unformatted binary (processor dependent format)
!   is assumed.
     module procedure load1d, load2d, load3d, load1di, load2di
  end interface binload
  
  interface unfname
! character(100) unfname(f,label)
!   construct a file name from kind of f, its shape and the provided label. 
!   example real(kind=8) f(33,45) and label "f", should yield "f-33x45.sr8"
!   s indicates stream file, r real and the number the kind.     
     module procedure unfname1d,unfname2d,unfname3d,unfiname1d,unfiname2d
  end interface unfname

  interface binsearch
     module procedure bsearch2r4, bsearch2r8
  end interface binsearch
  
  character :: lower_table(256)=(/" "," "," "," "," ",&
       " "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," ",&
       " "," "," "," "," ","!",'"',"#","$","%","&","'","(",")","*","+",",","-",".","/","0","1",&
       "2","3","4","5","6","7","8","9",":",";","<","=",">","?","@","a","b","c","d","e","f","g",&
       "h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z","["," ","]",&
       "^","_","`","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s",&
       "t","u","v","w","x","y","z","{","|","}","~"," "," "," "," "," "," "," "," "," "," "," ",&
       " "," "," "," "," "," "," "," ","'"," "," "," "," "," "," "," "," "," "," ","x","x","x",&
       "x","x","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","",""," " /)
  
  character :: upper_table(256)=(/" "," "," "," "," "," ",&
       " "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," ",&
       " "," "," "," ","!",'"',"#","$","%","&","'","(",")","*","+",",","-",".","/","0","1","2",&
       "3","4","5","6","7","8","9",":",";","<","=",">","?","@","A","B","C","D","E","F","G","H",&
       "I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","["," ","]","^",&
       "_","`","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T",&
       "U","V","W","X","Y","Z","{","|","}","~"," "," "," "," "," "," "," "," "," "," "," "," ",&
       " "," "," "," "," "," "," ","'"," "," "," "," "," "," "," "," "," "," "," "," "," "," ",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","","","","","","","","","","","",&
       "","","",""," "/)

  
contains

  
  function lower(string)
    implicit none
    character(*) string
    character(len(string)) lower
    integer i,j
    do i=1,len(string)
        j=ichar(string(i:i))
        lower(i:i) = lower_table(j)
    end do
  end function lower

  function upper(string)
    implicit none
    character(*) string
    character(len(string)) upper
    integer i,j
    do i=1,len(string)
        j=ichar(string(i:i))
        upper(i:i) = upper_table(j)
    end do
  end function upper
  
  subroutine split(string, splitstring, items)
    implicit none
    character(*) string
    character(*) splitstring(:)
    integer, optional :: items
    integer i

    splitstring(:) = " "    
    read(string,*,end=101) (splitstring(i), i=1,size(splitstring))
101 continue

    if(present(items)) then
! count the number of words on the line
        items=0
        do i=1,size(splitstring)
            if(trim(splitstring(i)) == "") exit
            items = items + 1
        end do
    end if
  end subroutine split

  subroutine join(splitstring, string, delimiter)
    implicit none
    character(*) string
    character(*) splitstring(:)
    character, optional :: delimiter
    character delim
    integer pos, sz, i
    
    delim=" "
    string=" "
    if(present(delimiter)) delim=delimiter
    
    pos=1
    do i=1,size(splitstring)
        sz = len(trim(splitstring(i)))
        string(pos:pos+sz) = trim(splitstring(i)) // delim
        pos = pos + sz +1
    end do
  end subroutine join
  
  subroutine replace(string, oldchars, newchar)
! replace all occurences of the character oldchar with newchar in the
! given string.
    implicit none
    character(*) string, oldchars
    character newchar
    integer i,j

    do i=1,len(string)
        do j=1,len(oldchars)
            if(string(i:i) == oldchars(j:j)) string(i:i) = newchar
        end do
    end do
  end subroutine replace

  integer function bsearch2r4(zlevel, zpos, first, last)
    implicit none
    real(4), dimension(:), intent(in) :: zpos
    real(4), intent(in) :: zlevel
    integer, optional :: first, last
    integer lo,hi,mid,n1,n2

    n1 = 1
    n2 = size(zpos)
    if(present(first)) n1 = first
    if(present(last)) n2 = last
    
    if(zlevel >= zpos(n1) .and. zlevel < zpos(n2)) then
        lo=n1;  hi=n2
        do
            if(lo >= hi) exit
            mid = (lo+hi)/2
            if(zlevel < zpos(mid)) then
                hi = mid
            else
                lo = mid+1
            end if
        end do
        bsearch2r4 = lo
    else if(zlevel >= zpos(n2)) then
        bsearch2r4 = n2+1
    else
        bsearch2r4 = n1-1
    end if
  end function bsearch2r4
  
  integer function bsearch2r8(zlevel, zpos, first, last)
    implicit none
    real(8), dimension(:), intent(in) :: zpos
    real(8), intent(in) :: zlevel
    integer, optional :: first, last
    integer lo,hi,mid,n1,n2

    n1 = 1
    n2 = size(zpos)
    if(present(first)) n1 = first
    if(present(last)) n2 = last
    
    if(zlevel >= zpos(n1) .and. zlevel < zpos(n2)) then
        lo=n1;  hi=n2
        do
            if(lo >= hi) exit
            mid = (lo+hi)/2
            if(zlevel < zpos(mid)) then
                hi = mid
            else
                lo = mid+1
            end if
        end do
        bsearch2r8 = lo
    else if(zlevel >= zpos(n2)) then
        bsearch2r8 = n2+1
    else
        bsearch2r8 = n1-1
    end if
  end function bsearch2r8
  

  integer function binsearch2(zlevel, zpos, first, last)
!        
! return BINSEARCH=lo so that ZPOS(lo) < ZLEVEL < ZPOS(lo-1).
!
! similar to BINSEARCH, but ZPOS(i) decrease with increasing index i.
! optionally search only in ZPOS(FIRST:LAST), by default FIRST=1, LAST=size(ZPOS).
! if ZLEVEL > ZPOS(FIRST) return FIRST-1.
! if ZLEVEL < ZPOS(LAST) return LAST+1. 
!    
    implicit none
    real, intent(in) :: zlevel,zpos(:)
    integer, optional :: first,last
    integer lo,hi,mid,n1,n2

    n1 = 1
    n2 = size(zpos)
    if(present(first)) n1 = first
    if(present(last)) n2 = last
    
    if(zlevel >= zpos(n2) .and. zlevel < zpos(n1)) then
        lo=n2 ;  hi=n1  ! index for lowest and highest initial level
        do
            if(lo <= hi) exit
            mid = (lo+hi)/2
            if(zlevel > zpos(mid)) then
                lo = mid
            else
                hi = mid+1
            end if
        end do
        binsearch2 = lo
        return
    else if(zlevel < zpos(n2) ) then
        binsearch2 = n2 + 1
        return
    else
        binsearch2 = n1 - 1
        return
    end if
  end function

  
  subroutine getargs(arg, val, ival, n, message)
!    
! Get all, or as many as fits, command line arguments into the array of strings ARG.
! If all arguments are real numbers, provide the real array VAL instead. 
! Integer only? IVAL is your friend.
! If N is present, the routine will return an error if N command line
! arguments is not provided.
! If the above error occur and MESSAGE is present, the error message will be .... MESSAGE
!    
    implicit none
    character(*), optional :: arg(:), message
    character(30) string
    real, optional :: val(:)
    integer, optional :: ival(:)
    integer, optional :: n
    integer i, err

    if(present(n)) then
        if(command_argument_count() .ne. n) then
            if(present(message)) then
                write(*,*) "UTIL GETARGS ERROR:", message
                stop
            else
                write(*,*) "UTIL GETARGS ERROR: expected ",n," command line arguments, got",command_argument_count()
                stop
            end if
        end if
    end if
    
    if(present(arg)) then
        do i=1, min(size(arg,1), command_argument_count())        
            call get_command_argument(i, arg(i), status=err)
            if(err == -1) then
                write(*,*) "command argument ",i,&
                     "was too long to fit into provided character array ARR, which has length=",len(arg(1))
                stop
            end if
            if(present(val))then
                if(size(val,1) >= i) read(arg(i),*) val(i)
            end if
        end do
    else if(present(val))then
        do i=1, command_argument_count()
            call get_command_argument(i, string, status=err)
            if(err == -1) then
                write(*,*) "command argument ",i,&
                     "was too long to fit into character array STRING, which has length=",len(string)
                stop
            end if
            if(size(val,1) >= i) read(string,*) val(i)
        end do        
    else if(present(ival))then
        do i=1, command_argument_count()
            call get_command_argument(i, string, status=err)
            if(err == -1) then
                write(*,*) "command argument ",i,&
                     "was too long to fit into character array STRING, which has length=",len(string)
                stop
            end if
            if(size(ival,1) >= i) read(string,*) ival(i)
        end do
    else
        write(*,*) "UTIL GETARGS WARNING: routine was called with no actual argument, does nothing"
        stop
    end if
  end subroutine getargs


  subroutine check_file(file)
! check if file is present and readable, stop if not.
    implicit none
    character(*) file
    logical is_here
    character(7) read_status

    inquire(file=file, exist=is_here, read=read_status)

    if(.not.is_here) then
        write(*,*) 'ERROR util.f90 check_file: file "'//trim(file)//'" does not exist'
        stop
    end if

!    if(read_status == "YES") return
! pgf90 wants this
    if(read_status == "YES" .or. read_status == "UNKNOWN") return

    write(*,*) 'ERROR util.f90 check_file: file"'//file//'" is not readable' ; stop
  end subroutine check_file

  
  subroutine get_lines(file, keywords, lines, srcline)
!
! from "file", get all lines that are not blank or begin with "!" into
! "lines". the number of lines fetched is "keywords", the actual line
! number of each fetched line is stored in "srcline".
!
    implicit none
    character(*) file
    integer keywords
    character(*) lines(:)
    character(300) :: line
    integer srcline(:)
    integer n

    call check_file(file)

    open(1000002,file=file, status='old', action='read', err=103)

    n=0
    keywords=0
    do
        n = n + 1
        line = ' '
        read(1000002,'(a)',end=101,err=102) line

        line = adjustl(line)        ! trim blanks at beginning of line

        if(trim(line) == '') cycle  ! skip blank lines
        if(line(1:1) == '!') cycle  ! skip comments

        keywords = keywords + 1
        
        if(keywords > size(lines)) then
            write(*,*) 'too many lines with data in datafile="' // trim(file) // '" max lines allowed=', &
                 size(lines)
        end if

        lines(keywords) = line
        srcline(keywords) = n
    end do
    
102 write(*,*)'error reading file=',file,' line=',n ; stop
    
103 write(*,*)'module util, get_lines: unable to open file=', file ; stop
    
101 close(1000002)
  end subroutine get_lines

  
  subroutine get_splitlines(file, splitlines, srcline, numlines)
!
! from FILE, get all lines that are not blank or begin with "!" into
! the matrix of strings SPLITLINES (first index is line number, second
! is column). the number of lines fetched is optionally returned in
! NUMLINES, the actual line number of each fetched line is optionally
! stored in SRCLINE.
!
    implicit none
    character(*) file
    character(*) splitlines(:,:)
    integer, optional :: srcline(:)
    integer, optional :: numlines   
    character(200) :: line
    integer n, m
    
    open(1000002, file=file, status='old', action='read', err=103)
    n=0
    m=0
    do
        n = n + 1
        read(1000002,'(a)',end=101,err=102) line
        line = adjustl(line)        ! trim blanks at beginning of line
        if(trim(line) == '') cycle  ! skip blank lines
        if(line(1:1) == '!') cycle  ! skip comments

        m = m + 1
        if(m > size(splitlines,1)) then
            write(*,*) "datafile=", trim(file)
            write(*,*) "max allowed lines=", size(splitlines,1)
            write(*,*) "too many lines with data in this datafile"
            return
        end if
        call split(line, splitlines(m,:))
        if(present(srcline)) srcline(m) = n
        if(present(numlines)) numlines = m
    end do
102 write(*,*)'module util; get_splitlines: error reading file=',file,' line=',n ; stop
103 write(*,*)'module util; get_splitlines: unable to open file=', file ; stop
101 close(1000002)
  end subroutine get_splitlines

  
  subroutine ensureendslash(string)
! make sure string ends with a slash
    character(*) string
    integer last
    last = len(trim(string))    
    if(string(last:last) .ne. "/") then
        if(last < len(string)) then
            string = trim(string)//"/"
        else
            write(*,*) "util, ensureendslash error: string not long enough to hold extra /"
            stop
        end if
    end if
  end subroutine ensureendslash
  
  
  character(100) function unfname1d(f, label, access)
    implicit none
    real f(:)
    character(*) label
    character(2) rkind,atype
    character(6) dim1
    character(*), optional :: access

    atype=".u"
    if(present(access)) then
        if(access=="stream") atype=".s"
    end if    
    write(rkind,'("r",i0)') kind(f)
    write(dim1,'(i0)') size(f,1)
    unfname1d = trim(label) // "-" // trim(dim1) // ".s" // trim(rkind)
  end function unfname1d

  character(100) function unfiname1d(f, label, access)
    implicit none
    integer f(:)
    character(*) label
    character(2) rkind,atype
    character(6) dim1
    character(*), optional :: access

    atype=".u"
    if(present(access)) then
        if(access=="stream") atype=".s"
    end if       
    write(rkind,'("i",i0)') kind(f)
    write(dim1,'(i0)') size(f,1)
    unfiname1d = trim(label) // "-" // trim(dim1) // ".s" // trim(rkind)
  end function unfiname1d
  
  character(100) function unfname2d(f, label, access)
    implicit none
    real f(:,:)
    character(*) label
    character(2) rkind,atype
    character(6) dim1,dim2
    character(*), optional :: access

    atype=".u"
    if(present(access)) then
        if(access=="stream") atype=".s"
    end if        
    write(rkind,'("r",i0)') kind(f)
    write(dim1,'(i0)') size(f,1)      
    write(dim2,'(i0)') size(f,2)      
    unfname2d = trim(label) // "-" // trim(dim1) // "x" // trim(dim2) // ".s" // trim(rkind)
  end function unfname2d

  character(100) function unfiname2d(f, label, access)
    implicit none
    integer f(:,:)
    character(*) label
    character(2) rkind,atype
    character(6) dim1,dim2
    character(*), optional :: access

    atype=".u"
    if(present(access)) then
        if(access=="stream") atype=".s"
    end if        
    write(rkind,'("i",i0)') kind(f)
    write(dim1,'(i0)') size(f,1)      
    write(dim2,'(i0)') size(f,2)      
    unfiname2d = trim(label) // "-" // trim(dim1) // "x" // trim(dim2) // ".s" // trim(rkind)
  end function unfiname2d
  
  character(100) function unfname3d(f, label, access)
    implicit none
    real f(:,:,:)
    character(*) label
    character(2) rkind,atype
    character(6) dim1,dim2,dim3
    character(*), optional :: access

    atype=".u"
    if(present(access)) then
        if(access=="stream") atype=".s"
    end if        
    write(rkind,'("r",i0)') kind(f)
    write(dim1,'(i0)') size(f,1)      
    write(dim2,'(i0)') size(f,2)      
    write(dim3,'(i0)') size(f,3)      
    unfname3d = trim(label) // "-" // trim(dim1) // "x" // trim(dim2) // "x" // trim(dim3) // ".s" // trim(rkind)
  end function unfname3d

  
  subroutine dump1d(label,f,access)
    character(*) label
    real f(:)
    character(*), optional :: access

    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    write(1000014) f
    close(1000014)
  end subroutine dump1d

  subroutine dump1di(label,f,access)
    character(*) label
    integer f(:)
    character(*), optional :: access

    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    write(1000014) f
    close(1000014)
  end subroutine dump1di
  
  subroutine dump2d(label,f,access)
    character(*) label
    real f(:,:)
    character(*), optional :: access
    
    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if

    write(1000014) f
    close(1000014)
  end subroutine dump2d
  
  subroutine dump3d(label,f,access)
    character(*) label
    real f(:,:,:)
    character(*), optional :: access
    
    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    write(1000014) f
    close(1000014)
  end subroutine dump3d
  
  subroutine load1d(label,f,access)
    character(*) label
    real f(:)
    character(*), optional :: access
    
    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    read(1000014) f
    close(1000014)
  end subroutine load1d
  
  subroutine load2d(label,f,access)
    character(*) label
    real f(:,:)
    character(*), optional :: access
    
    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    read(1000014) f
    close(1000014)
  end subroutine load2d
  
  subroutine load3d(label,f,access)
    character(*) label
    real f(:,:,:)
    character(*), optional :: access
    
    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    read(1000014) f
    close(1000014)
  end subroutine load3d
  
  subroutine load1di(label,f,access)
    character(*) label
    integer f(:)
    character(*), optional :: access
    
    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    read(1000014) f
    close(1000014)
  end subroutine load1di
  
  subroutine load2di(label,f,access)
    character(*) label
    integer f(:,:)
    character(*), optional :: access
    
    if(present(access))then
        open(1000014,file=unfname(f,label,access), form="unformatted", access=access)
    else
        open(1000014,file=unfname(f,label), form="unformatted")
    end if
    read(1000014) f
    close(1000014)
  end subroutine load2di
  
end module util


