 SUBROUTINE VERTVISCW
!
!  VERTVISCW propagates the solution of the equations
!
!  d(WD)/dt = d((KM /D)*d(W)/d(sigma))/d(sigma)
!
!  with boundary conditions:
!
!  W = 0  at sigma=0
!  W = 0 at sigma=-1
!
!  The solutions are propagated with an fully implicit method.
!
USE STATE   
USE MOD_BOUND
IMPLICIT NONE
!
! Local variables.
!
INTEGER I,J,K,IW
REAL DSQ
REAL, DIMENSION(kb-2) :: RHS,A,B,C,G, alf, bet
INTEGER GI,GJ
!
!
DO J = j_eq_2,JM_mi_1
 DO I = i_eq_2,IM_mi_1
   IF(FSM(I,J).NE.0.) THEN
      DSQ = D(I,J)*D(I,J)
!
! Define the right hand side of the solution matrix.
!
      DO K = 1,KB-2
        RHS(K) = W(I,J,K+1)
      END DO
!
! Define the upper diagonal elements of the tridiagonal solution matrix.
!
      DO K = 1,KB-2
        B(K) = -DT*KM(I,J,K+1)/(DSQ*DZ(K+1)*DZZ(K))
      END DO
!
! Define the lower diagonal elements.
!
      DO K = 1,KB-2
        C(K) = -DT*KM(I,J,K+1)/(DSQ*DZ(K)*DZZ(K))
      END DO
!
!  If upwell W at bottom not equal to 0.
!
      IF (UPWELL)THEN
          DO IW = 1,NUMWELL
              GI=BLOCK%FI+I-1 ; GJ=BLOCK%FJ+J-1
              IF (GI.EQ.IWELL(I) .AND. GJ.EQ.JWELL(I)) THEN
                  RHS(KB-2) = RHS(KB-2) + DT*KM(I,J,KB-1)/(DSQ*DZ(KB-1)*DZZ(KB-2))
              ENDIF
          END DO
      ENDIF
! 
!
!     A(1) = 1. - B(1)
      A(1) = 1. - B(1) - C(1)
      DO K = 2,KB-3
        A(K) = 1. - B(K) - C(K)
      END DO
!     A(KB-2) = 1. - C(KB-2)
      A(KB-2) = 1. - C(KB-2) - B(KB-2)
!
! Solve the system of equations.
!
!
!   LU-decompose the linear system.
!
      ALF(1) = A(1)
      DO K = 2,KB-2
        ALF(K) = A(K) - C(K)*B(K-1)/ALF(K-1)
      END DO
      DO K = 2,KB-2
        BET(K) = C(K)/ALF(K-1)
      END DO
!
!   Forward substitute the right hand side.
!
      G(1) = RHS(1)
      DO K = 2,KB-2
        G(K) = RHS(K) - BET(K)*G(K-1)
      END DO
!
!   Back substitute to compute the solution F.
!
      W(I,J,KB-1) = G(KB-2)/ALF(KB-2)
      DO K = KB-3,1,-1
        W(I,J,K+1) = (G(K) - B(K)*W(I,J,K+2))/ALF(K)
      END DO
    ENDIF
  END DO
END DO

CALL DUMP_STATE('vertviscw')
END SUBROUTINE VERTVISCW
