subroutine wall_drag_w()
!
! Apply the effect of vertical wall stress to W. As a first attempt we 
! compute the wall stress in "real" coordinates, transform the correction.
! The change in velocity due to drag;  Force = mass*dU/dt => dU = dt*Force/mass,
! where Force = Stress*Area.
!  
  use state
  implicit none
  real dxr,dyt,dti,area,volume,speed, sigmacorr
  real dragcorr(i1:i2,j1:j2,kb), zlev(i1:i2,j1:j2)
  integer dj,i,j,k
  
  dj=1
  if(jm == 1) dj=0 
  dxr = 1./dx
  dyt = 1./dy
  dti = 1./dt
  
  do k = 1,kb-1
      do j=j1,j2
          do i=i1,i2
              zlev(i,j) = zz(k)*d(i,j) + eta(i,j)
          end do
      end do
              
      do j = 1,j_eq_jm
          do i = 1,i_eq_im
              if(coast(i,j) > 0.) then

! first compute the correction in WR points
                  
                  area = coast(i,j)*dz(k)*d(i,j)
                  volume = dx*dy*dz(k)*d(i,j)
                  speed = abs(wr(i,j,k))
                  dragcorr(i,j,k) = dt/(volume*rho0) * cdw*area*wr(i,j,k)*speed

! sigma transform correction by computing W = WR - sigmacorr
! (see equation 37 in the 4.1 user guide)
                  
                  sigmacorr =  &
! we first assume flat bottom and gradient of ETA=0 
!                  0.5* ( u(i+1,j,k)  * (zlev(i+1,j)  - zlev(i,j)   )*dxr     &
!                       + u(i,j,k)    * (zlev(i,j)    - zlev(i-1,j) )*dxr     &                      
!                       + v(i,j+dj,k) * (zlev(i,j+dj) - zlev(i,j)   )*dyt     &
!                       + v(i,j,k)    * (zlev(i,j)    - zlev(i,j-dj))*dyt )   &
                        (1. + zz(k))*(eta(i,j) - etap(i,j)) * dti
                  
                  dragcorr(i,j,k) = dragcorr(i,j,k) - sigmacorr                  
              endif
          end do
      end do
  end do

  do k=2,kb-1
      do j = 1,j_eq_jm
          do i = 1,i_eq_im
              if(coast(i,j) > 0.) then
                  if(abs(dragcorr(i,j,k)) < abs(w(i,j,k))) then
                      w(i,j,k) = w(i,j,k) - 0.5*(dragcorr(i,j,k) + dragcorr(i,j,k-1))
                  else
                      w(i,j,k) = 0.0
                  end if
              end if
          end do
      end do
  end do
end subroutine wall_drag_w
