SUBROUTINE WIND1()
!
!  Calculate WUSURF, WVSURF and ATMPRESS for a simple wind, where the
!  strength varies sinusoidally with a given period, and the direction
!  rotates at a given frequency. The parameters to set in mod_bound are as follows:
!    wind1_direction              initial direction w.r. x axis in counterclockwise degrees
!    wind1_hourly_rotation        if.ne.0, wind direction changes by this number of degrees per hour. 
!    wind1_amplitude              the magnitude of the wind in m/s
!    wind1_period                 if>0, the wind strength varies sinusoidally with this period in hours.
!    wind1_rampup_time            if>0, the wind strength will ramp up linearly towards its max 
!                                 in this time in hours.
!  They all default to 0 (do nothing)  
!
  USE STATE
  USE MOD_BOUND
  USE TIME_UTIL
  IMPLICIT NONE
  REAL SPEED, DIRECTION, PI, TIMEHOUR, ROAD0, WS, FACT, CDRAG
  INTEGER I,J
  
  PI = 4.*ATAN(1.0)
  TIMEHOUR = TIME/3600.

  IF(WIND1_PERIOD > 0) THEN
      SPEED = WIND1_AMPLITUDE*SIN(TIMEHOUR*2*PI/WIND1_PERIOD)
  ELSE
      SPEED = WIND1_AMPLITUDE
  END IF
  
  IF(TIMEHOUR > 0.0 .AND. TIMEHOUR < WIND1_RAMPUP_TIME) THEN
      SPEED = SPEED * TIMEHOUR/WIND1_RAMPUP_TIME
  END IF

  DIRECTION = (WIND1_DIRECTION + WIND1_HOURLY_ROTATION*TIMEHOUR)*PI/180.
  
  WSPEEDU = SPEED*COS(DIRECTION)
  WSPEEDV = SPEED*SIN(DIRECTION)
!
!  Compute WUSURF and WVSURF from the wind speeds.
!
  ROAD0=ROAIR/RHO0

  DO I=1,I_eq_IM
      DO J=1,J_eq_JM
          WS=SPEED !SQRT(WSPEEDU(I,J)*WSPEEDU(I,J)+WSPEEDV(I,J)*WSPEEDV(I,J))
          FACT=1.2
          IF(WS.GE.11.) FACT=0.49 +0.065*WS
          CDRAG=1.0E-3*FACT
          WUSURF(I,J)=ROAD0*CDRAG*WS*WSPEEDU(I,J)
          WVSURF(I,J)=ROAD0*CDRAG*WS*WSPEEDV(I,J)
      END DO
  END DO
!
!  Define the atmospheric pressure
!
  PATM = PMEAN

  call dump_state("wind1")
END SUBROUTINE WIND1


