SUBROUTINE WIND2
!
!  WINDFORCING estimates 
!  the wind stresses WUSURF and WVSURF
!  from windspeeds 10m above the sea surface
!
!  PATM may also be redefined in WINDFORCING
!
  USE STATE
  USE MOD_BOUND
  USE TIME_UTIL
  IMPLICIT NONE
  
  REAL ROAD0,WS,FACT,CDRAG
  REAL SECONDS 
  REAL P0,PC,TS
  REAL STARTX,STARTY,SX,SY,USPEED,VSPEED,SPEED,PI
  REAL DISTX,DISTY,DIST,RADIUS,SIGMA
  REAL, PARAMETER :: F = 1.3E-4
  REAL WINDMAX, ANGLE
  INTEGER I,J,GI,GJ

  SIGMA = 0.7
  SECONDS = time 

! get the init data from mod_bound

  P0 =     wind2_strength
  STARTX = wind2_startx
  STARTY = wind2_starty
  SPEED =  wind2_center_speed
  RADIUS = wind2_radius
  TS =     wind2_rampup_time
  ANGLE =  wind2_center_direction
  
  PI = 4.*ATAN(1.0)
  USPEED = SPEED*COS(ANGLE*PI/180.)
  VSPEED = SPEED*SIN(ANGLE*PI/180.)
  SX = STARTX + SECONDS*USPEED
  SY = STARTY + SECONDS*VSPEED
!
!   Compute PC as a function of SECONDS.
!
  IF (SECONDS.LT.0.) THEN
      PC = 0.
  ELSEIF (SECONDS.GE.0. .AND. SECONDS.LT.TS) THEN
      PC = P0*SECONDS/TS
  ELSE
      PC = P0
  ENDIF
!
!   Compute TAUX and TAUY at the center of all horizontal cells.
!
  WINDMAX = 0.

!
!   Martinsen, Gjevik,Roed

!
!  Compute WUSURF and WVSURF from the wind speeds.
!
  ROAD0=ROAIR/RHO0
  PATM = PMEAN
  WSPEEDU = 0.
  WSPEEDV = 0.
  WUSURF = 0.
  WVSURF = 0.
!
  DO I=1,I_eq_IM
      DO J=1,J_eq_JM
          GI = BLOCK%FI + I - 1
          GJ = BLOCK%FJ + J - 1
!
!   Compute distance from cyclone center.
!
          DISTX = DX*(GI-0.5) - SX
          DISTY = DY*(GJ-0.5) - SY
          DIST = DISTX**2 + DISTY**2
!
!   Compute the wind speed after Gjevik.
!
          PATM(I,J) = PMEAN + PC*EXP(-DIST/(RADIUS**2))
      END DO
  END DO
  
  DO I=I_eq_2,IM_mi_1
      DO J=J_eq_2,JM_mi_1
          IF(JM>1) WSPEEDU(I,J) = -0.5*(SIGMA/(F*ROAIR))*(PATM(I,J+1)-PATM(I,J-1))/DY
          WSPEEDV(I,J) = 0.5*(SIGMA/(F*ROAIR))*(PATM(I+1,J)-PATM(I-1,J))/DX
          WS = SQRT(WSPEEDU(I,J)**2 + WSPEEDV(I,J)**2)
          IF (WS.GT.WINDMAX) THEN
              WINDMAX = WS
          ENDIF

!   Compute the wind stress coefficient CD2.
!
          FACT = 1.2
          IF (WS.GE.11) THEN
              FACT=0.49 +0.065*WS
          ENDIF
          FACT = 3.0
          CDRAG=1.0E-3*FACT
!
!   Compute TAUX and TAUY.
!
!          TAU = ROAD0*CDRAG*WS**2
          WUSURF(I,J) = ROAD0*CDRAG*WS*WSPEEDU(I,J)
          WVSURF(I,J) = ROAD0*CDRAG*WS*WSPEEDV(I,J)
      END DO
  END DO
  call dump_state("wind2")
END SUBROUTINE WIND2

